#========================================================================================
# package      DE_EPAGES::CrossSelling::API::Object::ProductComparison
# state        public
# base         DE_EPAGES::Shop::API::Object::ShopObject
# description  object interface for ProductComparison.
#========================================================================================
package DE_EPAGES::CrossSelling::API::Object::ProductComparison;
use base DE_EPAGES::Shop::API::Object::ShopObject;

use strict;
use DE_EPAGES::Trigger::API::Trigger qw ( TriggerHook );

#========================================================================================
# function     className
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ClassName = $ProductComparison->className;
#----------------------------------------------------------------------------------------
# description  Returns the class name, used for DAL access.
#----------------------------------------------------------------------------------------
# return       $ClassName | class name | String
#========================================================================================
sub className  { 'ProductComparison' }

#========================================================================================
# function     dbPackage
# state        public
#----------------------------------------------------------------------------------------
# syntax       $DbPackage = $ProductComparison->dbPackage;
#----------------------------------------------------------------------------------------
# description  Returns the database package, used for DAL access.
#----------------------------------------------------------------------------------------
# return       $DbPackage | database package name | String
#========================================================================================
sub dbPackage { __PACKAGE__ }

#========================================================================================
# function     insert
# state        private
#----------------------------------------------------------------------------------------
# syntax       $ProductComparison = DE_EPAGES::CrossSelling::API::Object::ProductComparison->insert($hInfo);
# example      my $Class = LoadClassByAlias('ProductComparison');
#               my $ProductComparison = $Class->insertObject($hInfo);
#----------------------------------------------------------------------------------------
# description  Insert this info as object, afterwards the hook 'OBJ_InsertProductComparison' is triggered.
#----------------------------------------------------------------------------------------
# input        $hInfo | attributes of new object | ref.hash
# return       $ProductComparison | new object | object
# hook         OBJ_InsertProductComparison | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>ProductComparisonID - object id            - int
#               </ul>
#========================================================================================
sub insert {
    my $class = shift;
    my ($hInfo) = @_;

    if ($hInfo->{'Parent'}->parent->id != $hInfo->{'Product'}->id) {
        $hInfo->{'Product'} = $hInfo->{'Parent'}->parent;
    }

    my $self = $class->SUPER::insert($hInfo);

    TriggerHook('OBJ_InsertProductComparison', {'Object' => $self, 'ProductComparisonID' => $self->id });
    delete $hInfo->{$_} foreach qw ( Product TargetProduct ); ## no critic qw(ProhibitLongLines)
    return $self;
}

#========================================================================================
# function     delete
# state        private
#----------------------------------------------------------------------------------------
# syntax       $ProductComparison->delete;
#----------------------------------------------------------------------------------------
# description  Deletes this object, therefore the hook 'OBJ_DeleteProductComparison' is triggered.
#----------------------------------------------------------------------------------------
# hook         OBJ_DeleteProductComparison | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>ProductComparisonID - object id            - int
#               </ul>
#========================================================================================
sub delete {
    my $self = shift;

    TriggerHook('OBJ_DeleteProductComparison', {'Object' => $self, 'ProductComparisonID' => $self->id });
    $self->SUPER::delete;
    return;
}

#========================================================================================
# function     set
# state        private
#----------------------------------------------------------------------------------------
# syntax       $ProductComparison->set($hValues, $LanguageID);
#----------------------------------------------------------------------------------------
# description  This function triggers hooks 'OBJ_BeforeUpdateProductComparison' and 'OBJ_AfterUpdateProductComparison'.
#----------------------------------------------------------------------------------------
# input        $hValues | attribute name => value hash | ref.hash
# input        $LanguageID | (optional; required for localized attributes) language id | int
#----------------------------------------------------------------------------------------
# hook         OBJ_BeforeUpdateProductComparison | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>ProductComparisonID - object id            - int
#                 <li>Values        - new values for object - ref.hash
#                 <li>LanguageID    - values of this language will be updated - int
#               </ul>
# hook         OBJ_AfterUpdateProductComparison | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>ProductComparisonID - object id            - int
#                 <li>Values        - updated values       - ref.hash
#                 <li>LanguageID    - values updated of this language - int
#               </ul>
#========================================================================================
sub set {
    my $self = shift;
    my ($hValues, $LanguageID) = @_;

    TriggerHook('OBJ_BeforeUpdateProductComparison', {'Object'=>$self, 'Values'=> $hValues, 'ProductComparisonID' => $self->id, 'LanguageID' => $LanguageID});
    $self->SUPER::set($hValues, $LanguageID);
    TriggerHook('OBJ_AfterUpdateProductComparison',  {'Object'=>$self, 'Values'=> $hValues, 'ProductComparisonID' => $self->id, 'LanguageID' => $LanguageID});
    return;
}

#========================================================================================
# function     default
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hValues = DE_EPAGES::CrossSelling::API::Object::ProductComparison->default( $Parent, $hInfo );
# example      $hDefaults = $Class->defaultAttributes($Parent, $hInfo); # for insert a new object
#               $hDefaults = $Object->class->defaultAttributes($Object->parent, $hValues); # for an existing object
#----------------------------------------------------------------------------------------
# description  Returns default attribute values for new objects.
#               Used by <function DE_EPAGES::Object::Object::Class::defaultAttributes>
#----------------------------------------------------------------------------------------
# input        $Parent | parent object of the new object | object
# input        $hInfo | attribute name => value hash (passed to insert()) | ref.hash
# return       $hValues | hash with default attribute values (name => value) | ref.hash
#========================================================================================
sub default {
    my $class = shift;
    my ($Parent, $hInfo) = @_;
    return {
        %{$class->SUPER::default($Parent, $hInfo)},
        'Product' => $Parent->parentByClass('Product'),
    };
}

1;
