#========================================================================================
# package      DE_EPAGES::Database::API::DAL::SQL
# state        private
# description  database abstraction layer without caching.
#========================================================================================
package DE_EPAGES::Database::API::DAL::SQL;

use strict;
use DE_EPAGES::Core::API::Error qw ( Error );
use DE_EPAGES::Core::API::Warning qw ( Warning );
use DE_EPAGES::Core::API::PerlTools qw (GetLoopFromDoubleArray);
use DE_EPAGES::Database::API::Connection qw (GetCurrentDBHandle);
use DE_EPAGES::Database::API::Constants qw (MAX_IN_CLAUSE_SIZE);

sub ErrorDefinition {
    return {
        'INDEX_NOT_FOUND'   => 'index #Index not found at class #Class',
        'COLUMN_NOT_FOUND'  => 'column #Column not found at class #Class',
        'NOT_ALL_RESULTS'   => 'count of identifier:#Identifier, result:#Result at class #Class',
        'INVALID_DIRECTION' => 'invalid direction for ORDER BY: #Column #Direction',
    }
}

#========================================================================================
# function     new
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self = $Package->new($hOptions);
#----------------------------------------------------------------------------------------
# description  bless options. use <function load> to get a new dal.
#----------------------------------------------------------------------------------------
# input        $hOptions | data for object| ref.hash
#========================================================================================
sub new {
    my $class = shift;
    my ($hOptions) = @_;

    return bless $hOptions, $class;
}

#========================================================================================
# function     class
# state        private
#----------------------------------------------------------------------------------------
# syntax       $ClassName = $DAL->class;
#----------------------------------------------------------------------------------------
# description  returns the class name of the load dal.
#----------------------------------------------------------------------------------------
# return       $ClassName | class name (not the package name) | string
#========================================================================================
sub class { return shift->{'CLASS'}; }

#========================================================================================
# function     table
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hTableInfo = $DAL->table($ClassName);
# example      $hTableInfo = $DAL->table($DAL->class);
# example      $hTableInfo = $DAL->table('Object');
#----------------------------------------------------------------------------------------
# description  Returns table info.
#----------------------------------------------------------------------------------------
# input        $ClassName | class name (not the package name) | string
# return       $TableInfo | table info keys:
#               <ul>
#                   <li>IDENTIFIER - name of identifier</li>
#                   <li>COLUMN - hash of columns</li>
#                   <li>INDEX - hash of indexes</li>
#                   <li>Base - base class name</li>
#               </ul> | ref.hash.*
#========================================================================================
sub table {
    my $self = shift;
    my ($TableName) = @_;
    return $self->{'TABLE'}{$TableName};
}

#========================================================================================
# function     identifier
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Identifier = $DAL->identifier;
# example      $Identifier = $DAL->identifier; # $Identifier eq 'ObjectID'
#----------------------------------------------------------------------------------------
# description  Returns indentity column name of current DAL object.
#----------------------------------------------------------------------------------------
# return       $Identifier | column name | string
#========================================================================================
sub identifier {
    my $self = shift;
    return $self->{'IDENTIFIER'};
}

#========================================================================================
# function     classNames
# state        private
#----------------------------------------------------------------------------------------
# syntax       @ClassNames = $DAL->classNames;
#----------------------------------------------------------------------------------------
# description  Returns a list of class names (Name of table).
#----------------------------------------------------------------------------------------
# return       @ClassNames | table name | array.string
#========================================================================================
sub classNames {
    my $self = shift;
    my $Path = $self->{'CLASSNAMES'};
    return @$Path if defined $Path;

    my $ClassName = $self->{'CLASS'};
    my @Path = ($ClassName);
    while (defined $ClassName) {
        my $Table = $self->table($ClassName);
        if (defined $Table->{'Base'}) {
            $ClassName = $Table->{'Base'};
            push @Path, $ClassName;
        } else {
            $ClassName = undef;
        }
    }
    $self->{'CLASSNAMES'} = \@Path;
    return @Path;
}

#========================================================================================
# function     baseClassName
# state        private
#----------------------------------------------------------------------------------------
# syntax       $BaseClassName = $DAL->baseClassName;
#----------------------------------------------------------------------------------------
# description  Returns the class name, which has the first table (e.g. Object).
#----------------------------------------------------------------------------------------
# return       @BaseClassName | table name | string
#========================================================================================
sub baseClassName {
    my $self = shift;

    return ($self->classNames)[-1];
}

#========================================================================================
# function     column
# state        private
#----------------------------------------------------------------------------------------
# syntax       ($hColumnInfo, $hTableInfo) = $DAL->column($ColumnName);
# example      ($hColumnInfo, $hTableInfo) = $DAL->column($DAL->identifier);
#----------------------------------------------------------------------------------------
# description  Returns column if of given column name.
#               Throws the error COLUMN_NOT_FOUND if the column does not exist.
#----------------------------------------------------------------------------------------
# input        $ColumnName | column name | string
# return       $hColumnInfo | column info keys:
#               <ul>
#                   <li>Name - name - string</li>
#                   <li>Code - database name - string</li>
#                   <li>Type - type of column - string</li>
#                   <li>AttributeName - attribute name - string</li>
#                   <li>AttributeClass - attribute class name - string</li>
#               </ul> | ref.hash.*
# return       $hTableInfo | table info keys:
#               <ul>
#                   <li>Name - name - string</li>
#                   <li>Code - database name - string</li>
#                   <li>Short - short name - string</li>
#               </ul> | ref.hash.*
#========================================================================================
sub column {
    my $self = shift;
    my ($AttributeName) = @_;

    foreach my $Table ( $self->tables ) {
        if (exists $Table->{'COLUMN'}{$AttributeName}) {
            return ($Table->{'COLUMN'}{$AttributeName}, $Table);
        }
        foreach my $ColumnName (keys %{$Table->{'COLUMN'}}) {
            if (exists $Table->{'COLUMN'}{$ColumnName}{'AttributeName'} and
                $Table->{'COLUMN'}{$ColumnName}{'AttributeName'} eq $AttributeName) {
                return ($Table->{'COLUMN'}{$ColumnName}, $Table);
            }
        }
    }

    Error('COLUMN_NOT_FOUND', { Column=>$AttributeName, Class=>$self->class});
}

#========================================================================================
# function     tables
# state        private
#----------------------------------------------------------------------------------------
# syntax       @Tables = $DAL->tables;
#----------------------------------------------------------------------------------------
# description  Returns a list of all tables of the class.
#               Example for class Product : tables = qw(product contentobject object).
#----------------------------------------------------------------------------------------
# return       @Tables | list of database table names | array.string
#========================================================================================
sub tables {
    my $self = shift;

    return map { $self->table( $_ ) } $self->classNames;
}

#========================================================================================
# function     index
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hIndex = $DAL->index($IndexName);
#----------------------------------------------------------------------------------------
# description  Returns index if of given index name.
#----------------------------------------------------------------------------------------
# input        $IndexName | column name | string
#========================================================================================
sub index {
    my $self = shift;
    my ($IndexName) = @_;

    foreach my $Table ( $self->tables ) {
        return $Table->{'INDEX'}{$IndexName} if exists $Table->{'INDEX'}{$IndexName};
    }
    Error('INDEX_NOT_FOUND', { Index=>$IndexName, Class=>$self->class});
}

#========================================================================================
# function     _insertStatements
# state        private
#----------------------------------------------------------------------------------------
# syntax       $ahStatements = $DAL->_insertStatements;
#----------------------------------------------------------------------------------------
# description  Build insert statements.
#----------------------------------------------------------------------------------------
# return       $ahStatements | list of statements keys:
#               <ul>
#                   <li>SQL - statement includes placeholder '?' - string</li>
#                   <li>PARAMETER - parameter (column names) for placeholder - string</li>
#               </ul> | ref.array.hash
#========================================================================================
sub _insertStatements{
    my $self = shift;

    my @Statements;
    foreach my $Table ( $self->tables ) {
        my (@Columns, @Parameter);
        my $ExistsTextColumn = 0;
        my %Types;
        foreach my $Parameter (sort keys %{$Table->{'COLUMN'}}) {
            push @Columns, $Table->{'COLUMN'}{$Parameter}{'Code'};
            push @Parameter, $Parameter;
            $ExistsTextColumn = 1 if $Table->{'COLUMN'}{$Parameter}{'IsText'};
            $Types{$Parameter} = $Table->{'COLUMN'}{$Parameter}{'Type'};
        }
        my $Statement = sprintf 'INSERT INTO `%s` (%s) VALUES (%s)',
            $Table->{'Code'},
            join (', ', @Columns),
            join (',', map { '?' } @Parameter);

        unshift @Statements, {
            'SQL'        => $Statement,
            'PARAMETER'  => \@Parameter,
            'IDENTIFIER' => $Table->{'IDENTIFIER'},
            'EXISTS_TEXTCOLUMN' => $ExistsTextColumn,
            'TYPES' => \%Types,
        };
    }
    return \@Statements;
}

#========================================================================================
# function     insert
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Identifier = $DAL->insert($hParameter);
#----------------------------------------------------------------------------------------
# description  Insert a new object in to database. The identifier can be part of
#               parameter.
#----------------------------------------------------------------------------------------
# input        $hParameter | key value pairs (column name => value) | ref.hash
# return       $Identifier | object identifier | integer
#========================================================================================
sub insert {
    my $self = shift;
    my ($hParameter) = @_;

    my %Info = %$hParameter;
    my $IdentifierName = $self->identifier;
    $Info{$IdentifierName} = $self->nextid unless exists $Info{$IdentifierName};
    my $Identifier = $Info{$IdentifierName};

    # get statement
    my $ahStatements = $self->{'InsertStatements'};
    if (not defined $ahStatements) {
        $ahStatements = $self->_insertStatements;
        $self->{'InsertStatements'} = $ahStatements;
    } # else found in cache

    # find unused keys in $hParameter
    my %ParameterNames = map { $_ => undef } map { @{$_->{'PARAMETER'}} } @$ahStatements;
    my @UnknownParameters = grep { !exists $ParameterNames{$_} } keys %$hParameter;
    Error('COLUMN_NOT_FOUND', { Column=> (join ',', @UnknownParameters), Class=>$self->class}) if scalar @UnknownParameters;

    # execute statement
    my $dbi = GetCurrentDBHandle();
    $dbi->transaction( sub {
        foreach my $hStatement (@$ahStatements) {
            $Info{$hStatement->{'IDENTIFIER'}} = $Identifier;
            my @Parameter = map { exists $Info{$_} ? $Info{$_} : undef } @{$hStatement->{'PARAMETER'}};
            $dbi->do($hStatement->{'SQL'}, \@Parameter);
            $Identifier = $dbi->dbh->last_insert_id(undef,undef,undef,undef) unless defined $Identifier;
        }
    });
    return $Identifier;
}

#========================================================================================
# function     _updateStatements
# state        private
#----------------------------------------------------------------------------------------
# syntax       $ahStatements = $DAL->_updateStatements($hParameter);
#----------------------------------------------------------------------------------------
# description  Build update statements. The given parameter $hParameter is used to know
#               which tables should be updated.
#----------------------------------------------------------------------------------------
# input        $hParameter | key value pairs (column name => value) | ref.hash
# return       $ahStatements | list of statements keys:
#               <ul>
#                   <li>SQL - statement includes placeholder '?' - string</li>
#                   <li>PARAMETER - parameter (column names) for placeholder - string</li>
#               </ul> | ref.array.hash
#========================================================================================
sub _updateStatements {
    my $self = shift;
    my ($hParameter) = @_;

    my @Statements = ();
    my %hUnknownParameter = ();
    $hUnknownParameter{$_} = 1 foreach keys %$hParameter;
    foreach my $Table ( $self->tables ) {
        my (@Columns, @Parameter) = ((),());
        my $ExistsTextColumn = 0;
        my %Types;
        foreach my $Parameter (grep { $_ ne $Table->{'IDENTIFIER'} and exists $Table->{'COLUMN'}{$_} } sort keys %$hParameter) {
            delete $hUnknownParameter{$Parameter};
            push @Columns,   $Table->{'COLUMN'}{$Parameter}{'Code'};
            push @Parameter, $Parameter;
            $ExistsTextColumn = 1 if $Table->{'COLUMN'}{$Parameter}{'IsText'};
            $Types{$Parameter} = $Table->{'COLUMN'}{$Parameter}{'Type'};
        }
        if (scalar @Columns) {
            my $Statement = sprintf 'UPDATE `%s` SET %s WHERE %s=?',
                $Table->{'Code'},
                join (',', map { $_ . '=?' } @Columns),
                $Table->{'COLUMN'}{$Table->{'IDENTIFIER'}}{'Code'};
            push @Parameter, $Table->{'IDENTIFIER'};
            $Types{$Table->{'IDENTIFIER'}} = $Table->{'COLUMN'}{$Table->{'IDENTIFIER'}}{'Type'};
            push @Statements, {
                'SQL'        => $Statement,
                'IDENTIFIER' => $Table->{'IDENTIFIER'},
                'PARAMETER'  => \@Parameter,
                'EXISTS_TEXTCOLUMN' => $ExistsTextColumn,
                'TYPES' => \%Types,
            };
        }
    }
    Error('COLUMN_NOT_FOUND', { Column=> (join ',', keys %hUnknownParameter), Class=>$self->class}) if scalar %hUnknownParameter;
    return \@Statements;
}

#========================================================================================
# function     update
# state        public
#----------------------------------------------------------------------------------------
# syntax       $DAL->update($Identifier, $hParameter);
#----------------------------------------------------------------------------------------
# description  Update an existing object in database.
#----------------------------------------------------------------------------------------
# input        $Identifier | object identifier | integer
# input        $hParameter | key value pairs (column name => value) | ref.hash
#========================================================================================
sub update {
    my $self = shift;
    my ($Identifier, $hParameter) = @_;

    # get statement
    my $CacheKey = join ':', keys %$hParameter;
    my $ahStatements = $self->{'UpdateStatements'}{$CacheKey};
    if (not defined $ahStatements) {
        $ahStatements = $self->_updateStatements($hParameter);
        $self->{'UpdateStatements'}{$CacheKey} = $ahStatements;
    } # else found in cache

    # execute statement
    my $dbi = GetCurrentDBHandle();
    $dbi->transaction( sub {
        foreach my $hStatement (@$ahStatements) {
            my %Parameter = %$hParameter;
            $Parameter{$hStatement->{'IDENTIFIER'}} = $Identifier;
            $dbi->do($hStatement->{'SQL'}, [map { $Parameter{$_}} @{$hStatement->{'PARAMETER'}}]);
        }
    });
    return;
}

#========================================================================================
# function     _deleteStatements
# state        private
#----------------------------------------------------------------------------------------
# syntax       $ahStatements = $DAL->_deleteStatements;
#----------------------------------------------------------------------------------------
# description  Build delete statements.
#----------------------------------------------------------------------------------------
# return       $ahStatements | list of statements keys:
#               <ul>
#                   <li>SQL - statement includes placeholder '?' - string</li>
#                   <li>PARAMETER - parameter (column names) for placeholder - string</li>
#               </ul> | ref.array.hash
#========================================================================================
sub _deleteStatements {
    my $self = shift;

    my @Statements = ();
    foreach my $Table ( $self->tables ) {
        my $Statement = sprintf 'DELETE FROM `%s` WHERE %s=?',
            $Table->{'Code'},
            $Table->{'COLUMN'}{$Table->{'IDENTIFIER'}}{'Code'};
        push @Statements, {
            'SQL'       => $Statement,
            'PARAMETER' => [$Table->{'IDENTIFIER'}],
        };
    }
    return \@Statements;
}

#========================================================================================
# function     delete
# state        public
#----------------------------------------------------------------------------------------
# syntax       $DAL->delete($Identifier);
#----------------------------------------------------------------------------------------
# description  Update an existing object in database.
#----------------------------------------------------------------------------------------
# input        $Identifier | object identifier | integer
#========================================================================================
sub delete {
    my $self = shift;
    my ($Identifier) = @_;

    # get statement
    my $ahStatements = $self->{'DeleteStatements'};
    if (not defined $ahStatements) {
        $ahStatements = $self->_deleteStatements;
        $self->{'DeleteStatements'} = $ahStatements;
    } # else found in cache

    # execute statement
    my $dbi = GetCurrentDBHandle();
    $dbi->transaction( sub {
        foreach my $hStatement (@$ahStatements) {
            $dbi->do($hStatement->{'SQL'}, [$Identifier]);
        }
    });
    return;
}

#========================================================================================
# function     _getStatements
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hStatement = $DAL->_getStatements($IndexName);
#----------------------------------------------------------------------------------------
# description  Build get statement.
#----------------------------------------------------------------------------------------
# input        $IndexName | name of index | string
# input        $aOrderBy | (optional) attributes for sorting,
#                           e.g. [{ Column => 'Position', Direction => 'ASC' }]
#                         | ref.array.hash
# return       $ahStatements | list of statements keys:
#               <ul>
#                   <li>SQL - statement includes placeholder '?' - string</li>
#                   <li>COLUMNS - list of column names - array.string</li>
#                   <li>TYPES - list of data types of columns in the result set - array.string</li>
#               </ul> | ref.hash
#========================================================================================
sub _getStatements {
    my $self = shift;
    my ($IndexName, $aOrderBy) = @_;

    my @Columns = ();
    my @ColumnNames = ();
    my @ColumnTypes = ();
    my %Keys = ();
    my $BaseClassName = $self->baseClassName;
    foreach my $Table ( $self->tables ) {

        # select part
        my $hTableColumns = $Table->{'COLUMN'};
        foreach my $ColumnName (grep { $_ ne $Table->{'IDENTIFIER'} or $BaseClassName eq $Table->{'Name'} } sort keys %$hTableColumns) {
            next if exists $Keys{$ColumnName};
            $Keys{$ColumnName} = undef;
            push @Columns, sprintf ('`%s`.`%s`', $Table->{'Short'}, $hTableColumns->{$ColumnName}{'Code'});
            push @ColumnNames, $ColumnName;
            push @ColumnTypes, $hTableColumns->{$ColumnName}{'Type'};
        }
    }

    my $Columns = join ', ', @Columns;
    my $SQL = "SELECT $Columns" . $self->_getFromPart . $self->_getWherePart( $IndexName );
    $SQL .= $self->_getOrderByPart( $aOrderBy ) if $aOrderBy;

    return {
        'SQL'       => $SQL,
        'COLUMNS'   => \@ColumnNames,
        'TYPES'     => \@ColumnTypes,
    };
}

#========================================================================================
# function     _getIteratorStatements
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hStatement = $DAL->_getIteratorStatements($IndexName);
#----------------------------------------------------------------------------------------
# description  Builds the SELECT statement for an iterator.
#----------------------------------------------------------------------------------------
# input        $IndexName | name of index | string
# input        $aOrderBy | (optional) attributes for sorting,
#                           e.g. [{ Column => 'Position', Direction => 'ASC' }]
#                         | ref.array.hash
# return       $ahStatements | list of statements keys:
#               <ul>
#                   <li>SQL - statement includes placeholder '?' - string</li>
#                   <li>COLUMNS - list of column names - array.string</li>
#                   <li>TYPES - list of data types of columns in the result set - array.string</li>
#               </ul> | ref.hash
#========================================================================================
sub _getIteratorStatements {
    my $self = shift;
    my ($IndexName, $aOrderBy) = @_;

    my $hTable = $self->table( $self->baseClassName );
    my $Identifier = $hTable->{'IDENTIFIER'};

    my $SQL = $self->getSelectIDPart
        . $self->_getFromPart
        . $self->_getWherePart( $IndexName );
    $SQL .= $self->_getOrderByPart( $aOrderBy ) if $aOrderBy;

    return {
        'SQL'       => $SQL,
        'COLUMNS'   => [$Identifier],
        'TYPES'     => [$hTable->{'COLUMN'}{$Identifier}{'Type'}],
    };
}

sub getSelectIDPart {
    my $self = shift;

    my $hTable = $self->table( $self->baseClassName );
    return "SELECT `$hTable->{'Short'}`.`$hTable->{'IDENTIFIER'}`"
}

sub _getFromPart {
    my $self = shift;

    my @Tables = $self->tables;
    my $hMaster = pop @Tables;
    my @From = ( "`$hMaster->{'Code'}` AS `$hMaster->{'Short'}`" );
    foreach my $hTable ( @Tables ) {
        push @From, sprintf 'INNER JOIN `%s` AS `%s` ON `%s`.`%s` = `%s`.`%s`',
            $hTable->{'Code'}, $hTable->{'Short'},
            $hTable->{'Short'}, $hTable->{'COLUMN'}{$hTable->{'IDENTIFIER'}}{'Code'},
            $hMaster->{'Short'}, $hMaster->{'COLUMN'}{$hMaster->{'IDENTIFIER'}}{'Code'};
    }

    return "\nFROM " . join( "\n", @From );
}

sub _getWherePart {
    my $self = shift;
    my ($IndexName) = @_;

    my $aColumns = $self->index($IndexName)->{'COLUMNREF'};
    my @Clauses;
    foreach my $ColumnName (@$aColumns) {
        my ($hColumn, $hColumnTable) = $self->column($ColumnName);
        push @Clauses, "$hColumnTable->{'Short'}.$hColumn->{'Code'} = ?";
    }

    return "\nWHERE " . join( "\n  AND ", @Clauses );
}

sub _getOrderByPart {
    my $self = shift;
    my ($aOrderBy) = @_;

    return '' unless defined $aOrderBy;
    return '' if 0 == scalar @$aOrderBy;

    my @OrderBy;
    foreach my $hSortColumn ( @$aOrderBy ) {
        my $Column = $hSortColumn->{'Column'};
        my $Direction = $hSortColumn->{'Direction'};
        ERROR( 'INVALID_DIRECTION', {
            Column => $Column,
            Direction => $Direction,
        }) unless $Direction ~~ [qw(ASC DESC)];

        my ($hColumn, undef) = $self->column($Column);
        push @OrderBy, "$hColumn->{'Code'} $Direction";
    }

    return "\nORDER BY " . join( ', ', @OrderBy );
}

sub _statementCacheName {
    my $self = shift;
    my ($IndexName, $aOrderBy) = @_;

    return $IndexName unless $aOrderBy;
    return $IndexName . join('', map { "$_->{Column}$_->{Direction}" } @$aOrderBy );
}

#========================================================================================
# function     get
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ahObjects = $DAL->get($IndexName, $aParameter);
#----------------------------------------------------------------------------------------
# description  Get objects from database.
#----------------------------------------------------------------------------------------
# input        $IndexName | name of index | string
# input        $aParameter | parameter of index | ref.array
# input        $aOrderBy | (optional) attributes for sorting,
#                           e.g. [{ Column => 'Position', Direction => 'ASC' }]
#                         | ref.array.hash
# return       $ahObjects | object data | ref.array.hash
#========================================================================================
sub get {
    my $self = shift;
    my ($IndexName, $aParameter, $aOrderBy) = @_;

    # get statement
    my $CacheName = $self->_statementCacheName( $IndexName, $aOrderBy );
    my $hStatement = $self->{'SelectStatements'}{$CacheName};
    if (not defined $hStatement) {
        $hStatement = $self->_getStatements($IndexName, $aOrderBy);
        $self->{'SelectStatements'}{$CacheName} = $hStatement;
    } # else found in cache

    # execute statement
    return GetCurrentDBHandle()->execute($hStatement->{'SQL'}, $aParameter, $hStatement->{'TYPES'}, $hStatement->{'COLUMNS'});
}

#========================================================================================
# function     iterator
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Iterator = $DAL->iterator($IndexName, $aParameter, $aOrderBy);
#----------------------------------------------------------------------------------------
# description  Get objects ids from database as iterator.
#----------------------------------------------------------------------------------------
# input        $IndexName | name of index | string
# input        $aParameter | parameter of index | ref.array
# input        $aOrderBy | (optional) attributes for sorting,
#                           e.g. [{ Column => 'Position', Direction => 'ASC' }]
#                         | ref.array.hash
# return       $Iterator | iterator of object ids | DE_EPAGES::Core::API::Iterator
#========================================================================================
sub iterator {
    my $self = shift;
    my ($IndexName, $aParameter, $aOrderBy) = @_;

    # get statement
    my $CacheName = $self->_statementCacheName( $IndexName, $aOrderBy );
    my $hStatement = $self->{'IteratorStatements'}{$CacheName};
    if (not defined $hStatement) {
        $hStatement = $self->_getIteratorStatements($IndexName, $aOrderBy);
        $self->{'IteratorStatements'}{$CacheName} = $hStatement;
    } # else found in cache

    return DE_EPAGES::Core::API::Iterator->wrap(
        Source => GetCurrentDBHandle()->iterator($hStatement->{'SQL'}, $aParameter),
        Handle => sub { $_[1]->[0] },
    );
}

#========================================================================================
# function     _infoStatements
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hStatement = $DAL->_infoStatements;
#----------------------------------------------------------------------------------------
# description  Build info statement (select for one object).
#----------------------------------------------------------------------------------------
# return       $hStatement | list of statements keys:
#               <ul>
#                   <li>SQL - statement includes placeholder '?' - string</li>
#                   <li>PARAMETER - parameter (column names) for placeholder - array.string</li>
#                   <li>COLUMNS - list of column names - array.string</li>
#               </ul> | ref.hash
#========================================================================================
sub _infoStatements {
    my $self = shift;

    my @Columns = ();
    my @ColumnNames = ();
    my @ColumnTypes = ();
    my @From = ();
    my @Clauses = ();
    my @Parameter = ();
    my %Keys = ();
    my $Master = undef;
    my $BaseClassName = $self->baseClassName;
    my @ClassNames = $self->classNames;
    foreach my $ClassName (@ClassNames) {
        my $Table = $self->table($ClassName);
        # select part
        foreach my $ColumnName (grep { $_ ne $Table->{'IDENTIFIER'} or $BaseClassName eq $ClassName } sort keys %{$Table->{'COLUMN'}}) {
            next if exists $Keys{$ColumnName};
            $Keys{$ColumnName} = undef;
            push @Columns, sprintf ('`%s`.`%s`', $Table->{'Short'}, $Table->{'COLUMN'}{$ColumnName}{'Code'});
            push @ColumnNames, $ColumnName;
            push @ColumnTypes, $Table->{'COLUMN'}{$ColumnName}{'Type'};
        }
        # from part
        if (defined $Master) {
            push @From, sprintf ('INNER JOIN `%s` AS `%s` ON (`%s`.`%s` = `%s`.`%s`)',
                $Table->{'Code'}, $Table->{'Short'},
                $Table->{'Short'}, $Table->{'COLUMN'}{$Table->{'IDENTIFIER'}}{'Code'},
                $Master->{'Short'}, $Master->{'COLUMN'}{$Master->{'IDENTIFIER'}}{'Code'});
        } else {
            push @From, sprintf ('`%s` AS `%s`', $Table->{'Code'}, $Table->{'Short'});
            push @Clauses, sprintf ('`%s`.`%s`=?', $Table->{'Short'}, $Table->{'COLUMN'}{$Table->{'IDENTIFIER'}}{'Code'});
            push @Parameter, $Table->{'IDENTIFIER'};
        }
        $Master = $Table;
    }
    return {
        'COLUMNS'   => \@ColumnNames,
        'TYPES'     => \@ColumnTypes,
        'PARAMETER' => \@Parameter,
        'SQL'       => sprintf ('SELECT %s FROM %s WHERE %s',
            join (', ', @Columns),
            join (' ', @From),
            join (' AND ', @Clauses)),
    };
}

#========================================================================================
# function     info
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hObject = $DAL->info($Identifier);
#----------------------------------------------------------------------------------------
# description  Get object from database.
#----------------------------------------------------------------------------------------
# input        $Identifier | object identifier | integer
# return       $hObject | object data | ref.hash
#========================================================================================
sub info {
    my $self = shift;
    my ($Identifier) = @_;

    # get statement
    my $hStatement = $self->{'InfoStatements'};
    if (not defined $hStatement) {
        $hStatement = $self->_infoStatements;
        $self->{'InfoStatements'} = $hStatement;
    } # else found in cache

    # execute statement
    my $ahResult = GetCurrentDBHandle()->execute($hStatement->{'SQL'}, [$Identifier], $hStatement->{'TYPES'}, $hStatement->{'COLUMNS'});
    return undef unless scalar @$ahResult;
    return $ahResult->[0];
}

#========================================================================================
# function     _multiStatements
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hStatement = $DAL->_multiStatements;
#----------------------------------------------------------------------------------------
# description  Build multi statement (select for multiple object).
#----------------------------------------------------------------------------------------
# return       $hStatement | list of statements keys:
#               <ul>
#                   <li>SQL - statement includes placeholder '?' - string</li>
#                   <li>PARAMETER - parameter (column names) for placeholder - array.string</li>
#                   <li>COLUMNS - list of column names - array.string</li>
#               </ul> | ref.hash
#========================================================================================
sub _multiStatements {
    my $self = shift;

    my @Columns = ();
    my @ColumnNames = ();
    my @ColumnTypes = ();
    my @From = ();
    my $InClause = undef;
    my %Keys = ();
    my $Master = undef;

    my $BaseClassName = $self->baseClassName;
    my @ClassNames = $self->classNames;
    foreach my $ClassName (@ClassNames) {
        my $Table = $self->table($ClassName);

        # select part
        foreach my $ColumnName (grep { $_ ne $Table->{'IDENTIFIER'} or $BaseClassName eq $ClassName } sort keys %{$Table->{'COLUMN'}}) {
            next if exists $Keys{$ColumnName};
            my $hColumn = $Table->{'COLUMN'}{$ColumnName};
            $Keys{$ColumnName} = undef;
            push @Columns, sprintf ('`%s`.`%s`', $Table->{'Short'}, $hColumn->{'Code'});
            push @ColumnNames, $ColumnName;
            push @ColumnTypes, $hColumn->{'Type'};
        }
        # from part
        if (defined $Master) {
            push @From, sprintf ('INNER JOIN `%s` AS `%s` ON (`%s`.`%s` = `%s`.`%s`)',
                $Table->{'Code'}, $Table->{'Short'},
                $Table->{'Short'}, $Table->{'COLUMN'}{$Table->{'IDENTIFIER'}}{'Code'},
                $Master->{'Short'}, $Master->{'COLUMN'}{$Master->{'IDENTIFIER'}}{'Code'});
        } else {
            push @From, sprintf ('`%s` AS `%s`', $Table->{'Code'}, $Table->{'Short'});
            $InClause = sprintf ('`%s`.`%s` IN ', $Table->{'Short'}, $Table->{'COLUMN'}{$Table->{'IDENTIFIER'}}{'Code'});
        }
        $Master = $Table;
    }
    return {
        'COLUMNCODES' => \@Columns,
        'COLUMNS'   => \@ColumnNames,
        'TYPES'     => \@ColumnTypes,
        'FROMS'     => \@From,
        'INCLAUSE'  => $InClause,
    };
}

#========================================================================================
# function     multi
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ahInfo = $DAL->multi($aIdentifier);
#----------------------------------------------------------------------------------------
# description  Get objects from database.
#----------------------------------------------------------------------------------------
# input        $aIdentifier | list of ids | ref.array.int
# return       $ahInfo | list of data hashes, one element for each id | ref.array.hash
#========================================================================================
sub multi {
    my $self = shift;
    my ($aIdentifier) = @_;
    return [] unless scalar @$aIdentifier;

    my $hStatement = $self->{'multiStatements'};
    if (not defined $hStatement) {
        $hStatement = $self->_multiStatements;
        $self->{'multiStatements'} = $hStatement;
    } # else found in cache

    # execute statement
    my $aaResult = [];
    my $dbi = GetCurrentDBHandle();
    my @UnLoadedIDs = @$aIdentifier;

    # get objects from database in chunks of 500, because the number of items in the
    # IN() clause is limited by ASE
    while (scalar @UnLoadedIDs) {
        my $BlockLength = scalar @UnLoadedIDs;
        $BlockLength = MAX_IN_CLAUSE_SIZE if $BlockLength > MAX_IN_CLAUSE_SIZE;
        my @CurrentBlock = splice( @UnLoadedIDs, 0, $BlockLength );
        my $SQL = sprintf 'SELECT %s FROM %s WHERE %s',
            join( ',',   @{$hStatement->{'COLUMNCODES'}} ),
            join( ' ',      @{$hStatement->{'FROMS'}} ),
            join( ' and ', $hStatement->{'INCLAUSE'} . '(' . join( ',', map { '?' } @CurrentBlock ). ')' );

        push @$aaResult, @{$dbi->execute($SQL, \@CurrentBlock, $hStatement->{'TYPES'})};
    }

    my $ahResult = GetLoopFromDoubleArray($aaResult, $hStatement->{'COLUMNS'});
    Warning('NOT_ALL_RESULTS', { 'Result' => scalar @$ahResult, 'Identifier' => scalar @$aIdentifier, 'Class'=> $self->class }) unless scalar @$ahResult == scalar @$aIdentifier;
    # sort result
    my ($runIndex, $IdentifierName, %Index, @Result) = (0, $self->identifier, (), ());
    $Index{$_} = $runIndex++ foreach @$aIdentifier;
    $Result[$Index{$_->{$IdentifierName}}] = $_ foreach @$ahResult;
    return \@Result;
}

#========================================================================================
# function     _existsStatements
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hStatement = $DAL->_existsStatements;
#----------------------------------------------------------------------------------------
# description  Build exists statement (select for one object).
#----------------------------------------------------------------------------------------
# return       $hStatement | list of statements keys:
#               <ul>
#                   <li>SQL - statement includes placeholder '?' - string</li>
#                   <li>PARAMETER - parameter (column names) for placeholder - array.string</li>
#               </ul> | ref.hash
#========================================================================================
sub _existsStatements {
    my $self = shift;

    my $Table = $self->table($self->{'CLASS'});
    my $Statement = sprintf 'SELECT 1 FROM `%s` WHERE %s=?',
        $Table->{'Code'},
        $Table->{'COLUMN'}{$Table->{'IDENTIFIER'}}{'Code'};
    return {
        'SQL'       => $Statement,
        'PARAMETER' => [$Table->{'IDENTIFIER'}],
    };
}

#========================================================================================
# function     exists
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hObject = $DAL->exists($Identifier);
#----------------------------------------------------------------------------------------
# description  Exists object in database.
#----------------------------------------------------------------------------------------
# input        $Identifier | object identifier | integer
# return       $Exists | true or false | boolean
#========================================================================================
sub exists {
    my $self = shift;
    my ($Identifier) = @_;

    # get statement
    my $hStatement = $self->{'ExistsStatements'};
    if (not defined $hStatement) {
        $hStatement = $self->_existsStatements;
        $self->{'ExistsStatements'} = $hStatement;
    } # else found in cache

    # execute statement
    my $result = GetCurrentDBHandle()->execute($hStatement->{'SQL'}, [$Identifier]);
    return scalar @$result ? 1 : 0;
}

#========================================================================================
# function     _nextidStatements
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hStatement = $DAL->_nextidStatements;
#----------------------------------------------------------------------------------------
# description  Build nextid statement calls procedure n_<TableName>.
#----------------------------------------------------------------------------------------
# return       $hStatement | list of statements keys:
#               <ul>
#                   <li>SQL - statement includes placeholder '?' - string</li>
#               </ul> | ref.hash
#========================================================================================
sub _nextidStatements {
    my $self = shift;

    die "method _nextidStatements is not implemented in $self"
}

#========================================================================================
# function     nextid
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hStatement = $DAL->nextid;
#----------------------------------------------------------------------------------------
# description  Calls procedure n_<TableName>.
#----------------------------------------------------------------------------------------
# return       $NextIdentifier | next identifier | integer
#========================================================================================
sub nextid {
    my $self = shift;

    # get statement
    my $hStatement = $self->{'NextIDStatements'};
    if (not defined $hStatement) {
        $hStatement = $self->_nextidStatements;
        $self->{'NextIDStatements'} = $hStatement;
    } # else found in cache

    # execute statement
    my $result = GetCurrentDBHandle()->execute($hStatement->{'SQL'}, []);
    return $result->[0][0];
}

#========================================================================================
# function     _allStatements
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hStatement = $DAL->_allStatements;
#----------------------------------------------------------------------------------------
# description  Build all statement (select for all objects).
#----------------------------------------------------------------------------------------
# return       $hStatement | list of statements keys:
#               <ul>
#                   <li>SQL - statement includes - string</li>
#               </ul> | ref.hash
#========================================================================================
sub _allStatements {
    my $self = shift;

    my @Columns = ();
    my @ColumnNames = ();
    my @ColumnTypes = ();
    my @From = ();
    my %Keys = ();
    my $Master = undef;

    my @ClassNames = $self->classNames;
    foreach my $ClassName (@ClassNames) {
        my $Table = $self->table($ClassName);

        # select part
        foreach my $ColumnName (sort keys %{$Table->{'COLUMN'}}) {
            next if exists $Keys{$ColumnName};
            $Keys{$ColumnName} = undef;
            push @Columns, sprintf ('`%s`.`%s`', $Table->{'Short'}, $Table->{'COLUMN'}{$ColumnName}{'Code'});
            push @ColumnNames, $ColumnName;
            push @ColumnTypes, $Table->{'COLUMN'}{$ColumnName}{'Type'};
        }
        # from part
        if (defined $Master) {
            push @From, sprintf ('INNER JOIN `%s` AS `%s` ON (`%s`.`%s` = `%s`.`%s`)',
                $Table->{'Code'}, $Table->{'Short'},
                $Table->{'Short'}, $Table->{'COLUMN'}{$Table->{'IDENTIFIER'}}{'Code'},
                $Master->{'Short'}, $Master->{'COLUMN'}{$Master->{'IDENTIFIER'}}{'Code'});
        } else {
            push @From, sprintf ('`%s` AS `%s`', $Table->{'Code'}, $Table->{'Short'});
        }
        $Master = $Table;
    }
    return {
        'COLUMNS'   => \@ColumnNames,
        'TYPES'     => \@ColumnTypes,
        'SQL'       => sprintf ('SELECT %s FROM %s',
            join (', ', @Columns),
            join (' ',  @From)),
    };
}

#========================================================================================
# function     all
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ahObject = $DAL->all;
#----------------------------------------------------------------------------------------
# description  all objects of this table at database.
#----------------------------------------------------------------------------------------
# return       $ahObject | objects | ref.array.hash
#========================================================================================
sub all {
    my $self = shift;

    # get statement
    my $hStatement = $self->{'allStatements'};
    if (not defined $hStatement) {
        $hStatement = $self->_allStatements;
        $self->{'allStatements'} = $hStatement;
    } # else found in cache

    # execute statement
    return GetCurrentDBHandle()->execute($hStatement->{'SQL'}, [], $hStatement->{'TYPES'}, $hStatement->{'COLUMNS'});
}

#========================================================================================
# function     mapObjects2IDs
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hFlatInfo = $DAL->mapObjects2IDs($hInfo);
#----------------------------------------------------------------------------------------
# description  Maps all objects (given at xml files) to there id.
#----------------------------------------------------------------------------------------
# input        $hInfo | object info incl. objects | ref.hash.*
# return       $hFlatInfo | object info without objects | ref.hash.*
#========================================================================================
sub mapObjects2IDs {
    my $self = shift;
    my ($hInfo) = @_;
    my %Data = ();
    # remove keys without database column
    my @ClassNames = $self->classNames;
    foreach my $ClassName (@ClassNames) {
        my $Table = $self->table($ClassName);
        foreach my $ColumnName (keys %{$Table->{'COLUMN'}}) {
            $Data{$ColumnName} = $hInfo->{$ColumnName} if exists $hInfo->{$ColumnName};
        }
    }

    # map objects to ids (e.g.Object -> ObjectID, Parent -> ParentID)
    my $Map = $self->{'OBJECT_MAP'};
    foreach my $key (keys %$Map) {
        if (exists $hInfo->{$key}) {
            $Data{$Map->{$key}} = (defined $hInfo->{$key} ? $hInfo->{$key}->id : undef) unless exists $hInfo->{$Map->{$key}};
        }
    }
    return \%Data;
}

1;
