
use strict;
use Getopt::Long;
use Sys::Hostname;
use Time::HiRes qw (time);
use DE_EPAGES::Core::API::Script qw (RunScript);
use DE_EPAGES::Database::API::Connection qw (GetCurrentDBHandle RunOnStore);
use DE_EPAGES::Database::API::Config qw (StoreConnections DatabaseConnectionData);

#========================================================================================
# function     Check
# state        private
#----------------------------------------------------------------------------------------
# syntax       Check( $table, $dbi, $Verbose );
#----------------------------------------------------------------------------------------
# description  Checks a table or tables for errors. Works for MyISAM, InnoDB, and
#               (as of MySQL 5.0.16) ARCHIVE tables. For MyISAM tables, the key statistics
#               are updated as well.
#
#               If Check finds a problem for an InnoDB table, the server shuts down to
#               prevent error propagation. Details of the error will be written to the
#               error log.
#               (From:http://dev.mysql.com/doc/refman/5.0/en/check-table.html)
#----------------------------------------------------------------------------------------
# input        $table | tablename | string
# input        $dbi | Database handle | object
# input        $Verbose | print progress messages | boolean
#========================================================================================
sub Check {
    my ( $TableName, $dbi, $Verbose ) = @_;

    my $aRows   = $dbi->execute("CHECK TABLE `$TableName`");
    foreach my $aRow (@$aRows) {
        my ($Type, $Text) = @{$aRow}[2,3];
        print "\t[Check $TableName] $Type: $Text\n" if( $Type =~ /^Error$/i || $Verbose );
    }
}

#========================================================================================
# function     Optimize
# state        private
#----------------------------------------------------------------------------------------
# syntax       Optimize( $table, $dbi, $Verbose );
#----------------------------------------------------------------------------------------
# description   Optimize should be used if you have deleted a large part of a table or
#                if you have made many changes to a table with variable-length rows
#               (tables that have VARCHAR, VARBINARY, BLOB, or TEXT columns). Deleted rows
#               are maintained in a linked list and subsequent INSERT operations reuse old
#               row positions. You can use Optimize to reclaim the unused space and to
#               defragment the data file.
#
#               In most setups, you need not run Optimize at all. Even if you do a lot
#               of updates to variable-length rows, it is not likely that you need to do
#               this more than once a week or month and only on certain tables.
#
#               For InnoDB tables, Optimize is mapped to ALTER TABLE, which rebuilds the
#               table to update index statistics and free unused space in the clustered
#               index.
#               (From:http://dev.mysql.com/doc/refman/5.0/en/optimize-table.html)
#----------------------------------------------------------------------------------------
# input        $table | tablename | string
# input        $dbi | Database handle | object
# input        $Verbose | print progress messages | boolean
#========================================================================================
sub Optimize {
    my ( $TableName, $dbi, $Verbose ) = @_;

    my $aRows = $dbi->execute("OPTIMIZE NO_WRITE_TO_BINLOG TABLE `$TableName`");
    foreach my $aRow (@$aRows) {
        my ($Type, $Text) = @{$aRow}[2,3];
        print "\t[Optimize $TableName] $Type: $Text\n" if( $Type =~ /^Error$/i || $Verbose );
    }
}

#========================================================================================
# function     Analyze
# state        private
#----------------------------------------------------------------------------------------
# syntax       Analyze( $table, $dbi, $Verbose );
#----------------------------------------------------------------------------------------
# description  Analyzes and stores the key distribution for a table. During the analysis,
#               the table is locked with a read lock for MyISAM and BDB. For InnoDB the
#               table is locked with a write lock. This statement works with MyISAM, BDB,
#               and InnoDB tables.
#
#               MySQL uses the stored key distribution to decide the order in which tables
#               should be joined when you perform a join on something other than a constant.
#               In addition, key distributions can be used when deciding which indexes to
#               use for a specific table within a query.
#               (From:http://dev.mysql.com/doc/refman/5.0/en/analyze-table.html)
#----------------------------------------------------------------------------------------
# input        $table | tablename | string
# input        $dbi | Database handle | object
# input        $Verbose | print progress messages | boolean
#========================================================================================
sub Analyze {
    my ( $TableName, $dbi, $Verbose ) = @_;

    my $aRows = $dbi->execute("ANALYZE NO_WRITE_TO_BINLOG TABLE `$TableName`");
    foreach my $aRow (@$aRows) {
        my ($Type, $Text) = @{$aRow}[2,3];
        print "\t[Analyze $TableName] $Type: $Text\n" if( $Type =~ /^Error$/i || $Verbose );
    }
}

#========================================================================================
# function     _ForeachTable
# state        private
#----------------------------------------------------------------------------------------
# syntax       _ForeachTable( $Sub, $Verbose );
#----------------------------------------------------------------------------------------
# description  Executes $Sub->($Table, $dbi, $Verbose) for each table in the current
#                database.
#----------------------------------------------------------------------------------------
# input        $Sub | function to run on each table | ref.code
# input        $Verbose | print progress messages | boolean
#========================================================================================
sub _ForeachTable {
    my ($Sub, $Verbose) = @_;

    my $dbi = GetCurrentDBHandle();

    my @Tables = map { $_->[0] } @{ $dbi->execute(<<'END_SQL') };
SELECT TABLE_NAME
FROM INFORMATION_SCHEMA.TABLES
WHERE TABLE_SCHEMA = DATABASE()
  AND TABLE_TYPE = 'BASE TABLE'
END_SQL
    foreach my $Table ( @Tables ) {
        my $time0 = time;
        $Sub->($Table, $dbi, $Verbose);
        printf( "%s (%.1f secs)\n", $Table, time - $time0 ) if $Verbose;
    }
}

sub Main {

    local $| = 1;

    my $Password;
    my $StoreName;
    my $Help;
    my $NoOutput;
    my $Optimize = 0;
    my $Analyze = 0;
    my $Check = 0;

    GetOptions(
        'help'           => \$Help,
        'passwd=s'       => \$Password,
        'storename=s'    => \$StoreName,
        'nooutput'       => \$NoOutput,
        'check'          => \$Check,
        'optimize'       => \$Optimize,
        'analyze'        => \$Analyze,
    );

    usage() if $Help;

    my $OptionsSum = $Check + $Analyze + $Optimize;
    if ( 0 == $OptionsSum ) {
        print STDERR "Missing parameter -check or -analyze or -optimize!\n\n";
        usage();
    }

    if ( $OptionsSum > 1 ) {
        print STDERR "Please specify only one parameter -check or -analyze or -optimize at a time!\n\n";
        usage();
    }

    my @StoreList = ( $StoreName );

    unless ($StoreName) {
        # without -storename option, check all stores
        my $HostName = hostname();
        @StoreList = ();
        $Password = undef; # use password from Database.d/$Store.conf
        foreach my $Store (@{StoreConnections()}) {
            my $hSection = DatabaseConnectionData($Store);
            next if defined $hSection->{active} and not $hSection->{active};
            next unless $hSection->{'datasource'} =~ /^dbi:mysql/;
            my $SectionHost = $hSection->{'BackupHost'};
            if ($SectionHost and $SectionHost !~ /^(ipv6-)?localhost$/) {
                next unless $SectionHost eq $HostName;
            }
            push @StoreList, $Store;
        }
    }

    foreach my $Store (@StoreList) {
        my $time1 = time;
        RunOnStore(
            'Store'      => $Store,
            'DBPassword' => $Password,
            'Sub'        => sub {
                _ForeachTable( \&Analyze, !$NoOutput ) if $Analyze;
                _ForeachTable( \&Optimize, !$NoOutput ) if $Optimize;
                _ForeachTable( \&Check, !$NoOutput ) if $Check;
            }
        );
        printf( "$Store: need %.1f seconds\n\n", time - $time1 ) unless $NoOutput;
    }
}

sub usage {
    print <<END_USAGE;
Usage:
    perl $0 [options] [flags]

Options:
    -passwd         database user password (optional)
    -storename      name of store (optional)

Flags:
    -help           show the command line options
    -nooutput       do not print any output
    -optimize       optimize index data of table(s) (includes analyze);
                    see mysql documentation "OPTIMIZE TABLE"
    -check          checks table(s);
                    see mysql documentation "CHECK TABLE"
    -analyze        analyzes and stores the key distribution for table(s);
                    see mysql documentation "ANALYZE TABLE"

Example:
    perl $0 -storename Store -analyze
END_USAGE
    exit 2;
}

RunScript( 'Sub' => \&Main );
