use strict;
use Encode qw( decode );
use Getopt::Long;
use Time::HiRes qw( time );
use DE_EPAGES::Core::API::Script  qw( RunScript );
use DE_EPAGES::Database::API::Connection qw( GetCurrentDBHandle RunOnStore );

RunScript( 'Sub' => \&Main );

sub _GetTablesInCopyOrder {
    my ($dbiSource, $dbiTarget)  = @_;

    my $aaReferences = $dbiSource->getAllTableReferences();
    my @Tables = reverse @{ $dbiSource->sortDeleteAllTables($aaReferences) };
    @Tables = grep { !/_nextid$/ } @Tables unless $dbiTarget->driverName eq 'Sybase';

    return \@Tables;
}

sub _SetUpDatabaseHandles {
    my ($dbiSource, $dbiTarget) = @_;

    my $dbhSource = $dbiSource->dbh;
    my $dbhTarget = $dbiTarget->dbh;
    $dbhSource->{'RaiseError'} = 1;
    $dbhSource->{'ShowErrorStatement'} = 1;
    $dbhTarget->{'RaiseError'} = 1;
    $dbhTarget->{'ShowErrorStatement'} = 1;
}

sub TruncateTables {
    my ($dbiTarget, $aTables) = @_;

    print 'truncate tables ';
    my $time0 = time();
    if( $dbiTarget->driverName eq 'Sybase' ) {
        $dbiTarget->do("DELETE FROM `$_`") foreach reverse @$aTables;
    }
    else {
        $dbiTarget->do("TRUNCATE TABLE `$_`") foreach reverse @$aTables;
    }
    printf "%.1f s\n", time()-$time0;
}

sub CopyTables {
    my ($aTables, $dbiSource, $dbiTarget) = @_;

    foreach my $Table ( @$aTables) {
        print "copy $Table ";
        my $time0 = time();
        CopyTable( $Table, $dbiSource, $dbiTarget );
        printf "%.1f s\n", time()-$time0;
    }
}

sub CopyTable {
    my ($Table, $dbiSource, $dbiTarget) = @_;

    my $Select = _SelectStatement( $dbiSource, $Table );
    my $dbhSource = $dbiSource->dbh;
    my $sthSource = $dbhSource->prepare($Select);
    $sthSource->execute;

    my @Columns = $dbiSource->tableColumns($Table);
    my $Columns = join ', ', @Columns;
    my $Placeholders = join ', ', map { '?' } @Columns;
    my $Insert = "INSERT INTO `$Table` ($Columns) VALUES ($Placeholders)";

    # With transaction it's much faster for MySQL.
    $dbiTarget->transaction( sub {
        my $dbhTarget = $dbiTarget->dbh;
        my $sthTarget = $dbhTarget->prepare($Insert);
        while( my $hValues = $sthSource->fetchrow_hashref ) {
            _DecodeRow( $hValues );
            $sthTarget->execute( @{$hValues}{@Columns} );
        }
        $sthTarget->finish;
    });

    $sthSource->finish;
}

sub _DecodeRow {
  my ($hValues) = @_;

    foreach my $Key (keys %$hValues) {
        unless (utf8::decode($hValues->{$Key})) {
            print "WARNING (utf8::decode failed for '$hValues->{$Key}'"
                . ' is_utf8: ' . utf8::is_utf8($hValues->{$Key})
                . ' valid: ' . utf8::valid($hValues->{$Key});
            $hValues->{$Key} = decode('utf-8', $hValues->{$Key});
        }
    }
    return;
}

sub _SelectStatement {
    my ($dbi, $Table, $Where) = @_;

    my @KeyColumns = $dbi->primaryKeyColumns( $Table );

    my $DriverName = $dbi->driverName;
    my $sth = $dbi->dbh->column_info( undef, undef, $Table, undef );
    my $ahColumns = $sth->fetchall_arrayref( {} );
    my @SelectColumns = map {
        $_->{'TYPE_NAME'} eq 'datetime'
            ? ($DriverName eq 'Sybase'
                ? "CONVERT(CHAR(19), $_->{'COLUMN_NAME'}, 117)"
                : "DATE_FORMAT($_->{'COLUMN_NAME'}, '%Y/%m/%d %H:%i:%S')"
              ). " AS $_->{'COLUMN_NAME'}"
            : $_->{'COLUMN_NAME'};
    } @$ahColumns;
    my $Select = 'SELECT ';
    $Select .= join ', ', @SelectColumns;
    $Select .= " FROM `$Table`";
    $Select .= $Where;
    $Select .= ' ORDER BY '.join(', ', @KeyColumns) if scalar @KeyColumns;
    return $Select;
}

sub _DisableForeignKeyChecks {
    my ($dbi, $Sub) = @_;

    if( $dbi->driverName eq 'mysql' ) {
        $dbi->disableForeignKeyChecks( $Sub );
    }
    else {
        $Sub->();
    }
}

sub Main {

    local $| = 1;

    my ($Help, $SourceStoreName, $SourcePassword, $TargetStoreName, $TargetPassword, $OnlyTable);
    GetOptions(
        'help'           => \$Help,
        'source=s'       => \$SourceStoreName,
        'sourcepasswd=s' => \$SourcePassword,
        'target=s'       => \$TargetStoreName,
        'targetpasswd=s' => \$TargetPassword,
        'table=s'         => \$OnlyTable,
    );

    usage() if ($Help);
    usage('Missing parameter -source') unless $SourceStoreName;
    usage('Missing parameter -target') unless $TargetStoreName;

    my $time0 = time;
    printf "CopyAllTables %s -> %s\n", $SourceStoreName, $TargetStoreName;
    RunOnStore( 'Store' => $SourceStoreName, 'DBPassword' => $SourcePassword, 'Sub' => sub {
        my $dbiSource = GetCurrentDBHandle();
        RunOnStore( 'Store' => $TargetStoreName, 'DBPassword' => $TargetPassword, 'Sub' => sub {
            my $dbiTarget = GetCurrentDBHandle();
            _SetUpDatabaseHandles ( $dbiSource, $dbiTarget );
            my $aTables = defined $OnlyTable ? [ $OnlyTable ] : _GetTablesInCopyOrder($dbiSource, $dbiTarget);
            _DisableForeignKeyChecks( $dbiTarget, sub {
                TruncateTables( $dbiTarget, $aTables );
                CopyTables( $aTables, $dbiSource, $dbiTarget );
            });

       });
    });
    printf "need %.1f seconds\n", time()-$time0;
}

sub usage {
    my ($Message) = @_;

    if (defined $Message) {
        print STDERR "$Message\n\n"
    } else {
        print <<END_USAGE;
Copies the content of all tables from one database to another.
Any existing data in the target tables is deleted first.

usage:
    perl $0 [options] [flags]

options:
    -source        source store store
    -sourcepasswd  source database password (optional)
    -target        target store store
    -targetpasswd  target database password (optional)
    -table         only copy this table
flags:
    -help       show the command line options

Example:
    perl $0 -source Store -target Store2
    perl $0 -source Store -target Store2 -table localizedstringattribute
END_USAGE
    }
    exit 2;
}

