use strict;
use Encode qw( encode );
use Getopt::Long;
use Time::HiRes qw (time);
use DE_EPAGES::Core::API::File qw( CreateDirectory );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw( GetCurrentDBHandle RunOnStore );

sub DumpAllTables {
    my ($dbiSource, $DumpDir) = @_;

    CreateDirectory( $DumpDir );

    my $dbhSource = $dbiSource->dbh;

    $dbhSource->{'RaiseError'} = 1;
    $dbhSource->{'ShowErrorStatement'} = 1;

    my $aaReferences = $dbiSource->getAllTableReferences();
    my $aTables = $dbiSource->sortDeleteAllTables($aaReferences);

    foreach my $Table (reverse @$aTables) {
        print "dump $Table ";

        my $time0 = time();

        my $FileName = "$DumpDir/$Table.txt";
        open my $fh, '>', $FileName or die "Can't open $FileName: $!";

        my $Select = _SelectStatement( $dbiSource, $Table );
        my $sth = $dbhSource->prepare($Select);
        $sth->execute;

        while( my $aValues = $sth->fetchrow_arrayref ) {
            my @Values = map { $_ // 'NULL' } @$aValues;
            my $Line = join("\t", @Values) . "\n";
            print $fh encode('utf-8', $Line);
        }

        $sth->finish;
        close $fh  or die "Can't close $FileName: $!";

        printf " %.1f s\n", time()-$time0;
    }

    return;
}

sub _SelectStatement {
    my ($dbi, $Table) = @_;

    my @KeyColumns = $dbi->primaryKeyColumns( $Table );

    my $DriverName = $dbi->driverName;
    my $sth = $dbi->dbh->column_info( undef, undef, $Table, undef );
    my $ahColumns = $sth->fetchall_arrayref( {} );
    my @SelectColumns = map {
            $_->{'TYPE_NAME'} eq 'datetime'
                ? ($DriverName eq 'Sybase'
                    ? "CONVERT(CHAR(19), $_->{'COLUMN_NAME'}, 117)"
                    : "DATE_FORMAT($_->{'COLUMN_NAME'}, '%Y/%m/%d %H:%i:%S')"
                  ). " AS $_->{'COLUMN_NAME'}"
                : $_->{'COLUMN_NAME'};
        }
        sort { $a->{'ORDINAL_POSITION'} <=> $b->{'ORDINAL_POSITION'} }
           @$ahColumns;
    my $Select = 'SELECT ';
    $Select .= join ', ', @SelectColumns;
    $Select .= " FROM `$Table`";
    $Select .= ' ORDER BY '.join(', ', @KeyColumns) if scalar @KeyColumns;
    return $Select;
}

sub Main {

    local $| = 1;

    my $StoreName;
    my $Password;
    my $DumpDir;
    my $Help;

    GetOptions(
        'help'        => \$Help,
        'storename=s' => \$StoreName,
        'passwd=s'    => \$Password,
        'dumpdir=s'   => \$DumpDir,
    );

    usage() if $Help;

    unless($StoreName) {
        print STDERR "Missing parameter -source\n\n";
        usage();
    }
    unless($DumpDir) {
        print STDERR "Missing parameter -dumpdir\n\n";
        usage();
    }

    my $time0 = time;

    RunOnStore( 'Store' => $StoreName, 'DBPassword' => $Password, 'Sub' => sub {
        my $dbiSource = GetCurrentDBHandle();
           DumpAllTables( $dbiSource, $DumpDir );
    });

    printf "need %.3f seconds\n", time()-$time0;
}

sub usage {
    print <<END_USAGE;
Writes the content of all tables to text files.
The character encoding is utf-8. Field separator is a tabulator.

usage:
    perl $0 [options] [flags]

options:
    -storename    store store
    -passwd       database password (optional)
    -dumpdir      target directory

flags:
    -help         show the command line options

Example:
    perl $0 -storename Store -dumpdir Dump
END_USAGE
    exit 2;
}

RunScript(
    'Sub' => \&Main
);
