#========================================================================================
# package      DE_EPAGES::Google::t::unit::UI_Shop
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for the Google view setup
#========================================================================================
package DE_EPAGES::Google::t::unit::UI_Shop;
use base DE_EPAGES::Core::API::Test;

use strict;

use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use DE_EPAGES::Test::Mock::MockifyServlet;

use Test::Mockify;
use Test::Mockify::Matcher qw ( HashRef Object String );
use Test::Mockify::Sut;
use Test::Mockify::Verify qw ( WasCalled );
use Test::More;


use HTTP::Response;

use constant GOOGLE_SITE_VERIFICATION_TOKEN => 'google96d1fc6bdb3dfb48.html';
use constant GOOGLE_SITE_VERIFICATION_TOKEN_FAIL => 'googlea1b2f3.html';
use constant GOOGLE_SITE_VERIFICATION_RESPONSE_SUCCESS => 'google-site-verification: '.GOOGLE_SITE_VERIFICATION_TOKEN;

sub test {
    my $self = shift;

    $self->test_UnitySFViewGoogleHTML_Success();
    $self->test_UnitySFViewGoogleHTML_Not_Found_When_Token_Not_Set();
    $self->test_UnitySFViewGoogleHTML_Not_Found_When_Wrong_Token_Requested();

    done_testing();
}

sub test_UnitySFViewGoogleHTML_Success {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $hAttributes = {
        'ID' => undef,
        'GoogleSiteVerificationToken' => GOOGLE_SITE_VERIFICATION_TOKEN,
        'GoogleSmartShoppingConfig' => undef,
    };

    my $Shop = $self->_createShop($hAttributes);
    my $hFormQueryParameters = {};
    my $HttpResponse = $self->_createHttpResponse();
    my $ServletForm = $self->_createServletForm($hFormQueryParameters);
    my $Servlet = $self->_createServlet($Shop, $ServletForm, $HttpResponse);
    my $ServerQueryString = 'URI='.GOOGLE_SITE_VERIFICATION_TOKEN;
    my $SUT = $self->_createSUT($ServerQueryString);

    $SUT->UnitySFViewGoogleHTML($Servlet);

    my $Response = $Servlet->response;

    ok( WasCalled($SUT, '_getServerVariables'), $SubTestName . ' - Verify that _getServerVariables was called' );
    is($Response->content, GOOGLE_SITE_VERIFICATION_RESPONSE_SUCCESS, $SubTestName . ' - Verify that correct site verification string is returned');

}


sub test_UnitySFViewGoogleHTML_Not_Found_When_Token_Not_Set {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $hAttributes = {
        'ID' => undef,
        'GoogleSiteVerificationToken' => undef,
        'GoogleSmartShoppingConfig' => undef,
    };

    my $Shop = $self->_createShop($hAttributes);
    my $hFormQueryParameters = {};
    my $HttpResponse = $self->_createHttpResponse();
    my $ServletForm = $self->_createServletForm($hFormQueryParameters);
    my $Servlet = $self->_createServlet($Shop, $ServletForm, $HttpResponse);
    my $ServerQueryString = 'URI='.GOOGLE_SITE_VERIFICATION_TOKEN;
    my $SUT = $self->_createSUT($ServerQueryString);

    $SUT->UnitySFViewGoogleHTML($Servlet);

    my $Response = $Servlet->response;

    ok( WasCalled($SUT, '_getServerVariables'), $SubTestName . ' - Verify that _getServerVariables was called' );
    is($Response->content, 'No google*.html available.', $SubTestName . ' - Verify that an error message is returned');

}

sub test_UnitySFViewGoogleHTML_Not_Found_When_Wrong_Token_Requested {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $hAttributes = {
        'ID' => undef,
        'GoogleSiteVerificationToken' => GOOGLE_SITE_VERIFICATION_TOKEN,
        'GoogleSmartShoppingConfig' => undef,
    };

    my $Shop = $self->_createShop($hAttributes);
    my $hFormQueryParameters = {};
    my $HttpResponse = $self->_createHttpResponse();
    my $ServletForm = $self->_createServletForm($hFormQueryParameters);
    my $Servlet = $self->_createServlet($Shop, $ServletForm, $HttpResponse);
    my $ServerQueryString = 'URI='.GOOGLE_SITE_VERIFICATION_TOKEN_FAIL;
    my $SUT = $self->_createSUT($ServerQueryString);

    $SUT->UnitySFViewGoogleHTML($Servlet);

    my $Response = $Servlet->response;

    ok( WasCalled($SUT, '_getServerVariables'), $SubTestName . ' - Verify that _getServerVariables was called' );
    is($Response->content, 'No google*.html available.', $SubTestName . ' - Verify that an error message is returned');

}

#----------------------------------------------------------------------------------------
# Mockery
#----------------------------------------------------------------------------------------
sub _createSUT {
    my $self = shift;
    my ($ServerQueryString) = @_;

    my $MockifySut = Test::Mockify::Sut->new('DE_EPAGES::Google::UI::Shop');
    if(defined($ServerQueryString)) {
        $MockifySut->mock('_getServerVariables')
            ->when(Object(), String('QUERY_STRING'))
            ->thenReturn($ServerQueryString);
    }

    return $MockifySut->getMockObject();
}

sub _createServlet {
    my $self = shift;
    my ($Shop, $ServletForm, $HttpResponse) = @_;

    my $MockifyServlet = DE_EPAGES::Test::Mock::MockifyServlet->new($Shop);
    $MockifyServlet->addMockWithReturnValue('form', $ServletForm);

    if(defined($HttpResponse)) {
        $MockifyServlet->addMockWithReturnValue('response', $HttpResponse);
    }

    return $MockifyServlet->getMockObject();
}

sub _createShop {
    my $self = shift;
    my ($hAttributes) = @_;

    my $MockifyShop = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Shop::API::Object::Shop', $hAttributes);
    my $Shop  = $MockifyShop->getMockObject();
    $Shop->set({'Site' => $Shop});

    return $Shop;
}

sub _createServletForm {
    my $self = shift;
    my ($hQueryParameters) = @_;

    my $Mockify = Test::Mockify->new( 'DE_EPAGES::Presentation::API::Form');
    $Mockify->mock('queryValues')->when()->thenReturn($hQueryParameters);
    $Mockify->mock('formValues')->when()->thenReturn($hQueryParameters);

    $Mockify->mock('executeFormError')->when()->thenReturnUndef();
    $Mockify->mock('executeFormError')->when(HashRef())->thenReturnUndef();
    $Mockify->mock('addFormError')->when(HashRef())->thenReturnUndef();
    $Mockify->mock('existsFormError')->when()->thenReturnUndef();

    return $Mockify->getMockObject();
}

sub _createHttpResponse {
    my $self = shift;

    return HTTP::Response->new();
}

__PACKAGE__->runScript();

