#========================================================================================
# package      DE_EPAGES::GoogleSmartShopping::API::GoogleAdsApiService
# state        public
#----------------------------------------------------------------------------------------
# description  Service to connect to Google Ads API services
#========================================================================================
package DE_EPAGES::GoogleSmartShopping::API::GoogleAdsApiService;

use base DE_EPAGES::GoogleSmartShopping::API::GoogleApiClient;
use DE_EPAGES::GoogleSmartShopping::API::Constants qw ( EPAGESJ_BASE_URI );
use DE_EPAGES::GoogleSmartShopping::API::Tools qw ( LogGoogleSmartShopping );
use DE_EPAGES::Core::API::PerlTools qw ( GetLocalTimeZone isValid );

use strict;

use constant CAMPAIGN_STATUS_STRINGS => {
    0 => 'PAUSED',
    1 => 'ENABLED'
};

use constant ADS_ACCOUNT_BILLING_STATUS_APPROVED => 'APPROVED';

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       DE_EPAGES::GoogleSmartShopping::API::GoogleAdsApiService->new( %args );
#----------------------------------------------------------------------------------------
# description  Instantiates a new GoogleAdsApiService.
#----------------------------------------------------------------------------------------
# return       GoogleAdsApiService | object | DE_EPAGES::GoogleSmartShopping::API::GoogleAdsApiService
#========================================================================================
sub new {
    my $class = shift;
    my ( $hSettings ) = @_;

    my $self = $class->SUPER::new($hSettings);

    return $self;
}

#========================================================================================
# function     getGoogleAdsAccounts
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hResponse = $self->getGoogleAdsAccounts();
#----------------------------------------------------------------------------------------
# description  Returns GoogleAdsAccounts array, or empty array if no Ads accounts found.
#----------------------------------------------------------------------------------------
# return       $aResponse | array with Google Ads Accounts | array.ref
#========================================================================================
sub getGoogleAdsAccounts {
    my $self = shift;

    my $Url = EPAGESJ_BASE_URI . 'accounts/ads/list';
    my $Headers = {'X-DeveloperToken' => $self->_getDeveloperToken()};

    my $aResponse;

    $aResponse =  $self->doGet($Url, $Headers);

    return $aResponse;
}

#========================================================================================
# function     createGoogleAdsSubAccount
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hResponse = $self->createGoogleAdsSubAccount($Email, $Name);
#----------------------------------------------------------------------------------------
# description  Creates GoogleAds Sub Account
#----------------------------------------------------------------------------------------
# return       $hResponse | hash with Google Ads Account Id and name | hash.ref
#========================================================================================
sub createGoogleAdsSubAccount {
    my $self = shift;

    my $ManagerAccountId = $self->_getAdsManagerAccountId();
    my $Url = EPAGESJ_BASE_URI . "accounts/ads/$ManagerAccountId/sub-accounts";
    my $hHeaders = {
        'AdminToken' => $self->_createAdminToken(),
        'X-DeveloperToken' => $self->_getDeveloperToken()
    };
    my $Name = $self->_getAdsAccountName();
    my $hPayload = {
        name =>  $Name,
        currencyCode => $self->_getCurrencyCode(),
        timeZone => $self->_getTimeZone(),
        email => $self->_getEmail(),
    };

    LogGoogleSmartShopping('createGoogleAdsAccount request: ', {
        Url => $Url,
        Payload => $hPayload,
    });

    my $hResponse = $self->doPost($Url, $hHeaders, $hPayload);

    LogGoogleSmartShopping('createGoogleAdsAccount response: ', {
        Response => $hResponse,
    });

    $self->{'GoogleSmartShoppingConfig'}->set({
        GoogleAdsAccountID => $hResponse->{'id'},
        GoogleAdsAccountName => $Name,
        IsAdsAccountSubAccount => 1
    });

    return $hResponse;

}

#========================================================================================
# function     unlinkGoogleAdsAccount
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hResponse = $self->unlinkGoogleAdsAccount();
#----------------------------------------------------------------------------------------
# description  Unlink GoogleAds
#========================================================================================
sub unlinkGoogleAdsAccount {
    my $self = shift;

    my $AdsManagerAccountId = $self->_getAdsManagerAccountId();
    my $AdsAccountId = $self->_getAdsAccountId();

    my $Url = EPAGESJ_BASE_URI . "accounts/ads/$AdsManagerAccountId/sub-accounts/$AdsAccountId";
    my $hHeaders = {
        'AdminToken' => $self->_createAdminToken(),
        'X-DeveloperToken' => $self->_getDeveloperToken()
    };

    LogGoogleSmartShopping('unlinkGoogleAdsAccount request: ', { Url => $Url });
    my $hResponse = $self->doDelete($Url, $hHeaders);
    LogGoogleSmartShopping('unlinkGoogleAdsAccount response: ', {
        Response => $hResponse,
    });
}

#========================================================================================
# function     createAndEnableGoogleAdsLink
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hResponse = $self->createAndEnableGoogleAdsLink();
#----------------------------------------------------------------------------------------
# description  Creates and enables a link between MerchantCenter Account and Ads Account
#               [wrapper for createGoogleAdsLink() and enableGoogleAdsLink()]
#========================================================================================
sub createAndEnableGoogleAdsLink {
    my $self = shift;

    $self->createGoogleAdsLink();
    $self->enableGoogleAdsLink();
}

#========================================================================================
# function     createGoogleAdsLink
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hResponse = $self->createGoogleAdsLink();
#----------------------------------------------------------------------------------------
# description  Creates a link between MerchantCenter Account and Ads Account
#========================================================================================
sub createGoogleAdsLink {
    my $self = shift;

    $self->loadGoogleSettings();

    my $hGoogleSmartShoppingConfig = $self->{'GoogleSmartShoppingConfig'}->get([qw(
        GoogleAdsAccountID
        MerchantCenterAccountID
        MerchantCenterMultiClientAccountID
    )]);

    my $AdsAccountId = $hGoogleSmartShoppingConfig->{'GoogleAdsAccountID'};
    my $MerchantCenterAccountID = $hGoogleSmartShoppingConfig->{'MerchantCenterAccountID'};
    my $MultiClientAccountID = $hGoogleSmartShoppingConfig->{'MerchantCenterMultiClientAccountID'} // $hGoogleSmartShoppingConfig->{'MerchantCenterAccountID'};

    my $Url = EPAGESJ_BASE_URI . "accounts/$MultiClientAccountID/sub-accounts/$MerchantCenterAccountID/ads-link";
    my $hHeaders = {};

    if ($MultiClientAccountID eq $self->_getEpagesMultiCLientAccountId()) {
        $hHeaders->{'AdminToken'} = $self->_createAdminToken();
    };

    my $hPayload = {
        adsAccountId =>  $AdsAccountId,
    };

    LogGoogleSmartShopping('createGoogleAdsLink request: ', {
        Url => $Url,
        Payload => $hPayload,
    });

    my $hResponse = $self->doPut($Url, $hHeaders, $hPayload);

    LogGoogleSmartShopping('createGoogleAdsLink response: ', {
        Response => $hResponse,
    });
}

#========================================================================================
# function     enableGoogleAdsLink
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hResponse = $self->enableGoogleAdsLink();
#----------------------------------------------------------------------------------------
# description  Enables Link between MerchantCenter Account and Ads Account
#               Only supports newly ePages created Accounts
#========================================================================================
sub enableGoogleAdsLink {
    my $self = shift;

    $self->loadGoogleSettings();

    my $AdsAccountId = $self->{'GoogleSmartShoppingConfig'}->get('GoogleAdsAccountID');
    my $MerchantCenterAccountID = $self->{'GoogleSmartShoppingConfig'}->get('MerchantCenterAccountID');
    my $AdsManagerAccountId = $self->_getAdsManagerAccountId();

    my $Url = EPAGESJ_BASE_URI . "accounts/ads/$AdsManagerAccountId/sub-accounts/$AdsAccountId/merchant-center-link/enable";
    my $hHeaders = {
        'AdminToken' => $self->_createAdminToken(),
        'X-DeveloperToken' => $self->_getDeveloperToken()
    };

    my $hPayload = {
        'merchantAccountId' => $MerchantCenterAccountID
    };

    LogGoogleSmartShopping('enableGoogleAdsLink request: ', {
        Url => $Url,
        Payload => $hPayload,
    });

    my $hResponse = $self->doPost($Url, $hHeaders, $hPayload);

    LogGoogleSmartShopping('enableGoogleAdsLink response: ', {
        Response => $hResponse,
    });
}

#========================================================================================
# function     createGoogleAdsCampaign
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hResponse = $self->createGoogleAdsCampaign($Email, $Name);
#----------------------------------------------------------------------------------------
# description  Creates GoogleAds Campaign
#               ads/{managerId}/sub-accounts/{subAccountId}/smart-shopping-campaigns"
#----------------------------------------------------------------------------------------
# input       $AdsSubAccountId | string with Ads Account Id | string
# input       $MerchantCenterAccountId | string with MerchantCenterAccountId | string
# input       $DailyBudget | string with DailyBudget | string
# input       $Name | string with campaign name | string
#========================================================================================
sub createGoogleAdsCampaign {
    my $self = shift;
    my ($hParams) = @_;

    my $ManagerAccountId = $self->_getAdsManagerAccountId();
    my $Url = sprintf("%saccounts/ads/%s/sub-accounts/%s/smart-shopping-campaigns",
        EPAGESJ_BASE_URI,
        $ManagerAccountId,
        $hParams->{'AdsSubAccountId'}
    ) ;
    my $hHeaders = {
        'AdminToken' => $self->_createAdminToken(),
        'X-DeveloperToken' => $self->_getDeveloperToken()
    };

    my $hPayload = {
        name =>  $hParams->{'AdsCampaignName'},
        salesCountry => $self->_getSalesCountry() ,
        dailyBudget => {
            name => $hParams->{'AdsCampaignName'}." Budget",
            amount =>  $hParams->{'DailyBudget'},
            currency => "EUR"
        },
        merchantCenterAccountId => $hParams->{'MerchantCenterAccountId'},
    };

    LogGoogleSmartShopping('createGoogleAdsCampaign  request: ', {
        Url => $Url,
        Payload => $hPayload,
    });

    my $hResponse = $self->doPost($Url, $hHeaders, $hPayload);

    LogGoogleSmartShopping('createGoogleAdsCampaign response: ', {
        Response => $hResponse,
    });

    if (defined($hResponse) && defined($hResponse->{'id'})) {
        $self->{'GoogleSmartShoppingConfig'}->set({'AdsCampaignId' => $hResponse->{'id'}});
    }

    return $hResponse;
}

#========================================================================================
# function     getSmartShoppingCampaigns
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aResponse = $self->getSmartShoppingCampaigns();
#----------------------------------------------------------------------------------------
# description  Returns SmartShoppingCampaigns array,
#               or empty array if no campaigns found.
#----------------------------------------------------------------------------------------
# return       $aResponse | array with Google Ads Campaigns | array.ref
#========================================================================================
sub getSmartShoppingCampaigns {
    my $self = shift;
    my ($hParams) = @_;

    my $ManagerAccountId = $self->_getAdsManagerAccountId();
    my $Url = sprintf("%saccounts/ads/%s/sub-accounts/%s/smart-shopping-campaigns",
        EPAGESJ_BASE_URI,
        $ManagerAccountId,
        $hParams->{'AdsSubAccountId'}
    ) ;
    my $hHeaders = {
        'AdminToken' => $self->_createAdminToken(),
        'X-DeveloperToken' => $self->_getDeveloperToken()
    };

    LogGoogleSmartShopping('getSmartShoppingCampaigns  request: ', {
        Url => $Url
    });

    my $aResponse = $self->doGet($Url, $hHeaders);

    LogGoogleSmartShopping('getSmartShoppingCampaigns response: ', {
        Response => $aResponse,
    });

    return $aResponse;
}

#========================================================================================
# function     updateGoogleAdsCampaign
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hResponse = $self->updateGoogleAdsCampaign($CampaignStatusEnabled);
#----------------------------------------------------------------------------------------
# description  Updates status of GoogleAds Campaign
#               ads/{managerId}/sub-accounts/{subAccountId}/smart-shopping-campaigns"
#----------------------------------------------------------------------------------------
# input       $hCampaignStatusUpdate | hash with campaign Id and status enabled/disabled value | hash.ref
#========================================================================================
sub updateGoogleAdsCampaign {
    my $self = shift;
    my ($hCampaignStatusUpdate) = @_;

    my $ManagerAdsAccountId = $self->_getAdsManagerAccountId();
    my $AdsAccountId = $self->_getAdsAccountId();
    my $Url = sprintf("%saccounts/ads/%s/sub-accounts/%s/smart-shopping-campaigns/%s",
        EPAGESJ_BASE_URI,
        $ManagerAdsAccountId,
        $AdsAccountId,
        $hCampaignStatusUpdate->{'CampaignId'}
    );
    my $hHeaders = {
        'AdminToken' => $self->_createAdminToken(),
        'X-DeveloperToken' => $self->_getDeveloperToken()
    };
    my $hPayload = { 'status' => CAMPAIGN_STATUS_STRINGS->{$hCampaignStatusUpdate->{'CampaignStatusEnabled'}}};

    LogGoogleSmartShopping('updateGoogleAdsCampaign  request: ', {
        Url => $Url,
        Payload => $hPayload,
    });
    $self->doPut($Url, $hHeaders, $hPayload);
}

#========================================================================================
# function     updateGoogleAdsCampaignBudget
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hResponse = $self->updateGoogleAdsCampaignBudget($hCampaignBudgetUpdate);
#----------------------------------------------------------------------------------------
# description  Updates budget of GoogleAds Campaign
#               ads/{managerId}/sub-accounts/{subAccountId}/smart-shopping-campaigns/{campapaignId}/daily-budget"
#----------------------------------------------------------------------------------------
# input       $hCampaignBudgetUpdate | hash with campaign Id and budget value | hash.ref
#========================================================================================
sub updateGoogleAdsCampaignBudget {
    my $self = shift;
    my ($hCampaignBudgetUpdate) = @_;

    my $ManagerAdsAccountId = $self->_getAdsManagerAccountId();
    my $AdsAccountId = $self->_getAdsAccountId();
    my $Url = sprintf("%saccounts/ads/%s/sub-accounts/%s/smart-shopping-campaigns/%s/daily-budget",
        EPAGESJ_BASE_URI,
        $ManagerAdsAccountId,
        $AdsAccountId,
        $hCampaignBudgetUpdate->{'AdsCampaignId'}
    );
    my $hHeaders = {
        'AdminToken' => $self->_createAdminToken(),
        'X-DeveloperToken' => $self->_getDeveloperToken()
    };
    my $hPayload = {
        'amount' => $hCampaignBudgetUpdate->{'DailyBudget'},
        'currency' => $hCampaignBudgetUpdate->{'DailyBudgetCurrency'},
    };

    LogGoogleSmartShopping('updateGoogleAdsCampaignBudget  request: ', {
        Url => $Url,
        Payload => $hPayload,
    });
    $self->doPut($Url, $hHeaders, $hPayload);
}

#========================================================================================
# function     deleteAdsCampaign
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hResponse = $self->deleteAdsCampaign($AdsCampaignId);
#----------------------------------------------------------------------------------------
# description  Deletes GoogleAds Campaign
#----------------------------------------------------------------------------------------
# input       $AdsCampaignId | campaign Id | string
#========================================================================================
sub deleteAdsCampaign {
    my $self = shift;
    my ($AdsCampaignId) = @_;

    my $ManagerAdsAccountId = $self->_getAdsManagerAccountId();
    my $AdsAccountId = $self->_getAdsAccountId();

    my $Url = sprintf("%saccounts/ads/%s/sub-accounts/%s/smart-shopping-campaigns/%s",
        EPAGESJ_BASE_URI,
        $ManagerAdsAccountId,
        $AdsAccountId,
        $AdsCampaignId
    );

    my $hHeaders = {
        'AdminToken' => $self->_createAdminToken(),
        'X-DeveloperToken' => $self->_getDeveloperToken()
    };

    LogGoogleSmartShopping('deleteAdsCampaign  request: ', {
        Url => $Url,
    });

    $self->doDelete($Url, $hHeaders);
}

#========================================================================================
# function     createGoogleConversionAction
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hResponse = $self->createGoogleConversionAction($AdsCampaignId);
#----------------------------------------------------------------------------------------
# description  Creates Google Ads Conversion Action
#               ads/{managerId}/sub-accounts/{subAccountId}/smart-shopping-campaigns/conversion-actions"
#----------------------------------------------------------------------------------------
# input       $AdsCampaignId | string with Ads Campaign Id | string
#========================================================================================
sub createGoogleConversionAction {
    my $self = shift;
    my ($AdsCampaignId) = @_;

    $self->loadGoogleSettings();

    my $DefaultLanguageID = $self->{'Shop'}->get('LanguageID');
    my $AdsCampaignName = $self->{'Shop'}->get('NameOrAlias', $DefaultLanguageID);

    my $AdsSubAccountId = $self->_getAdsAccountId();
    my $ManagerAccountId = $self->_getAdsManagerAccountId();

    my $Url = sprintf("%saccounts/ads/%s/sub-accounts/%s/smart-shopping-campaigns/conversion-actions",
        EPAGESJ_BASE_URI,
        $ManagerAccountId,
        $AdsSubAccountId
    );

    my $hHeaders = {
        'AdminToken' => $self->_createAdminToken(),
        'X-DeveloperToken' => $self->_getDeveloperToken()
    };

    my $hPayload = {
        name =>  "$AdsCampaignName - $AdsCampaignId Conversion Action",
    };

    LogGoogleSmartShopping('createGoogleConversionAction request: ', {
        Url => $Url,
        Payload => $hPayload,
    });

    my $hResponse = $self->doPost($Url, $hHeaders, $hPayload);

    LogGoogleSmartShopping('createGoogleConversionAction response: ', {
        Response => $hResponse,
    });

    $self->{'GoogleSmartShoppingConfig'}->set({
        'AdsCampaignConversionId' => $hResponse->{'awConversionId'},
        'AdsCampaignConversionLabel' => $hResponse->{'awConversionLabel'},
    });

    return $hResponse;
}

#========================================================================================
# function     createGoogleRemarketingAction
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hResponse = $self->createGoogleRemarketingAction();
#----------------------------------------------------------------------------------------
# description  Creates Google Ads Remarketing Action
#               ads/{managerId}/sub-accounts/{subAccountId}/smart-shopping-campaigns/remarketing-actions"
#----------------------------------------------------------------------------------------
# input       $AdsCampaignId | string with Ads Campaign Id | string
#========================================================================================
sub createGoogleRemarketingAction {
    my $self = shift;
    my ($AdsCampaignId) = @_;

    $self->loadGoogleSettings();

    my $DefaultLanguageID = $self->{'Shop'}->get('LanguageID');
    my $AdsCampaignName = $self->{'Shop'}->get('NameOrAlias', $DefaultLanguageID);

    my $ManagerAccountId = $self->_getAdsManagerAccountId();
    my $AdsSubAccountId = $self->_getAdsAccountId();

    my $Url = sprintf("%saccounts/ads/%s/sub-accounts/%s/smart-shopping-campaigns/remarketing-actions",
        EPAGESJ_BASE_URI,
        $ManagerAccountId,
        $AdsSubAccountId
    ) ;

    my $hHeaders = {
        'AdminToken' => $self->_createAdminToken(),
        'X-DeveloperToken' => $self->_getDeveloperToken()
    };

    my $hPayload = {
        name => "$AdsCampaignName - $AdsCampaignId Remarketing Action",
    };

    LogGoogleSmartShopping('createGoogleRemarketingAction request: ', {
        Url => $Url,
        Payload => $hPayload,
    });

    my $hResponse = $self->doPost($Url, $hHeaders, $hPayload);

    LogGoogleSmartShopping('createGoogleRemarketingAction response: ', {
        Response => $hResponse,
    });

    $self->{'GoogleSmartShoppingConfig'}->set({
        'AdsCampaignRemarketingId' => $hResponse->{'awRemarketingId'},
    });

    return $hResponse;
}

#========================================================================================
# function     getAdsAccountBillingStatus
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aResponse = $self->getAdsAccountBillingStatus();
#----------------------------------------------------------------------------------------
# description  Returns Ads Account BillingStatus.
#----------------------------------------------------------------------------------------
# return       $hResponse | hash with BillingStatusHash | hash.ref
#========================================================================================
sub getAdsAccountBillingStatus {
    my $self = shift;

    my $Url = sprintf("%saccounts/ads/%s/sub-accounts/%s/billing-setup",
        EPAGESJ_BASE_URI,
        $self->_getAdsManagerAccountId(),
        $self->_getAdsAccountId()
    );

    my $hHeaders = {
        'AdminToken' => $self->_createAdminToken(),
        'X-DeveloperToken' => $self->_getDeveloperToken()
    };

    LogGoogleSmartShopping('getAdsAccountBillingStatus  request: ', {
        Url => $Url
    });

    my $hResponse = $self->doGet($Url, $hHeaders);

    LogGoogleSmartShopping('getAdsAccountBillingStatus response: ', {
        Response => $hResponse,
    });

    return $hResponse->{'status'};
}


#========================================================================================
# function     getCampaignShoppingPerformance
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hResponse = $self->getCampaignShoppingPerformance();
#----------------------------------------------------------------------------------------
# description  Returns a hash with total number of campaign products and the products
#               array with performance metrics.
#----------------------------------------------------------------------------------------
# input        $CampaignId | campaign ID number | string
# input        $Page | page number from paginated request | integer
# input        $ResultsPerPage | size of the page | integer
# return       $hResponse | hash with campaign products performance | hash.ref
#========================================================================================
sub getCampaignShoppingPerformance {
    my $self = shift;
    my ($CampaignId, $Page, $ResultsPerPage, $OrderBy, $OrderDesc) = @_;

    my $ManagerAccountId = $self->_getAdsManagerAccountId();
    my $AdsAccountId = $self->_getAdsAccountId();
    my $Url = sprintf("%saccounts/ads/%s/sub-accounts/%s/smart-shopping-campaigns/%s/shopping-performance",
        EPAGESJ_BASE_URI,
        $ManagerAccountId,
        $AdsAccountId,
        $CampaignId
    ) ;
    my $hHeaders = {
        'AdminToken' => $self->_createAdminToken(),
        'X-DeveloperToken' => $self->_getDeveloperToken()
    };
    my $hQuery = {
        'page'           => $Page,
        'resultsPerPage' => $ResultsPerPage
    };
    if (isValid($OrderBy)) {
        $hQuery->{'orderBy'} = $OrderBy;
        $hQuery->{'orderDesc'} = $OrderDesc ? 'true' : 'false';
    }

    LogGoogleSmartShopping('getCampaignShoppingPerformance request: ', {
        Url   => $Url,
        Query => $hQuery
    });

    my $aResponse = $self->doGet($Url, $hHeaders, $hQuery);

    LogGoogleSmartShopping('getCampaignShoppingPerformance response: ', {
        Response => $aResponse,
    });

    return $aResponse;
}

#========================================================================================
# function     isAdsAccountBillingApproved
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aResponse = $self->isAdsAccountBillingApproved();
#----------------------------------------------------------------------------------------
# description  Returns Ads Account BillingStatus.
#----------------------------------------------------------------------------------------
# return       $IsApproved | bool approved | bool
#========================================================================================
sub isAdsAccountBillingApproved {
    my $self = shift;

    my $Status = $self->getAdsAccountBillingStatus();

    return ($Status eq ADS_ACCOUNT_BILLING_STATUS_APPROVED);
}

#----------------------------------------------------------------------------------------
sub _getAdsAccountName {
    my $self = shift;

    my $Shop = $self->{'Shop'};
    my $DefaultLanguageID = $Shop->get('LanguageID');
    return $Shop->get('NameOrAlias', $DefaultLanguageID);
}

#----------------------------------------------------------------------------------------
sub _getEmail {
    my $self = shift;

    $self->loadGoogleSettings();
    my $GoogleSmartShoppingConfig = $self->{'GoogleSmartShoppingConfig'};
    return $GoogleSmartShoppingConfig->get('AccountEmail');
}

#----------------------------------------------------------------------------------------
sub _getSalesCountry {
    my $self = shift;

    my $Shop = $self->{'Shop'};
    return $Shop->get('Address')->get('Country')->{'Code2'};
}

#----------------------------------------------------------------------------------------
sub _createAdminToken {
    my $self = shift;

    $self->loadGoogleSettings();
    return $self->GetAccessToken($self->{'GoogleSmartShopping_AdminRefreshToken'});
}

#----------------------------------------------------------------------------------------
sub _getCurrencyCode {
    my $self = shift;

    my $Shop =$self->{'Shop'};
    return $Shop->get('CurrencyID');
}

#----------------------------------------------------------------------------------------
sub _getTimeZone {
    my $self = shift;

    return GetLocalTimeZone();
}

#----------------------------------------------------------------------------------------
sub _getDeveloperToken {
    my $self = shift;

    $self->loadGoogleSettings();
    return $self->{'GoogleSmartShopping_AdsDeveloperToken'};
}

#----------------------------------------------------------------------------------------
sub _getAdsManagerAccountId {
    my $self = shift;

    $self->loadGoogleSettings();
    return $self->{'GoogleSmartShopping_AdsManagerAccountId'};
}

#----------------------------------------------------------------------------------------
sub _getAdsAccountId {
    my $self = shift;

    $self->loadGoogleSettings();
    my $GoogleSmartShoppingConfig = $self->{'GoogleSmartShoppingConfig'};
    return $GoogleSmartShoppingConfig->get('GoogleAdsAccountID');
}

#----------------------------------------------------------------------------------------
sub _getEpagesMultiCLientAccountId {
    my $self = shift;

    $self->loadGoogleSettings();
    return $self->{'GoogleSmartShopping_MerchantCenterMultiClientAccountID'};
}

1;