#========================================================================================
# package      DE_EPAGES::GoogleSmartShopping::API::GoogleApiService
# state        public
#----------------------------------------------------------------------------------------
# description  Service to (directly) connect to Google API services
#========================================================================================
package DE_EPAGES::GoogleSmartShopping::API::GoogleApiService;

use base DE_EPAGES::GoogleSmartShopping::API::GoogleApiClient;

use Encode qw ( encode );
use HTTP::Request;
use HTTP::Status qw ( is_success HTTP_OK HTTP_INTERNAL_SERVER_ERROR );
use LWP::UserAgent;
use Mojo::JWT;

use JSON::MaybeXS qw ( decode_json encode_json );
use DE_EPAGES::Core::API::PerlTools qw ( GetCurrentDateTime );
use DE_EPAGES::WebService::API::Util qw (ReadJSONBoolean);

use DE_EPAGES::Core::API::Error qw ( Error ExistsError GetError );
use DE_EPAGES::Object::API::Factory qw (LoadRootObject);
use DE_EPAGES::GoogleSmartShopping::API::Constants qw (
    EPAGESJ_BASE_URI
    ERROR_TOKEN_EXPIRED_OR_REVOKED
    ERROR_UNKNOWN
);
use DE_EPAGES::GoogleSmartShopping::API::Tools qw ( LogGoogleSmartShopping GetErrorCode );

use strict;

use constant GOOGLE_OPENID_USERINFO_URL => 'https://openidconnect.googleapis.com/v1/userinfo';

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       DE_EPAGES::GoogleSmartShopping::API::GoogleApiService->new( %args );
#----------------------------------------------------------------------------------------
# description  Instantiates a new GoogleApiService.
#----------------------------------------------------------------------------------------
# return       GoogleApiService | object | DE_EPAGES::GoogleSmartShopping::API::GoogleApiService
#========================================================================================
sub new {
    my $class = shift;
    my ( $hSettings ) = @_;

    my $self = $class->SUPER::new($hSettings);

    return $self;
}

#========================================================================================
# function     getAndSaveOAuthTokens
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->getAndSaveOAuthTokens($OAuthCode);
#----------------------------------------------------------------------------------------
# description  Given an OAuth code it gets Google API access tokens and saves these in
#               the GoogleSmartShoppingConfig of the Shop, creating a new ShoppingConfig
#               if necessary.
#----------------------------------------------------------------------------------------
# input        $OAuthCode | authorization code string | string
# return       undef | may throw an UNKNOWN_ERROR with code HTTP_INTERNAL_SERVER_ERROR
#========================================================================================
sub getAndSaveOAuthTokens {
    my $self = shift;
    my ($OAuthCode) = @_;

    my $Shop = $self->{'Shop'};
    my $hSystemSettings = LoadRootObject()->get([qw(
        GoogleSmartShopping_AuthTokenURL
        GoogleSmartShopping_AppClientID
        GoogleSmartShopping_AppClientSecret
        GoogleSmartShopping_AppRedirectURL
        GoogleSmartShopping_GoogleAppURL
    )]);

    my $RedirectURL = $hSystemSettings->{'GoogleSmartShopping_GoogleAppURL'};

    my $Header = [ 'Content-Type' => 'application/x-www-form-urlencoded' ];
    my $RequestContent = sprintf(
        'code=%s&client_id=%s&client_secret=%s&grant_type=%s&redirect_uri=%s',
        $OAuthCode,
        $hSystemSettings->{'GoogleSmartShopping_AppClientID'},
        $hSystemSettings->{'GoogleSmartShopping_AppClientSecret'},
        'authorization_code',
        $RedirectURL
    );

    my $Url = $hSystemSettings->{'GoogleSmartShopping_AuthTokenURL'};

    LogGoogleSmartShopping("Google OAuth token exchange request: ", {
        'Url'            => $Url,
        'RequestContent' => $RequestContent
    });
    my $Request = HTTP::Request->new('POST', $Url, $Header, encode("UTF-8", $RequestContent));
    my $UserAgent = LWP::UserAgent->new();
    my $Response = $UserAgent->request($Request);
    my $Status = $Response->code();
    my $hResponse = decode_json($Response->decoded_content());
    LogGoogleSmartShopping("Google OAuth token exchange response ($Status): ", $hResponse);

    if (is_success($Status) && defined $hResponse->{'access_token'}) {
        unless($Shop->existsChild('GoogleSmartShoppingConfig')) {
            $Shop->insertChild('GoogleSmartShoppingConfig', { 'Alias' => 'GoogleSmartShoppingConfig' });
        }
        my $GoogleSmartShoppingConfig = $Shop->get('GoogleSmartShoppingConfig');
        $GoogleSmartShoppingConfig->set({ 'AccessToken' => $hResponse->{'access_token'} });

        if (defined $hResponse->{'refresh_token'}) {
            $GoogleSmartShoppingConfig->set({ 'RefreshToken' => $hResponse->{'refresh_token'} });
        }
    } else {
        Error(ERROR_UNKNOWN, {
            'Status' => $Status != HTTP_OK ? $Status : HTTP_INTERNAL_SERVER_ERROR,
            'Response' => $hResponse
        });
    }
}

#========================================================================================
# function     revokeAccessToken
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hResponse = $self->revokeAccessToken($AccessToken);
#----------------------------------------------------------------------------------------
# description  Revoke $AccessToken
#----------------------------------------------------------------------------------------
# input        $AccessToken | access token string | string
# return       $hResponse | hash with response | hashref
#========================================================================================
sub revokeAccessToken {
    my $self = shift;
    my ($AccessToken) = @_;

    my $Url = "https://oauth2.googleapis.com/revoke?token=$AccessToken";
    my $Header = [ 'Accept'  => 'application/json',
        'Content-type' => "application/x-www-form-urlencoded"];
    my $Data = {token => $AccessToken};
    my $Encoded_data = encode_json($Data);

    LogGoogleSmartShopping("Revoking access token ", $Data);

    my $Request = HTTP::Request->new('POST', $Url, $Header, $Encoded_data);
    my $UserAgent = LWP::UserAgent->new();
    my $Response = $UserAgent->request($Request);
    my $Status = $Response->code();

    if (is_success($Status)) {
        LogGoogleSmartShopping("Revoked access token successfully", {});
        return decode_json($Response->decoded_content());
    }

    LogGoogleSmartShopping('Revoking AcessToken failed', $Response->decoded_content());
    Error('Revoking AcessToken failed', {
        'Status' => $Status,
        'Response' => $Response->decoded_content()
    });
}

#========================================================================================
# function     getUserInfo
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hResponse = $self->getAndSaveUserInfo($AccessToken);
#----------------------------------------------------------------------------------------
# description  Given an $AccessToken it gets email of the connected account
#----------------------------------------------------------------------------------------
# input        $AccessToken | access token string | string
# return       $hResponse | hash with response | hashref
#========================================================================================
sub getAndSaveUserInfo {
    my $self = shift;
    my $Shop = $self->{'Shop'};
    unless ($Shop && $Shop->existsChild('GoogleSmartShoppingConfig')) {
        return;
    }
    my $GoogleSmartShoppingConfig = $Shop->get('GoogleSmartShoppingConfig');

    my $Url = GOOGLE_OPENID_USERINFO_URL;
    my $Header = [ 'Accept'  => 'application/json' ];

    my $hUserInfo;
    eval {
        $hUserInfo = $self->doGet($Url, $Header, undef);
    };
    if (ExistsError()) {
        my $Error = GetError();
        LogGoogleSmartShopping('Error getting account user information: ', $Error->message());
        my $ErrorCode = GetErrorCode($Error);
        if ($ErrorCode ne ERROR_TOKEN_EXPIRED_OR_REVOKED) {
            Error($Error);
        }
        $hUserInfo = { 'is_token_expired' => 1 };
        $hUserInfo->{'email'} = $GoogleSmartShoppingConfig->get('AccountEmail');
        if (!defined($hUserInfo->{'email'})) {
            Error($Error);
        }
    } else {
        $GoogleSmartShoppingConfig->set({'AccountEmail' => $hUserInfo->{'email'}});
    }
    return $hUserInfo;
}

#========================================================================================
# function     getUserAccountInfo
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hResponse = $self->getUserAccountInfo();
#----------------------------------------------------------------------------------------
# description  Given an OAuth code it gets Google API access tokens.
#----------------------------------------------------------------------------------------
# return       $hResponse | hash with response | hashref
#========================================================================================
sub getUserAccountInfo {
    my $self = shift;

    my $Url = EPAGESJ_BASE_URI . 'accounts/user-account-info';

    return $self->doGet($Url);
}

#========================================================================================
# function     getMerchantCenterAccounts
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aResponse = $self->getMerchantCenterAccounts();
#----------------------------------------------------------------------------------------
# description  Returns MerchantCenterAccounts array, or empty array if no MCA found.
#----------------------------------------------------------------------------------------
# return       $aResponse | array with Merchant Center Accounts | array.ref
#========================================================================================
sub getMerchantCenterAccounts {
    my $self = shift;

    my $Url = EPAGESJ_BASE_URI . 'accounts/merchant-center-accounts';

    return $self->doGet($Url);
}

#========================================================================================
# function     claimWebsite
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hResponse = $self->claimWebsite();
#----------------------------------------------------------------------------------------
# description  Claims a website using the currently configured MerchantCenterAccount
#========================================================================================
sub claimWebsite {
    my $self = shift;

    $self->loadGoogleSettings();

    my $Url = EPAGESJ_BASE_URI.'accounts/claim-website';

    my $hGoogleSmartShoppingConfig = $self->{'GoogleSmartShoppingConfig'}->get([qw(
        MerchantCenterAccountID
        MerchantCenterMultiClientAccountID
        IsMerchantCenterSubAccount
    )]);

    my $MerchantAccountID = $hGoogleSmartShoppingConfig->{'MerchantCenterAccountID'};
    my $MultiClientAccountID = $hGoogleSmartShoppingConfig->{'MerchantCenterMultiClientAccountID'} // $hGoogleSmartShoppingConfig->{'MerchantCenterAccountID'};

    my $hPayload = {
        merchantAccountId => $MerchantAccountID,
        managerAccountId => $MultiClientAccountID,
    };

    my $hHeaders = {};

    LogGoogleSmartShopping('claimWebsite request: ', {
        Url => $Url,
        Payload => $hPayload,
    });

    my $hResponse = $self->doPost($Url, $hHeaders, $hPayload);

    LogGoogleSmartShopping('claimWebsite success', {
        Response => $hResponse,
    });

    return;
}

#========================================================================================
# function     createMerchantCenterSubAccount
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hResponse = $self->createMerchantCenterSubAccount($ConnectedGoogleAccountEmail, $HasAdultContent);
#----------------------------------------------------------------------------------------
# description  Creates a new SubAccount in ePages MultiClientAccount
#----------------------------------------------------------------------------------------
# return       $hResponse | hash with created Merchant Center Account | hashref
#========================================================================================
sub createMerchantCenterSubAccount {
    my $self = shift;
    my ($HasAdultContent) = @_;

    $self->loadGoogleSettings();

    my $Url = sprintf(EPAGESJ_BASE_URI.'accounts/%s/sub-accounts',
        $self->{'GoogleSmartShopping_MerchantCenterMultiClientAccountID'},
    );

    my $DefaultLanguageID = $self->{'Shop'}->get('LanguageID');
    my $ShopName = $self->{'Shop'}->get('NameOrAlias', $DefaultLanguageID);
    my $UnityWebsiteUrl = $self->{'Shop'}->get('WebUrlUnitySSL');
    my $ConnectedGoogleAccountEmail = $self->{'GoogleSmartShoppingConfig'}->get('AccountEmail');

    my $hPayload = {
        name => $ShopName,
        websiteUrl => $UnityWebsiteUrl,
        adultContent => $HasAdultContent ? 'true' : 'false',
        emails => [$ConnectedGoogleAccountEmail],
    };

    my $hHeaders = {'AdminToken' => $self->_createAdminToken()};

    LogGoogleSmartShopping('createMerchantCenterSubAccount request: ', {
        Url => $Url,
        Payload => $hPayload,
    });

    my $hResponse = $self->doPost($Url, $hHeaders, $hPayload);

    LogGoogleSmartShopping('createMerchantCenterSubAccount response: ', {
        Response => $hResponse,
    });

    $hResponse->{adultContent} = ReadJSONBoolean($hResponse->{adultContent});

    $self->{'GoogleSmartShoppingConfig'}->set({
        MerchantCenterSubAccountCreationTime  => GetCurrentDateTime(),
        MerchantCenterAccountName => $ShopName,
        IsMerchantCenterSubAccount => 1,
        MerchantCenterAccountID => $hResponse->{'id'},
        MerchantCenterMultiClientAccountID => $self->{'GoogleSmartShopping_MerchantCenterMultiClientAccountID'},
    });

    return $hResponse;
}


#========================================================================================
# function     deleteMerchantCenterSubAccount
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hResponse = $self->deleteMerchantCenterSubAccount($SubAccountId);
#----------------------------------------------------------------------------------------
# description  Deletes a SubAccount in ePages MultiClientAccount
#----------------------------------------------------------------------------------------
# input       $SubAccountId | string with Merchant Center Account Id | string
#========================================================================================
sub deleteMerchantCenterSubAccount {
    my $self = shift;
    my ($SubAccountId) = @_;

    $self->loadGoogleSettings();

    my $Url = sprintf(EPAGESJ_BASE_URI.'accounts/%s/sub-accounts/%s',
        $self->{'GoogleSmartShopping_MerchantCenterMultiClientAccountID'},
        $SubAccountId,
    );

    my $hHeaders = {'AdminToken' => $self->_createAdminToken()};

    LogGoogleSmartShopping('deleteMerchantCenterSubAccount request: ', {
        Url => $Url,
    });

    $self->doDelete($Url, $hHeaders);

    LogGoogleSmartShopping('deleteMerchantCenterSubAccount success', {});

    return;
}

#========================================================================================
# function     updateMerchantCenterShopUrl
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hResponse = $self->updateMerchantCenterShopUrl($ConnectedGoogleAccountEmail, $HasAdultContent);
#----------------------------------------------------------------------------------------
# description  Creates a new SubAccount in ePages MultiClientAccount
#----------------------------------------------------------------------------------------
# return       $hResponse | hash with created Merchant Center Account | hashref
#========================================================================================
sub updateMerchantCenterShopUrl {
    my $self = shift;
    my ($ShopWebSiteUrl) = @_;

    $self->loadGoogleSettings();

    my $GoogleSmartShoppingConfig = $self->{'GoogleSmartShoppingConfig'};
    my $MerchantCenterAccountID = $GoogleSmartShoppingConfig->get('MerchantCenterAccountID');
    # Merchant center account should have admin access role
    my $MerchantCenterMultiClientAccountID = $MerchantCenterAccountID;
    my $hHeaders = {};
    my $trailPathUrl = sprintf('accounts/%s/sub-accounts/%s', $MerchantCenterMultiClientAccountID , $MerchantCenterAccountID );
    my $Url =  EPAGESJ_BASE_URI . $trailPathUrl;

    my $hUserInfo = $self->doPut($Url, $hHeaders, { 'websiteUrl' => $ShopWebSiteUrl });

    LogGoogleSmartShopping("Google Update Merchant Center Website URL response: ", $hUserInfo);

    return;
}

sub _createAdminToken {
    my $self = shift;

    $self->loadGoogleSettings();
    return $self->GetAccessToken($self->{'GoogleSmartShopping_AdminRefreshToken'});
}

#========================================================================================
# function     getSiteVerificationToken
# state        public
#----------------------------------------------------------------------------------------
# syntax       $response = $self->getSiteVerificationToken($websiteURL);
#----------------------------------------------------------------------------------------
# description  Returns site verification token
#----------------------------------------------------------------------------------------
# input        $WebsiteURL | website url string | string
# return       $hResponse | hash with token  | hash.ref
#========================================================================================
sub getSiteVerificationToken {
    my $self = shift;
    my ($WebsiteURL) = @_;

    my $Url = EPAGESJ_BASE_URI . 'accounts/site-verification-token';

    my $hHeaders = {};
    my $hRequestContent = {
        'websiteUrl' =>  $WebsiteURL
    };

    return $self->doPost($Url, $hHeaders, $hRequestContent);
}

#========================================================================================
# function     triggerSiteVerification
# state        public
#----------------------------------------------------------------------------------------
# syntax       $response = $self->triggerSiteVerification($WebsiteURL, $Token);
#----------------------------------------------------------------------------------------
# description  Given an URL for the shop website, this method makes a call to Google,
#               triggering a HTTP call with a token in the path defined by Google. This
#               token should have been obtained by getSiteVerificationToken() before
#               calling this method.
#----------------------------------------------------------------------------------------
# input        $WebsiteURL | website url string | string
#----------------------------------------------------------------------------------------
# return       Undef unless there was an error.
#========================================================================================
sub triggerSiteVerification {
    my $self = shift;
    my ($WebsiteURL, $VerificationToken) = @_;

    my $URL = EPAGESJ_BASE_URI . 'accounts/site-verification';

    my $hHeaders = {};
    my $hRequestContent = {
        'websiteUrl' =>  $WebsiteURL
    };

    return $self->doPost($URL, $hHeaders, $hRequestContent);
}

#========================================================================================
# function     getMerchantCenterAccountStatuses
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hResponse = $self->getMerchantCenterAccountStatuses($MultiClientAccountId, $MerchantId);
#----------------------------------------------------------------------------------------
# description  Returns a hash with AccountStatuses.
#----------------------------------------------------------------------------------------
# input        $MultiClientAccountId | multiClientAccount Id | string
#               $MerchantId | account Id  | string
# return       $hResponse | hash with response | ref.hash
#========================================================================================
sub getMerchantCenterAccountStatuses {
    my $self = shift;
    $self->loadGoogleSettings();
    my $hGoogleSmartShoppingConfig = $self->{'GoogleSmartShoppingConfig'}->get([
        'MerchantCenterAccountID',
        'MerchantCenterMultiClientAccountID',
    ]);


    my $MerchantId = $hGoogleSmartShoppingConfig->{'MerchantCenterAccountID'};
    my $MultiClientAccountId = $hGoogleSmartShoppingConfig->{'MerchantCenterMultiClientAccountID'} // $MerchantId;

    my $hHeaders;
    if ($MultiClientAccountId ne $MerchantId) {
        $hHeaders->{'AdminToken'} = $self->_createAdminToken();
    }

    my $Url = EPAGESJ_BASE_URI . "accounts/$MultiClientAccountId/account-statuses/$MerchantId";

    LogGoogleSmartShopping('getMerchantCenterAccountStatuses request: ', {
        Url => $Url,
    });

    my $hResponse = $self->doGet($Url, $hHeaders);

    LogGoogleSmartShopping('getMerchantCenterAccountStatuses response: ', {
        Response => $hResponse,
    });

    return $hResponse;
}

#========================================================================================
# function     getRatedProductsStatistics
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hResponse = $self->getRatedProductsStatistics($MultiClientAccountId, $MerchantCenterAccountID);
#----------------------------------------------------------------------------------------
# description  Returns a hash with Product Rating.
#----------------------------------------------------------------------------------------
# input        $MultiClientAccountId | multiClientAccount Id | string
#               $MerchantCenterAccountID | merchant center account Id  | string
# return       $hResponse | hash with response | ref.hash
#========================================================================================
sub getRatedProductsStatistics {
    my $self = shift;

    my $hResult = $self->getMerchantCenterAccountStatuses();

    my $hRatedProductsStatistics = {
        HasProductsUploaded => 0,
        HasWarnings         => 0,
        HasActiveProducts   => 0,
        TooManyPending      => 0,
        TooManyDisapproved  => 0,
        NotEnoughActive     => 0,
        active              => 0,
        pending             => 0,
        disapproved         => 0,
    };

    if (!defined ($hResult->{'products'} && $hResult->{'products'}->[0] && $hResult->{'products'}->[0]->{'statistics'})) {
        LogGoogleSmartShopping('Rated Google Product Statistics early return: ', {Return => $hRatedProductsStatistics });
        return $hRatedProductsStatistics;
    }

    my $hProductsStatistics = $self->_getLocalizedStatistics( $hResult->{'products'});
    if (!defined $hProductsStatistics) {
        LogGoogleSmartShopping('Rated Google Product Statistics none found: ', {Return => $hProductsStatistics });
        return $hRatedProductsStatistics;
    }
    my $ProductCount = $hProductsStatistics->{'active'} + $hProductsStatistics->{'disapproved'} + $hProductsStatistics->{'expiring'} + $hProductsStatistics->{'pending'};
    LogGoogleSmartShopping('Rated Google Product Statistics  total product count: ', { totalCount => $ProductCount });

    if ($ProductCount <= 0) {
        return $hRatedProductsStatistics;
    }
    $hRatedProductsStatistics->{'active'} = $hProductsStatistics->{'active'};
    $hRatedProductsStatistics->{'pending'} = $hProductsStatistics->{'pending'};
    $hRatedProductsStatistics->{'disapproved'} = $hProductsStatistics->{'disapproved'};

    $hRatedProductsStatistics->{'active'} = $hProductsStatistics->{'active'};
    $hRatedProductsStatistics->{'disapproved'} = $hProductsStatistics->{'disapproved'};
    $hRatedProductsStatistics->{'HasProductsUploaded'} = 1;
    if ($hProductsStatistics->{'active'} > 0) {
        $hRatedProductsStatistics->{'HasActiveProducts'} = 1;
    }

    # if pending > 50%
    my $hCriteriaPending = {
        totalProductCount                => $ProductCount,
        valueUnderTest                   => $hProductsStatistics->{'pending'},
        thresholdPercentage              => 51,
    };

    # if disapproved > 50%
    my $hCriteriaDisapproved = {
        totalProductCount                => $ProductCount,
        valueUnderTest                   => $hProductsStatistics->{'disapproved'},
        thresholdPercentage              => 51,
    };

    # if active >= 50% && totalActive < 10
    my $hCriteriaActive = {
        totalProductCount                => $ProductCount,
        valueUnderTest                   => $hProductsStatistics->{'active'},
        thresholdPercentage              => 50,
        thresholdAbsoluteCount           => 10,
    };

    if ($self->_isPercentageHigherThanThreshold($hCriteriaPending)) {
        $hRatedProductsStatistics->{'TooManyPending'} = 1;
        $hRatedProductsStatistics->{'HasWarnings'} = 1;
    } elsif ($self->_isPercentageHigherThanThreshold($hCriteriaDisapproved)) {
        $hRatedProductsStatistics->{'TooManyDisapproved'} = 1;
        $hRatedProductsStatistics->{'HasWarnings'} = 1;
    } elsif ($self->_isPercentageEqualToOrHigherThanThreshold($hCriteriaActive) && $self->_isAbsoluteLowerThanThreshold($hCriteriaActive)) {
        $hRatedProductsStatistics->{'NotEnoughActive'} = 1;
        $hRatedProductsStatistics->{'HasWarnings'} = 1;
    }
    LogGoogleSmartShopping('Rated Google Product Statistics response: ', $hRatedProductsStatistics);
    return $hRatedProductsStatistics;
}

sub _getLocalizedStatistics {
    my $self = shift;
    my ($aProductsFolders) = @_;


    my $CountryCode = $self->{'Shop'}->get('Address')->get('Country')->{'Code2'};

    for my $hProductsFolder (@{$aProductsFolders}) {
        if (($hProductsFolder->{'channel'} eq 'online') &&
            ($hProductsFolder->{'country'} eq $CountryCode) &&
            ($hProductsFolder->{'destination'} eq 'Shopping')
        ) {
            return $hProductsFolder->{'statistics'};
        }
    }

    return undef;
}

#----------------------------------------------------------------------------------------
sub _isPercentageHigherThanThreshold {
    my $self = shift;
    my ($hAssessmentCriteria) = @_;
    my $ProductsPercentage = $hAssessmentCriteria->{'valueUnderTest'} / $hAssessmentCriteria->{'totalProductCount'} * 100;

    if ($hAssessmentCriteria->{'thresholdPercentage'} < $ProductsPercentage) {
        return 1;
    }
    return 0;
}

#----------------------------------------------------------------------------------------
sub _isPercentageEqualToOrHigherThanThreshold {
    my $self = shift;
    my ($hAssessmentCriteria) = @_;
    my $ProductsPercentage = $hAssessmentCriteria->{'valueUnderTest'} / $hAssessmentCriteria->{'totalProductCount'} * 100;

    if ($hAssessmentCriteria->{'thresholdPercentage'} <= $ProductsPercentage) {
        return 1;
    }
    return 0;
}
sub _isAbsoluteLowerThanThreshold {
    my $self = shift;
    my ($hAssessmentCriteria) = @_;
    if ($hAssessmentCriteria->{'valueUnderTest'} < $hAssessmentCriteria->{'thresholdAbsoluteCount'}) {
        return 1;
    }
    return 0;
}

#========================================================================================
# function     requestMerchantCenterPhoneVerification
# state        public
#----------------------------------------------------------------------------------------
# syntax       $VerificationId = $self->requestMerchantCenterPhoneVerification( $hPhoneInfo );
#----------------------------------------------------------------------------------------
# description  Sends merchant center phone number information (phone number and its localization
#               codes) to trigger an SMS message from Google for ownership confirmation.
#               Returns a verification id to be sent among with the phone's SMS code.
#----------------------------------------------------------------------------------------
# input        $hPhoneInfo | hash with phone number and country code | ref.hash
# return       $VerificationId | verification id of sent request | string
#========================================================================================
sub requestMerchantCenterPhoneVerification {
    my $self = shift;
    my ($hPhoneInfo) = @_;

    $self->loadGoogleSettings();

    my $GoogleSmartShoppingConfig = $self->{'GoogleSmartShoppingConfig'};
    my $MerchantCenterAccountID = $GoogleSmartShoppingConfig->get('MerchantCenterAccountID');
    my $MerchantCenterMultiClientAccountID =
        $GoogleSmartShoppingConfig->get('MerchantCenterMultiClientAccountID') // $MerchantCenterAccountID;

    my $trailPathUrl = sprintf(
        'accounts/%s/sub-accounts/%s/phone-verification',
        $MerchantCenterMultiClientAccountID,
        $MerchantCenterAccountID
    );
    my $Url =  EPAGESJ_BASE_URI . $trailPathUrl;
    my $hHeaders = {};

    LogGoogleSmartShopping('merchant center phone verification request: ', {
        Url => $Url,
        Payload => $hPhoneInfo,
    });

    my $hResponse = $self->doPost($Url, $hHeaders, $hPhoneInfo);

    LogGoogleSmartShopping('merchant center phone verification response: ', $hResponse);

    return $hResponse;
}

#========================================================================================
# function     confirmMerchantCenterPhoneVerification
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->confirmMerchantCenterPhoneVerification();
#----------------------------------------------------------------------------------------
# description  Sends confirmation code (received by SMS) to confirm and close merchant
#               center verification process.
#----------------------------------------------------------------------------------------
# input        $VerificationId | phone number verification id reference | string
# input        $hPhoneSettings | hash with confirmation code | ref.hash
#========================================================================================
sub confirmMerchantCenterPhoneVerification {
    my $self = shift;
    my ($VerificationId, $hPhoneSettings) = @_;

    $self->loadGoogleSettings();

    my $GoogleSmartShoppingConfig = $self->{'GoogleSmartShoppingConfig'};
    my $MerchantCenterAccountID = $GoogleSmartShoppingConfig->get('MerchantCenterAccountID');
    my $MerchantCenterMultiClientAccountID =
        $GoogleSmartShoppingConfig->get('MerchantCenterMultiClientAccountID') // $MerchantCenterAccountID;

    my $trailPathUrl = sprintf(
        'accounts/%s/sub-accounts/%s/phone-verification/%s',
        $MerchantCenterMultiClientAccountID ,
        $MerchantCenterAccountID,
        $VerificationId
    );
    my $Url =  EPAGESJ_BASE_URI . $trailPathUrl;
    my $hHeaders = {};

    LogGoogleSmartShopping('merchant center phone verification confirmation request: ', {
        Url => $Url,
        Payload => $hPhoneSettings,
    });

    $self->doPut($Url, $hHeaders, $hPhoneSettings);

    return;
}

1;
