#========================================================================================
# package      DE_EPAGES::Order::API::Shop
# state        public
#----------------------------------------------------------------------------------------
# description  This package provides the shop functions of order.
#========================================================================================
package DE_EPAGES::Order::API::Shop;
use base Exporter;
use strict;

our @EXPORT_OK = qw (
    GetLatestOrderCreationDate
    GetNextOrderNo
    GetRegisterSessionScriptTagUrl
    ProcessOrderDocumentTextAttributes
    ProcessOrderDocumentTextAttributesByLanguageID
    TriggerMailEventDeliveryThresholdReached
);

use DE_EPAGES::Database::API::Connection qw( GetCurrentDBHandle );
use DE_EPAGES::Core::API::PerlTools qw( GetLocalTimeZone );
use DE_EPAGES::Shop::API::Shop qw( GetNextNo );
use DE_EPAGES::Object::API::Language qw( GetCodeByLanguageID );
use DE_EPAGES::Order::API::Constants  qw ( UNITY_REGISTER_SESSION_SCRIPT_PAGE_TYPE );
use DE_EPAGES::Presentation::API::PageType qw( LoadPageTypeByAlias );
use DE_EPAGES::Presentation::API::Template qw( ProcessPage );
use DE_EPAGES::Core::API::Log qw (SetLogLevel);
use DE_EPAGES::ShopMailType::API::ShopMailType qw( GetShopMailType );

#========================================================================================
# function     GetNextOrderNo
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $OrderNo = GetNextOrderNo($Shop);
# syntax       my $OrderNo = GetNextOrderNo($Shop, $Object);
#----------------------------------------------------------------------------------------
# description  Returns the next order alias for this shop.
#----------------------------------------------------------------------------------------
# input        $Shop | shop | DE_EPAGES::Shop::API::Object::Shop
# input        $Object | object; used if OrderNo shall contain this objects alias
#                       | DE_EPAGES::Object::API::Object::Object
# return       $OrderNo | alias string for order | string
#========================================================================================
sub GetNextOrderNo($;$) {
    my ($Shop, $Object) = @_;
    return GetNextNo($Shop, 'CustomerOrder', $Object);
}

#========================================================================================
# function     ProcessOrderDocumentTextAttributes
# state        public
#----------------------------------------------------------------------------------------
# syntax       ProcessOrderDocumentTextAttributes($Shop, $PageTypeAlias, $Update);
#----------------------------------------------------------------------------------------
# description  set shop attributes for order documents by the given PageType alias
#----------------------------------------------------------------------------------------
# input        $Shop | shop | DE_EPAGES::Shop::API::Object::Shop
# input        $PageTypeAlias | PageType to be processed | string
# input        $Update | attributes are always updated | boolean
#========================================================================================
sub ProcessOrderDocumentTextAttributes {
    my ($Shop, $PageTypeAlias, $Update) = @_;

    my $hVars = $Shop->tleHash;
    my @ShopLanguages = map {$_->{'LanguageID'}} @{$Shop->get('Languages')};

    my $PageType = LoadPageTypeByAlias($PageTypeAlias);
    my $ObjectPageType = DE_EPAGES::Presentation::API::Object::ObjectPageType->load($PageType,$Shop);

    my $Text;
    SetLogLevel('WARN', sub {
        $Text = ProcessPage($Shop, $ObjectPageType, GetCodeByLanguageID($Shop->get('LanguageID')), $hVars);
    },'DE_EPAGES::Presentation::API::Template::INCLUDE');

    foreach my $Text2Key ('OrderDocumentsInvoiceText2','OrderDocumentsPackingSlipText2','OrderDocumentsCreditNoteText2'){
        foreach my $ShopLanguageID (@ShopLanguages){
            if($Update or not defined $Shop->get($Text2Key, $ShopLanguageID) ){
                $Shop->set({ $Text2Key => $Text }, $ShopLanguageID );
            }
        }
    }

}

#========================================================================================
# function     ProcessOrderDocumentTextAttributesByLanguageID
# state        public
#----------------------------------------------------------------------------------------
# syntax       ProcessOrderDocumentTextAttributesByLanguageID($Shop, $PageTypeAlias, $Update, $LanguageID);
#----------------------------------------------------------------------------------------
# description  set shop attributes for order documents by the given PageType alias and language id
#----------------------------------------------------------------------------------------
# input        $Shop | shop | DE_EPAGES::Shop::API::Object::Shop
# input        $PageTypeAlias | PageType to be processed | string
# input        $Update | attributes are always updated | boolean
# input        $LanguageID | language id | string
#========================================================================================
sub ProcessOrderDocumentTextAttributesByLanguageID {
    my ($Shop, $PageTypeAlias, $Update, $LanguageID) = @_;

    my $hVars = $Shop->tleHash;
    my $PageType = LoadPageTypeByAlias($PageTypeAlias);
    my $ObjectPageType = DE_EPAGES::Presentation::API::Object::ObjectPageType->load($PageType,$Shop);

    my $Text;
    SetLogLevel('WARN', sub {
        $Text = ProcessPage($Shop, $ObjectPageType, GetCodeByLanguageID($LanguageID), $hVars);
    },'DE_EPAGES::Presentation::API::Template::INCLUDE');

    foreach my $Text2Key ('OrderDocumentsInvoiceText2','OrderDocumentsPackingSlipText2','OrderDocumentsCreditNoteText2'){
        if($Update or not defined $Shop->get($Text2Key, $LanguageID) ){
            $Shop->set({ $Text2Key => $Text }, $LanguageID );
        }
    }
}


#========================================================================================
# function     GetLatestOrderCreationDate
# state        public
#----------------------------------------------------------------------------------------
# syntax       $LatestOrderCreationDate = GetLatestOrderCreationDate($Shop);
#----------------------------------------------------------------------------------------
# description  Returns the datetime of the latest created order of the given shop
#----------------------------------------------------------------------------------------
# input        $Shop | shop object | object
# return       $LatestOrderCreationDate | latest oder creation date | datettime
#========================================================================================
sub GetLatestOrderCreationDate {
    my ($Shop) = @_;

    my $dbi = GetCurrentDBHandle();
    my $sql = <<'END_SQL';
SELECT MAX(oo.creationdate) FROM customerorder AS o
INNER JOIN object AS oo ON oo.objectid = o.orderid AND oo.siteid=o.shopid
WHERE o.shopid = ?
END_SQL
    my $aaRows = $dbi->execute($sql, [$Shop->id], ['DateTime']);
    return $aaRows->[0]->[0];
}

#========================================================================================
# function     GetRegisterSessionScriptTagUrl
# state        public
#----------------------------------------------------------------------------------------
# syntax       GetRegisterSessionScriptTagUrl($Shop);
#----------------------------------------------------------------------------------------
# description  Returns the URL for the register session script tag
#----------------------------------------------------------------------------------------
# input        $Shop | shop object | object
# return       $RegisterSessionScriptUrl | relative URL | string
#========================================================================================
sub GetRegisterSessionScriptTagUrl {
    my ($Shop) = @_;

    my $RegisterSessionScriptPageType = LoadPageTypeByAlias(UNITY_REGISTER_SESSION_SCRIPT_PAGE_TYPE);
    my $RegisterSessionScriptViewAction = $RegisterSessionScriptPageType->get('ViewAction')->alias;
    return $Shop->get('WebServerScriptName') . '?ViewAction=' . $RegisterSessionScriptViewAction;
}

#========================================================================================
# function     TriggerMailEventDeliveryThresholdReached
# state        public
#----------------------------------------------------------------------------------------
# syntax       TriggerMailEventDeliveryThresholdReached($hParams);
#----------------------------------------------------------------------------------------
# description  Check for Unity shops if the delivery threshold is reached
#               when reached trigger mail event and add notification to dashboard
#               gets called in UI Hook (OnMBOViewMBO)
#               also from a scheduler script (calculateSalesVolume.pl)
#----------------------------------------------------------------------------------------
# input        $hParams | used keys are
#               <ul>
#                   <li>Shop
#                   <li>Servlet (Only in UI context)
#               </ul> | ref.hash.*
#========================================================================================
sub TriggerMailEventDeliveryThresholdReached {
    my ($hParams) = @_;
    my $Shop = $hParams->{'Shop'};
    my $Servlet = $hParams->{'Servlet'};

    my $WarningThresholdReachValue = 8000;
    my $hShopInfo = $Shop->get([qw( IsUnityRegistered DeliveryThresholdIsActive Address DeliveryThresholdSalesVolume )]);

    # DeliveryThresholdSalesVolume
    my $hDeliveryThresholdSalesVolume = $hShopInfo->{'DeliveryThresholdSalesVolume'};
    my $SalesVolume = $hDeliveryThresholdSalesVolume->{'SalesVolume'};
    my $IsLimitReachedMailSent = $hDeliveryThresholdSalesVolume->{'LimitReachedMailSent'};

    if ($hShopInfo->{'Address'}->get('IsEU') && $hShopInfo->{'IsUnityRegistered'}) {
        my $ShopNotifications = $Shop->folder('ShopNotifications');
        my $DeliveryThresholdReachedNotification = $ShopNotifications->child('DeliveryThresholdReached');
        # Only gets executed if we are in an UI context not from an scheduler context
        # Scheduler context gets called from Order/Scripts/calculateSalesVolume.pl
        if (defined $Servlet) {
            $Servlet->vars('ThresholdSalesVolume', $SalesVolume);
        }

        my $TimeZone = GetLocalTimeZone();

        # Only gets executed if we are in an UI context not from an scheduler context
        # Scheduler context gets called from Order/Scripts/calculateSalesVolume.pl
        if (defined $hDeliveryThresholdSalesVolume->{'LastUpdated'} && defined $Servlet){
            my $DateTimeDeliveryThreshold = DateTime->from_epoch('epoch' => $hDeliveryThresholdSalesVolume->{'LastUpdated'});
            # set timezone on dateTime object
            $DateTimeDeliveryThreshold->set_time_zone( $TimeZone );
            # make DeliveryThresholdSalesVolumeLastUpdate available on servlet
            $Servlet->vars('DeliveryThresholdSalesVolumeLastUpdate', $DateTimeDeliveryThreshold)
        }

        if(defined $IsLimitReachedMailSent && $SalesVolume >= $WarningThresholdReachValue && !$hShopInfo->{'DeliveryThresholdIsActive'} && $IsLimitReachedMailSent == 0) {
            # Activate dashboard notification addObject triggers activate
            $DeliveryThresholdReachedNotification->addObject($ShopNotifications);
        }
    }

    if (defined $IsLimitReachedMailSent && $IsLimitReachedMailSent == 0 && $SalesVolume >= $WarningThresholdReachValue && !$hShopInfo->{'DeliveryThresholdIsActive'} ) {
        # Send mail to merchant
        my $ShopMailType = GetShopMailType( $Shop, 'DeliveryThresholdReached' );
        my $Email = $Shop->get('MailFrom');

        return unless defined $Email;

        my %MailParams = (
            'HTMLAllowed' => 1,
        );
        my $LanguageID = $Shop->get('LanguageID');
        $MailParams{'TO'} = $Email;
        $MailParams{'FROM'} = $Email;
        $MailParams{'Subject'} = $ShopMailType->get('Subject', $LanguageID);

        my %RegionalParams = (
            Locale      => $Shop->get('LocaleID'),
            Language    => GetCodeByLanguageID($LanguageID),
        );
        my %Vars = (
            'DeliveryThresholdSalesVolume' => $SalesVolume,
        );

        $ShopMailType->send( $Shop, \%MailParams, \%RegionalParams, \%Vars );

        # Set Mail sent flag
        my $Container = {
            'SalesVolume'          => $SalesVolume,
            'LastUpdated'          => $hDeliveryThresholdSalesVolume->{'LastUpdated'},
            'LimitReachedMailSent' => 1,
        };
        $Shop->set({'DeliveryThresholdSalesVolume' => $Container});
    }

    return;
}

1;
