#==============================================================================
# §package      DE_EPAGES::Product::UI::VariationProduct
# §state        private
#==============================================================================
package DE_EPAGES::Product::UI::VariationProduct;
use base qw( DE_EPAGES::Presentation::UI::Object );

use strict;
use JSON;
use DE_EPAGES::Core::API::Error qw ( Error ExistsError GetError );
use DE_EPAGES::Core::API::Unicode qw ( Unicode2Alias );
use DE_EPAGES::Core::API::Warning qw ( BlockWarnings Warning );
use DE_EPAGES::Object::API::Constants qw (TYPEINFO );
use DE_EPAGES::Object::API::Factory qw ( LoadClassByAlias );
use DE_EPAGES::Product::API::Constants qw (PRODUCT_IMAGE_SIZE_NAMES );
use DE_EPAGES::Product::API::Product qw (
    CropProductImages
    InsertMultipleSubProducts
    UsedVariationValues
    StartUnityProgressMonitor
    UpdateUnityProgressMonitor
);
use DE_EPAGES::Product::API::ProductImageService;
use DE_EPAGES::Product::UI::Product;
use DE_EPAGES::Product::UI::ProductAttribute;
use DE_EPAGES::Trigger::API::BatchProcessor qw ( RunBatchProcess );

my $ATTRIBUTE_PACKAGE = 'DE_EPAGES::Product::UI::ProductAttribute';
my $TYPE_PACKAGE = 'DE_EPAGES::Product::UI::Product';

use constant MAX_VARIATIONS => 200;
use constant BIG_POSITION => 9999;

#==============================================================================
# Event handlers
#==============================================================================

sub WizardSaveProductAttributes {
    my $self = shift;
    my ($Servlet) = @_;

    my $VariationProduct = $Servlet->object;
    my $Form = $Servlet->form->form($VariationProduct, 'VariationWizard');
    my $ProductType = $VariationProduct->getSite()->loadObject($Form->{'ClassID'}, 'ProductType');

    if($Form->{'Name'} and $Form->{'Name'} ne 'ProductClass') {
        # get product type and/or new alias via form, insert new product type unless it exists
        my $NewProductTypeAlias = $self->getAlias($Form, $VariationProduct);

        if ($ProductType->alias eq 'ProductClass' and not defined $NewProductTypeAlias) {
            $Servlet->form->executeFormError({
                'Reason'=> 'NoAlias',
                'Name'  => 'Alias',
            });
        } elsif ($ProductType->alias eq 'ProductClass' and defined $NewProductTypeAlias) {
            $ProductType = $self->insertNewProductType($Servlet, $Form);
        }
    }

    my $hCreatedAttributes;
    RunBatchProcess('UpdateProducts', sub {
        # set new product type for variation product
        $hCreatedAttributes = $self->wizardSetProductTypeForProduct($Servlet, $ProductType, { 'noInsertValues' => 1 });
    });

    $Servlet->vars('ResponseContent', {
        'ProductType' => { 'ObjectID' => $ProductType->id, 'Alias' => $ProductType->alias },
        'Attributes' => $hCreatedAttributes
    });
    return;
}

sub NewProductVariation {
    my $self = shift;
    my ($Servlet) = @_;

    my $VariationProduct = $Servlet->object;
    my $Form = $Servlet->form->form($VariationProduct, 'VariationWizard');

    # get product type and/or new alias via form, insert new product type unless it exists
    my $NewProductTypeAlias = $self->getAlias($Form, $VariationProduct);

    my $ProductType = $VariationProduct->getSite()->loadObject($Form->{'ClassID'}, 'ProductType');
    if ($ProductType->alias eq 'ProductClass' and not defined $NewProductTypeAlias) {
        $Servlet->form->executeFormError({
            'Reason'=> 'NoAlias',
            'Name'  => 'Alias',
        });
    } elsif ($ProductType->alias eq 'ProductClass' and defined $NewProductTypeAlias) {
        $ProductType = $self->insertNewProductType($Servlet, $Form);
    }

    RunBatchProcess('UpdateProducts', sub {
        # set new product type for variation product
        $self->setProductTypeForProduct($Servlet, $ProductType);

        # create variation products
        $self->saveVariations($Servlet);
    });

    # save product type design
    $TYPE_PACKAGE->SaveDesign($Servlet);
    return;
}

sub setProductTypeForProduct {
    my $self = shift;
    my ($Servlet, $ProductType, $hOptions) = @_;
    my $Form = $Servlet->form;
    my $VariationProduct = $Servlet->object;
    my $IsUnityRegistered = $VariationProduct->getSite()->get('IsUnityRegistered');
    my $Shop = $VariationProduct->getSite();

    my $StandardProductClass = $Shop->get('ProductClass');

    $VariationProduct->set({'Class' => $ProductType}) if 0 == $VariationProduct->countSubProducts;

    # find out number of attributes
    my $maxidx;
    for(my $i=0; defined $Form->value('UseAttribute' . $i); $i++) {
        $maxidx = $i;
    }

    my $AttributePosition = BIG_POSITION;
    my $hCreatedAttributes;
    for (my $idx = 0; $idx <= $maxidx; $idx++) {

        my $Alias = $self->getAttributeAlias($Form, $VariationProduct, $idx);
        next unless defined $Alias and $Alias =~ /\S/;
        #store original Name/Alias for later use
        my $UnityAttributeName = $Alias;

        if ($IsUnityRegistered) {
            #do not transform alias if it already exists
            unless ($ProductType->existsAttribute($UnityAttributeName) || $StandardProductClass->existsAttribute($UnityAttributeName)) {
                $Alias = $self->getAttributeAliasByName($Form, $VariationProduct, $idx)
            }
        }

        my $VariationAttribute;
        my $UpdateSubProducts = 0;
        if (scalar grep { $_->alias eq $Alias } @{$ProductType->get('UserDefinedAttributes')}) {
            $VariationAttribute = $ProductType->attribute($Alias);
            $Form->executeFormError({
                'Reason'=> 'ProductAttributeExists',
                'Name'  => 'Alias',
                'Value' => $Alias
            }) unless $VariationAttribute->get('Type') =~ /PreDef/;
            $UpdateSubProducts = 1 if $Form->value('DefaultAttributeValue' . $idx);
        } else {
        # get the Type
            my $TypePackageAlias = $Servlet->params('TypePackage' . $idx);
            next if not defined $TypePackageAlias; # no type?!?!?
            my $hTypePackageInfo = TYPEINFO( $TypePackageAlias );

            $ATTRIBUTE_PACKAGE->checkAlias($Servlet, $Alias, undef, $ProductType);
            $Form->executeFormError;
            my $hAttribute = {
                Alias => $Alias,
                Name => $IsUnityRegistered ? $UnityAttributeName : $Alias,
                AttributeClass => $ProductType,
                Type => $hTypePackageInfo->{'Type'},
                Package => $hTypePackageInfo->{'Package'},
                IsArray => $hTypePackageInfo->{'IsArray'},
                IsUserDefined => 1,
                IsObject => $hTypePackageInfo->{'IsObject'},
                IsExportable => 1,
                ExportLevel => 1,
                Position => $AttributePosition++,
                IsCacheable => $hTypePackageInfo->{'IsCacheable'}
            };
            if($Form->value('VariationAttributeIsVisible' . $idx)) {
                $hAttribute->{'IsVisible'} = $Form->value('VariationAttributeIsVisible' . $idx);
            }

            $VariationAttribute = $ProductType->insertAttribute($hAttribute);
            $hCreatedAttributes->{$VariationAttribute->id} = $self->getAttributeAliasByName($Form, $VariationProduct, $idx);
            $UpdateSubProducts = 1;
        }

        if(!$hOptions->{'noInsertValues'}) {
            $self->insertAttributeValues($Form, $VariationProduct, $VariationAttribute, $idx, $UpdateSubProducts );
        }
    }
    $Form->executeFormError();

    # renumber product type attributes
    my $AttributeFolder = $ProductType->folder('Attributes');
    $AttributeFolder->renumberChildren() if $AttributePosition > BIG_POSITION;

    return $hCreatedAttributes;
}

sub insertAttributeValues {
    my $self = shift;
    my ($Form, $VariationProduct, $VariationAttribute, $idx, $UpdateSubProducts ) = @_;

    # insert new product type attribute values/update
    my $VariationAttributeValue = 'VariationAttributeValues' . $idx;
    my @Aliases = $Form->value($VariationAttributeValue);
    return if not scalar @Aliases and not $Form->value('DefaultAttributeValue' . $idx);

    my $Folder = $VariationAttribute->folder('PreDefAttributes');
    my $PreDefAttribute;
    foreach my $Alias (@Aliases) {
        next unless defined $Alias and $Alias =~ /\S/;
        if ($Folder->existsChild($Alias)) {
            $Form->addFormError({
                'Reason'=> 'AttributeValueExists',
                'Name'  => 'Type',
                'Value' => $Alias
            });
        } else {

            $PreDefAttribute = $Folder->insertChild('PreDefAttribute',{
                'Alias'       => $Alias,
                'Attribute'   => $VariationAttribute,
                'PreDefClass' => $VariationAttribute->class,
                'ValueString' => $Alias,
                'Position'    => BIG_POSITION,
            });
            if ($VariationAttribute->get('Type') eq 'PreDefLocalizedString') {
                $PreDefAttribute->set( {'ValueLocString' => $Alias} );
            }
            $Folder->renumberChildren;
        }
    }

    # check for default value
    my $DefaultAttributeValue = 'DefaultAttributeValue' . $idx;
    return unless (defined $Form->value($DefaultAttributeValue) and $Form->value($DefaultAttributeValue) =~ /\S/);

    my $DefaultValue = $Folder->existsChild($Form->value($DefaultAttributeValue))
        ?   $Folder->child($Form->value($DefaultAttributeValue))
        :   undef;
    return unless (defined $DefaultValue);

    my $DefaultAlias = $DefaultValue->get('Attribute')->alias;

    # update __existing__ variations
    if ($UpdateSubProducts) {
        $_->set({ $DefaultAlias => $DefaultValue }) foreach @{ $VariationProduct->subProducts };
    }
    # update master product with selected variations
    my $aSelectedVariations = $VariationProduct->get('SelectedVariations');
    push @$aSelectedVariations, {'SelectedID' => $DefaultValue->id};
    $VariationProduct->set({ 'SelectedVariations' => $aSelectedVariations });
}

sub wizardSetProductTypeForProduct {
    my $self = shift;
    my ($Servlet, $ProductType, $hOptions) = @_;
    my $Form = $Servlet->form;
    my $VariationProduct = $Servlet->object;

    $VariationProduct->set({'Class' => $ProductType}) if 0 == $VariationProduct->countSubProducts;

    # find out number of attributes
    my $maxidx;
    for(my $i=0; defined $Form->value('UseAttribute' . $i); $i++) {
        $maxidx = $i;
    }

    my $AttributePosition = BIG_POSITION;
    my @CreatedAttributes;
    for (my $idx = 0; $idx <= $maxidx; $idx++) {

        my $Alias = $self->getAttributeAlias($Form, $VariationProduct, $idx);
        next unless defined $Alias and $Alias =~ /\S/;

        my $VariationAttribute;
        my $UpdateSubProducts = 0;
        if (scalar grep { $_->alias eq $Alias } @{$ProductType->get('UserDefinedAttributes')}) {
            $VariationAttribute = $ProductType->attribute($Alias);
            # TODO: HANDLE ALL THE OTHER TYPES AS WELL!!!!
            next if $VariationAttribute->get('Type') =~ /PreDef/;
            $UpdateSubProducts = 1 if $Form->value('DefaultAttributeValue' . $idx);
        } else {
            # get the Type
            my $TypePackageAlias = $Servlet->params('TypePackage' . $idx);
            next if not defined $TypePackageAlias; # no type?!?!?
            my $hTypePackageInfo = TYPEINFO( $TypePackageAlias );

            $ATTRIBUTE_PACKAGE->checkAlias($Servlet, $Alias, undef, $ProductType);
            $Form->executeFormError;
            my $hAttribute = {
                Alias => $Alias,
                Name => $Form->value('VariationAttributeName' . $idx) ? $Form->value('VariationAttributeName' . $idx) : $Alias,
                AttributeClass => $ProductType,
                Type => $hTypePackageInfo->{'Type'},
                Package => $hTypePackageInfo->{'Package'},
                IsArray => $hTypePackageInfo->{'IsArray'},
                IsUserDefined => 1,
                IsObject => $hTypePackageInfo->{'IsObject'},
                IsExportable => 1,
                ExportLevel => 1,
                Position => $AttributePosition++,
                IsCacheable => $hTypePackageInfo->{'IsCacheable'}
            };
            if($Form->value('VariationAttributeIsVisible' . $idx)) {
                $hAttribute->{'IsVisible'} = $Form->value('VariationAttributeIsVisible' . $idx);
            }

            $VariationAttribute = $ProductType->insertAttribute($hAttribute);
            push(@CreatedAttributes, { 'ObjectID' => $VariationAttribute->id, 'Alias' => $Alias });
            $UpdateSubProducts = 1;
        }

        if(!$hOptions->{'noInsertValues'}) {
            $self->insertAttributeValues($Form, $VariationProduct, $VariationAttribute, $idx, $UpdateSubProducts );
        }
    }
    $Form->executeFormError();

    # renumber product type attributes
    my $AttributeFolder = $ProductType->folder('Attributes');
    $AttributeFolder->renumberChildren() if $AttributePosition > BIG_POSITION;

    return \@CreatedAttributes;
}

sub insertNewProductType {
    my $self = shift;
    my ($Servlet, $Form) = @_;

    # insert new product type unless it exists
    my $VariationProduct = $Servlet->object;
    my $Shop = $VariationProduct->getSite();
    my $ProductTypeFolder = $Shop->folder('ProductTypes');

    my $NewProductTypeAlias = $self->getAlias($Form, $VariationProduct);

    my $ProductType;
    if ($ProductTypeFolder->existsChild($NewProductTypeAlias)) {
        $ProductType = $ProductTypeFolder->child($NewProductTypeAlias);
    } else {
        my $BaseClass = $Shop->get('ProductClass');
        my $ProductTypeClass = LoadClassByAlias('ProductType');
        eval {
            $ProductType = $self->NewObject( $Servlet, $ProductTypeFolder, $ProductTypeClass, {
                BaseClass => $BaseClass,
                Name => $NewProductTypeAlias,
            } );
        };
        if (ExistsError()) {
            my $Error = GetError();
            if ($Error->code =~ /FeatureLimitExceeded/){
                $Servlet->form->executeFormError({
                    'Reason'=> 'FeatureMaxValueReached',
                    'Name'  => 'Type',
                });
                Warning($Error);
            } else {
                Error($Error);
            }
        }
    }
    return $ProductType;
}

sub getAttributeAlias {
    my $self = shift;
    my ($Form, $Object, $idx) = @_;

    my $VariationAttributeAlias = 'VariationAttributeAlias' . $idx;
    my $Alias = $Form->value($VariationAttributeAlias);
    # try using name
    if(!$Alias) {
        my $VariationAttributeName = 'VariationAttributeName' . $idx;
        $Alias = $Object->uniqueChildAlias($Form->value($VariationAttributeName));
    }
    return $Alias;
}

sub getAttributeAliasByName {
    my $self = shift;
    my ($Form, $Object, $idx) = @_;

    my $VariationAttributeAlias = 'VariationAttributeAlias' . $idx;
    my $Name = $Object->uniqueChildAlias(Unicode2Alias($Form->value($VariationAttributeAlias)));

    return $Name;
}

sub getAlias {
    my $self = shift;
    my ($Form, $Object) = @_;

    if($Form->{'Alias'}) {
        return $Form->{'Alias'};
    } elsif($Form->{'Name'}) {
        return $Object->uniqueChildAlias($Form->{'Name'});
    }
    return undef;
}

sub saveVariations {
    my $self = shift;
    my ($Servlet) = @_;

    my $Form = $Servlet->form;
    my $VariationProduct = $Servlet->object;
    my $ProductType = $VariationProduct->class;
    my %SelectedVariations = map { $_ => 1 } @{ UsedVariationValues($VariationProduct) };
    my @ListItems = $Form->value('ListItem');
    my $IsVisible = $Form->value('VariationProductsVisible') ? 1 : 0;
    my $NewVariationsCount = 0;
    my $TooManyVariations = 0;
    my $Shop = $VariationProduct->getSite();

    my $StandardProductClass =  $Shop->get('ProductClass');

    my $IsUnityRegistered = $Shop->get('IsUnityRegistered');
    # we have visible variation products for unity if the master is visible
    if ($IsUnityRegistered){
        $IsVisible = 1 if $VariationProduct->get('IsVisible');
    }

    # get the list of variation attributes
    my @AttributeAliases;
    my $AttributeIndex = 0;
    while( defined $Form->value('UseAttribute' . $AttributeIndex) ) {
        if( $Form->value('UseAttribute' . $AttributeIndex) ) {
            my $AttributeAlias = $Form->value('VariationAttributeAlias' . $AttributeIndex);
            if ($IsUnityRegistered){
                #do not transform alias if it already exists
                unless ($ProductType->existsAttribute($AttributeAlias) || $StandardProductClass->existsAttribute($AttributeAlias)) {
                    $AttributeAlias = $self->getAttributeAliasByName($Form, $VariationProduct, $AttributeIndex);
                }
            }
            die "Missing attribute alias for index $AttributeAlias" unless defined $AttributeAlias && $AttributeAlias =~ /\S/;

            if (! $ProductType->existsAttribute($AttributeAlias) ) {
                $Form->executeFormError({
                    'Reason'=> 'AttributeDoesNotExist',
                    'Name'  => 'VariationAttributeAlias' . $AttributeIndex,
                    'Value' => defined $AttributeAlias ? $AttributeAlias : 'undef',
                });
            }
            my $Atttribute = $ProductType->attribute($AttributeAlias);
            die "Attribute $AttributeAlias does not have any values" unless $Atttribute->existsChild('PreDefAttributes');

            push @AttributeAliases, $AttributeAlias;
        }

        $AttributeIndex++;
    }

    die 'No attributes selected' if 0 == scalar @AttributeAliases;

    StartUnityProgressMonitor($Servlet, $VariationProduct, \@ListItems) if $IsUnityRegistered;

    # create the variations
    foreach my $idx (@ListItems) {
        # for Unity shops use the feature value to determine the maximum
        # number of variation products for the assistant
        my $MaxVariations = $IsUnityRegistered
            ? $Shop->featureMaxValue('ProductMaxSubProducts') : MAX_VARIATIONS;
        # don't create more than $MaxVariations variations in a single run
        if ($NewVariationsCount >= $MaxVariations) {
            $TooManyVariations = 1;
            last;
        }

        my $ProductAlias = $Form->value('VariationAlias' . $idx);
        die "Missing alias for index $idx" unless defined $ProductAlias && $ProductAlias =~ /\S/;

        my %VariationValues;
        @VariationValues{@AttributeAliases} = $Form->value('NewVariationAttributeValues' . $idx);
        my @VariationTypes;
        foreach my $AttributeAlias (keys %VariationValues) {
            my $ValueAlias = $VariationValues{$AttributeAlias};
            die "Missing value for attribute $AttributeAlias at index $idx" unless defined $ValueAlias && $ValueAlias =~ /\S/;

            my $Attribute = $ProductType->attribute($AttributeAlias);
            my $ValuesFolder = $Attribute->child('PreDefAttributes');
            $Form->executeFormError({
                'Reason'=> 'AttributeValueDoesNotExist',
                'Name'  => 'VariationAttributeValues',
                'Value' => $ValueAlias,
            }) unless $ValuesFolder->existsChild($ValueAlias);

            my $Value = $ValuesFolder->child($ValueAlias);
            push @VariationTypes, {'Attribute' => $Attribute, 'Values' => [$Value] };

            # note which attribute values are selected/used
            $SelectedVariations{$Value->id} = 1;
        }

        # insert product variation
        my %Values = (
            'Alias' => $ProductAlias,
            'Position' => BIG_POSITION + $idx,
            'IsVisible' => $IsVisible,
        );

        my $aWarnings = BlockWarnings( sub {
            InsertMultipleSubProducts($VariationProduct, \@VariationTypes, \%Values );
        }, ['VariationCombinationExists', 'SubProductsCountFeatureLimitExceeded'] );
        $Servlet->form->mapFormError({
            'Name'  => 'NewSubProducts',
            'Form'  => 'NewSubProducts'
        }, $_) foreach @$aWarnings;

        $NewVariationsCount++;
        UpdateUnityProgressMonitor($Servlet) if $IsUnityRegistered;
    }

    # set selected/used variations for product
    $VariationProduct->set( { 'SelectedVariations' => [ map { {'SelectedID' => $_} } keys %SelectedVariations ] } );

    # make the master product visible if variations are created visible
    $VariationProduct->set( { 'IsVisible' => 1 } ) if $IsVisible;

    # set position number 10, 20, 30, ...
    $VariationProduct->folder('SubProducts')->renumberChildren;

    if( $TooManyVariations ) {
        $Form->executeFormError({
            'Reason'=> 'TooManyVariations',
            'Name'  => 'NoAlias',
            'Value' => 'NoValue',
        });
    }

    return;
}

sub ViewProductVariationWizard {
    my $self = shift;
    my ($Servlet) = @_;

    my $Product = $Servlet->object;
    my $aSubProducts = $Product->get('SubProducts');
    my $ConsecutiveNumber = -1;
    foreach my $SubProduct (@$aSubProducts) {
        my $Alias = $SubProduct->alias;
        my $NextAliasNo = $Alias =~ /-(\d+)$/ ? $1 : -1;
        $ConsecutiveNumber = $NextAliasNo if $ConsecutiveNumber <= $NextAliasNo;
    }

    $Servlet->vars('NextAliasNo', $ConsecutiveNumber + 1);
    return;
}

sub UnitySetVariationSlideshow {
    my $self      = shift;
    my ($Servlet) = @_;

    # crop images and save the slideshow for a given product....
    my $Product   = $Servlet->object;
    my $Form      = $Servlet->form;
    my $Shop      = $Product->getSite;

    # here we have the slideshowstring from the form
    # these data can be manipulated and saved after the function 'CropProductImages'
    my $ahSubProductsSlideShow = from_json($Form->value('SlideshowStrings'), { utf8  => 1 });
   
    # 1st crop and rotate images if needed
    # the json SlideshowStrings are changed by the cropping function
    if ($Form->value('cropData')){
        my $ahCropDataFromJSON = from_json($Servlet->form->value('cropData'), { utf8  => 1 });
        my $hChangedFiles      = CropProductImages($Product, $ahCropDataFromJSON, {'IsVariation' => 1});
        # manipulate $JSONSlideshowStrings with the changed filenames
        foreach my $hSubProductSlideShow (@$ahSubProductsSlideShow) {
            foreach my $Key (keys %$hChangedFiles ){
                if( $hSubProductSlideShow->{'slideshowString'} =~ /$Key/g ) {
                    $hSubProductSlideShow->{'slideshowString'} =~ s/$Key/$hChangedFiles->{$Key}/g;
                }                 
            }
        }
    }

    # set the slideshow for each sub product
    foreach my $hSubProductSlideShow (@$ahSubProductsSlideShow) {
        my $SubProduct = $Shop->loadObject($hSubProductSlideShow->{'id'},'Product');
        my $SlideShowString = $hSubProductSlideShow->{'newSlideshow'} // $hSubProductSlideShow->{'slideshowString'};
        $SubProduct->set({'ImagesSlideShowString' => $SlideShowString });

        # set the first slideshow image as product image or delete image attributes on product if we have no slideshow
        if(scalar(@{$SubProduct->get('ImagesSlideShow') })){
            $self->_setFirstSlideShowImageAsProductImage($SubProduct, $SlideShowString);
        }else{
            my $hScaledImages = {};
            foreach my $SizeExt (keys(%{PRODUCT_IMAGE_SIZE_NAMES()})) {
                $hScaledImages->{PRODUCT_IMAGE_SIZE_NAMES->{$SizeExt}} = undef;
            }
            $hScaledImages->{'ImageLarge'} = undef;
            $SubProduct->set($hScaledImages);
        }
    }

    return;
}

sub _setFirstSlideShowImageAsProductImage {
    my $self      = shift;
    my ($Product, $SlideShowString) = @_;

    $SlideShowString = defined $SlideShowString ? $SlideShowString : $Product->get('ImagesSlideShowString');

    # use the first image of the slideshow as main image for the product
    my $aImages = [split(/;/, $SlideShowString)];
    shift (@$aImages) unless $aImages->[0];

    # use the image service
    next unless $aImages->[0];
    my $ProductImageService = DE_EPAGES::Product::API::ProductImageService->new($Product);
    $ProductImageService->setSlideShowImageAsProductImage($aImages->[0]);

}

1;
