#!/bin/bash

. /etc/default/epages6

set -e

#=============================================================================
# ECHO
#=============================================================================

ECHO () {
  local LINE_NUMBER=$(printf "%3d" $1)
  shift
  echo -e "[${0##*/}:$LINE_NUMBER] $@" 1>&2
}

#=============================================================================
# stores_with_cartridge
#=============================================================================

stores_with_cartridge() {
  grep -rl "\b$1\b" $EPAGES_CONFIG/Cartridges.d \
    | xargs -L 1 basename 2>/dev/null | sed 's/.conf//g' | sort
}

##############################################################################
# arguments
##############################################################################

usage() {
  [[ -n $1 ]] && echo -e "Error:\n  $@"
  cat <<HERE
Usage:
  $0 [--dry-run] --all PSWD --tbo PSWD --pbo PSWD --diag PSWD --ws PSWD ...
Options:
  --dry-run     : just show commands to execute, do not execute them
  --all 'PSWD'  : set passwords for --tbo --pbo --diag and --ws
  --tbo 'PSWD'  : set password for user /Users/tboadmin in BU Site
  --pbo 'PSWD'  : set password for user /Providers/Distributor/Users/admin in BU Site
  --diag 'PSWD' : for each BU with installed DE_EPAGES::Diagnostics cartridge
                  set password for user /Users/developer
  --ws 'PSWD'   : set password for each WebServiceLogin (like /Users/wsadmin)
                  mentioned in j/conf/ep6WebService.conf
  --nostart     : do not restart ePages
  --write-conf  : if PSWD is defined then update epages.conf with PSWD
  --read-conf   : if PSWD is not defined then read PSWD from epages.conf like:
                  [DE_EPAGES::ShopConfiguration]
                  tbo_password=PSWD
                  pbo_password=PSWD
                  diag_password=PSWD
                  ws_password=PSWD
Examples:
  # 1) use 'geheim' for all passwords and update epages.conf:
  $0 --dry-run --all 'geheim' --write-conf
  # 2) find all passwords in epages.conf, ignore missing passwords:
  $0 --dry-run --all --read-conf
  # 3) contrary to 2) throw error if TBO password is missing in epages.conf:
  $0 --dry-run --all --tbo --read-conf
  # 4) use 'geheim' as TBO password and find all other passwords in epages.conf:
  $0 --dry-run --all --tbo 'geheim' --read-conf
  # 5) use 'geheim' for all passwords but find TBO password in epages.conf:
  $0 --dry-run --all 'geheim' --tbo --read-conf
  # 6) use 'geheim' for all passwords (here, --read-conf is ignored)
  $0 --dry-run --all 'geheim' --read-conf
HERE
  exit 1
}

#=============================================================================
# get arguments
#=============================================================================

unset READ_CONF WRITE_CONF DRYRUN NOSTART
declare -A PSWD ARGS CMDLINE
while [[ -n $1 ]] ; do
  case "$1" in
    --nostart) NOSTART=1 ; shift ;;
    --read-conf) READ_CONF=1 ; shift ;;
    --write-conf) WRITE_CONF=1 ; shift ;;
    --dry-run) DRYRUN=1 ; shift ;;
    --all|--tbo|--pbo|--diag|--ws)
      ARGS[${1#--}]=1
      CMDLINE[${1#--}]=1
      if [[ ${2-undef} = undef || ${2#--} != $2 ]] ; then
        shift
      else
        PSWD[${1#--}]=$2 ; shift 2
      fi ;;
    --help) usage ;;
    *) usage "invalid argument: $1" ;;
  esac
done

#=============================================================================
# check arguments
#=============================================================================

[[ -n ${!ARGS[@]} ]] || usage "option(s) required: --all --tbo --pbo --diag --ws"

if [[ ${PSWD[all]+defined} = defined ]] ; then
  for i in tbo pbo diag ws ; do
    [[ ${PSWD[$i]+defined} = defined ]] && continue
    [[ ${ARGS[$i]+defined} = defined ]] && continue
    PSWD[$i]=${PSWD[all]}
    ARGS[$i]=1
  done
elif [[ ${ARGS[all]+defined} = defined ]] ; then
  for i in tbo pbo diag ws ; do
    [[ ${ARGS[$i]+defined} = defined ]] && continue
    ARGS[$i]=1
  done
fi

ic=/var/epages/ini-config.sh
sc=DE_EPAGES::ShopConfiguration

if [[ -n $WRITE_CONF ]] ; then
  for i in ${!PSWD[@]} ; do
    [[ $i = all ]] && continue
    ECHO $LINENO "$ic -f $EPAGES_CONFIG/epages.conf -s $sc -k ${i}_password -v \"${PSWD[$i]}\" -i"
    [[ -n $DRYRUN ]] || $ic -f $EPAGES_CONFIG/epages.conf -s $sc -k ${i}_password -v "${PSWD[$i]}" -i
  done
fi

for i in ${!ARGS[@]} ; do
  [[ $i = all ]] && continue
  [[ ${PSWD[$i]+defined} = defined ]] && continue
  if [[ -n $READ_CONF ]] ; then
    p=$($ic -f $EPAGES_CONFIG/epages.conf -s $sc -k ${i}_password -G)
    [[ -n $p ]] && { PSWD[$i]=$p; continue; }
    [[ -z ${CMDLINE[$i]} ]] && continue
    usage "$EPAGES_CONFIG/epages.conf w/ empty value: [$sc]${i}_password="
  fi
  [[ ${PSWD[all]+defined} = defined ]] && { PSWD[$i]=${PSWD[all]}; continue; }
  [[ -z $READ_CONF ]] && usage "use either --read-conf or PSWD"
done

##############################################################################
# SetPassword.pl
##############################################################################

set_pswd="$PERL $EPAGES_CARTRIDGES/DE_EPAGES/Permission/Scripts/SetPassword.pl"
set="$PERL $EPAGES_CARTRIDGES/DE_EPAGES/Object/Scripts/set.pl"

if [[ ${PSWD[tbo]+defined} = defined ]] ; then
  ECHO $LINENO "$set_pswd -storename Site -user /Users/tboadmin -password '${PSWD[tbo]}'"
  if [[ -z $DRYRUN ]] ; then
    $set_pswd -storename Site -user /Users/tboadmin -password "${PSWD[tbo]}"
  fi
fi

if [[ ${PSWD[pbo]+defined} = defined ]] ; then
  ECHO $LINENO "$set_pswd -storename Site -user /Providers/Distributor/Users/admin -password '${PSWD[pbo]}'"
  [[ -n $DRYRUN ]] || $set_pswd -storename Site -user /Providers/Distributor/Users/admin -password "${PSWD[pbo]}"
fi

if [[ ${PSWD[diag]+defined} = defined ]] ; then
  for BU in $(stores_with_cartridge DE_EPAGES::Diagnostics) ; do
    ECHO $LINENO "$set_pswd -storename $BU -user /Users/developer -password '${PSWD[diag]}'"
    [[ -z $DRYRUN ]] || continue
    $set_pswd -storename $BU -user /Users/developer -password "${PSWD[diag]}"
  done
fi

if [[ ${PSWD[ws]+defined} = defined ]] ; then
  TMPFILE=$(mktemp)

  FIRSTLINE=1
  while read line ; do
    if [[ -n $FIRSTLINE ]] ; then
      echo "$line"
      unset FIRSTLINE
      continue
    fi
    aLine=($(tr ',' ' ' <<< "$line"))
    sed -r "s/^([^,]*,[^,]*),[^,]*,(.*)/\1,${PSWD[ws]},\2/" <<< "$line"
    ECHO $LINENO "$set_pswd -storename ${aLine[0]} -user '${aLine[1]}' -password '${PSWD[ws]}'"
    if [[ -z $DRYRUN ]] ; then
      $set_pswd -storename ${aLine[0]} -user "${aLine[1]}" -password "${PSWD[ws]}" 1>&2
    fi
    ECHO $LINENO "$set -storename Site -path /Stores/${aLine[0]} WebServicePassword='${PSWD[ws]}'"
    if [[ -z $DRYRUN ]] ; then
      $set -storename Site -path /Stores/${aLine[0]} WebServicePassword="${PSWD[ws]}" 1>&2
    fi
  done < $EPAGESJ/conf/ep6WebService.conf > $TMPFILE
  ECHO $LINENO "$set -storename Site -path /Stores/Site WebServicePassword='${PSWD[ws]}'"
  if [[ -z $DRYRUN ]] ; then
    $set -storename Site -path /Stores/Site WebServicePassword="${PSWD[ws]}"
  fi

  ECHO $LINENO "cat $EPAGESJ/conf/ep6WebService.conf"
  if [[ -n $DRYRUN ]] ; then
    cat $TMPFILE
  else
    cat $TMPFILE | tee $EPAGESJ/conf/ep6WebService.conf
  fi
  rm -f $TMPFILE

  if [[ -z $NOSTART ]] ; then
    ECHO $LINENO "EPAGES_IGNORE=cron /etc/init.d/epages6 start_j"
    [[ -n $DRYRUN ]] || EPAGES_IGNORE=cron /etc/init.d/epages6 start_j
    ECHO $LINENO "EPAGES_IGNORE=cron /etc/init.d/epages6 start"
    [[ -n $DRYRUN ]] || EPAGES_IGNORE=cron /etc/init.d/epages6 start
  fi
fi

##############################################################################
# exit
##############################################################################

exit 0
