#========================================================================================
# package      DE_EPAGES::ShopMigration::API::PriceListQuantityDiscountsCommon
# state        public
#----------------------------------------------------------------------------------------
# description common functions for exporting/importing PriceListQuantityDiscounts
#========================================================================================
package DE_EPAGES::ShopMigration::API::PriceListQuantityDiscountsCommon;
use base qw(Exporter);

use strict;
use feature 'switch';

our @EXPORT_OK = qw(
    GetPriceListQuantityDiscounts
    SetPriceListQuantityDiscounts
    SetPriceListQuantityDiscountData
    GetPriceListQuantityDiscountData
    RemoveQuantityDiscountsForUnityBulkPrices
);
use DE_EPAGES::Core::API::PerlTools qw(ElementOf);
use DE_EPAGES::Core::API::Error qw(Error);
use DE_EPAGES::Core::API::Log qw(LogDebug);
use DE_EPAGES::Object::API::Factory qw ( LoadObject LoadObjectByPath LoadClassByAlias );
use DE_EPAGES::PriceList::API::Table::QuantityDiscount qw ( GetQuantityDiscountsByPriceListID );
use DE_EPAGES::ShopMigration::API::Constants qw(
    PRICELIST_QUANTITYDISCOUNT_ATTRIBUTES
);
use DE_EPAGES::PriceList::API::PriceList qw (GetExportablePriceLists);
use DE_EPAGES::PriceList::API::Constants qw(UNITY_PRICELIST_PREFIX);

#ERRORDEFINITION
sub ErrorDefinition {
    return {
        'REFERENCED_PRODUCT_NOT_FOUND' => 'Cannot create quantity discount for product #Name as product was not found in shop #Shop',
        'REFERENCED_PRICELIST_NOT_FOUND' => 'Cannot create scale price as Pricelist #Name was not found in shop #Shop',
    };
}
#ERRORDEFINITION END


#========================================================================================
# function     GetPriceListQuantityDiscounts
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $ahPriceListQuantityDiscounts = GetPriceListQuantityDiscounts($Shop);
#----------------------------------------------------------------------------------------
# description  Get the quantity discounts for a shop
#----------------------------------------------------------------------------------------
# input        $Shop | BaseShop (epages6) | object
#----------------------------------------------------------------------------------------
# return       $ahPriceListQuantityDiscounts| see GetPriceListQuantityDiscountData() |
#               <ul>
#                   <li>PriceListAlias   - pricelist alias |   - sting
#                   <li>RelativePath - relative product path | - string
#                   <li>Quantity - quantity |            - decimal
#                   <li>Discount - discount |            - double
#               </ul> | reference to array of hashes
#========================================================================================
sub GetPriceListQuantityDiscounts {
    my ($Shop) = @_;
    my @Items;

    my $PriceLists = GetExportablePriceLists($Shop);
    foreach my $PriceList (@$PriceLists) {
        my $QuantityDiscountsForPriceList = GetQuantityDiscountsByPriceListID($Shop->id, $PriceList->id);

        foreach my $QuantityDiscountEntry (@$QuantityDiscountsForPriceList) {
            my $Product = LoadObject($QuantityDiscountEntry->{'ProductID'});
            #do not export QuantityDiscounts for subproducts with HasSubOwnPrice=0, possible leftovers in DB
            next if (!$Product->get('HasSubOwnPrices') && defined $Product->get('SuperProduct'));
            my $hValues = $Product->get(['RelativePath', 'IsDownloadProduct', 'IsBundleProduct', 'IsCustomizable']);
            my @FailReason = grep { $hValues->{$_} == 1 && ($_ ne 'RelativePath')  } keys %$hValues;
            if (scalar @FailReason) {
                LogDebug(sprintf("Skip Product '%s' because of %s:%s\n", $Product->pathString, $FailReason[0], '1'), 1);
                next;
            }
            $QuantityDiscountEntry->{'PriceListAlias'} = $PriceList->alias;
            $QuantityDiscountEntry->{'RelativePath'} = $hValues->{'RelativePath'};
            push @Items, $QuantityDiscountEntry;
        }
    }
    return \@Items;
}

#========================================================================================
# function     GetPriceListQuantityDiscountData
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $hPriceListQuantityDiscount = GetPriceListQuantityDiscountData($Shop, $PriceListQuantityDiscounts);
#----------------------------------------------------------------------------------------
# description  Constructs a hash with the exportable fields for a single PriceListQuantityDiscount
#----------------------------------------------------------------------------------------
# input        $Shop | BaseShop (epages6) | object
# input        $PriceListQuantityDiscount | PriceListQuantityDiscount | object
#----------------------------------------------------------------------------------------
# return       $hPriceListQuantityDiscount | hash with following fields
#               <ul>
#                   <li>Attributes
#                       <ul>
#                           <li>RelativePath                     - string</li>
#                           <li>PriceListAlias                   - string</li>
#                           <li>Quantity                         - integer</li>
#                           <li>Discount                         - double</li>
#                       </ul>
#                   </li>
#               </ul>| reference to hash
#========================================================================================
sub GetPriceListQuantityDiscountData {
    my ($Shop, $PriceListQuantityDiscounts) = @_;
    my %Info = ();

    my $Attributes = [ @{PRICELIST_QUANTITYDISCOUNT_ATTRIBUTES()} ];
    foreach my $Attribute (@$Attributes) {
        $Info{$Attribute} = $PriceListQuantityDiscounts->{$Attribute} if defined $PriceListQuantityDiscounts->{$Attribute};
    }
    return \%Info;
}

#========================================================================================
# function     SetPriceListQuantityDiscounts
# state        public
#----------------------------------------------------------------------------------------
# syntax       SetPriceListQuantityDiscounts($Shop, $ahPriceListQuantityDiscounts);
#----------------------------------------------------------------------------------------
# description  sets attributes on PriceListQuantityDiscount as specified in the $ahPriceListQuantityDiscounts
#----------------------------------------------------------------------------------------
# input        $Shop         | baseshop or prepshop | object
# input        $ahPriceListQuantityDiscounts  | see SetPriceListQuantityDiscountData() | reference to array of hashes
#========================================================================================
sub SetPriceListQuantityDiscounts {
    my ($Shop, $ahPriceListQuantityDiscounts) = @_;

    foreach my $hPriceListQuantityDiscount (@$ahPriceListQuantityDiscounts) {
        SetPriceListQuantityDiscountData($Shop, $hPriceListQuantityDiscount);
    }
}

#========================================================================================
# function     SetPriceListQuantityDiscountData
# state        public
#----------------------------------------------------------------------------------------
# syntax       SetPriceListQuantityDiscountData($Shop, $hPriceListQuantityDiscount);
#----------------------------------------------------------------------------------------
# description  sets attributes on a PriceListQuantityDiscount as specified in the hash $hPriceListQuantityDiscount
#----------------------------------------------------------------------------------------
# input        $Shop         | baseshop or prepshop | object
# input        $hPriceListQuantityDiscount    | hash with following fields
#               <ul>
#                   <li>Attributes
#                       <ul>
#                           <li>ProductAlias                     - string</li>
#                           <li>PriceListAlias                   - string</li>
#                           <li>Quantity                         - integer</li>
#                           <li>Discount                         - double</li>
#                       </ul>
#                   </li>
#               </ul>| reference to hash
#========================================================================================
sub SetPriceListQuantityDiscountData {
    my ($Shop, $hPriceListQuantityDiscount) = @_;
    my @Attributes = @{PRICELIST_QUANTITYDISCOUNT_ATTRIBUTES()};


    foreach my $Key (keys %{$hPriceListQuantityDiscount->{Attributes}}) {
        die "Invalid Element for Attributes: $Key" unless ElementOf($Key, @Attributes);
    }

    my $PriceListAlias;
    if ($Shop->get('IsMigrationPrepShop')) {
        #add temporary Prefix "PL_epages6_" to $PriceListAlias
        $PriceListAlias = "PL_epages6_".$hPriceListQuantityDiscount->{'PriceListAlias'};
    } else{
        $PriceListAlias = $hPriceListQuantityDiscount->{'PriceListAlias'};
    }

    my $PriceList = $Shop->folder('PriceLists')->existsChild($PriceListAlias) ?
        $Shop->folder('PriceLists')->child($PriceListAlias) :
        Error('REFERENCED_PRICELIST_NOT_FOUND', { Name => $PriceListAlias, Shop => $Shop->alias });

    my $PrepShopPath = $Shop->get('Path');
    my $Product = LoadObjectByPath($PrepShopPath.'/'.$hPriceListQuantityDiscount->{'RelativePath'});
    Error('REFERENCED_PRODUCT_NOT_FOUND', { Name => $hPriceListQuantityDiscount->{'RelativePath'}, Shop => $Shop->alias }) unless defined $Product;

    if ($Product->get('IsMaster') && $Shop->get('IsMigrationPrepShop')) {
        #get MigrationShop for SubProduct handling
        my $BaseShop = $Shop->get('MigrationShop');
        my $BaseShopProduct = LoadObjectByPath($BaseShop->get('Path').'/'.$hPriceListQuantityDiscount->{'RelativePath'});

        #get all SubProducts from Master beacuse of HasSubOwnPrices flag
        foreach my $SubProduct (@{$BaseShopProduct->get('SubProducts')}) {

            if (!$SubProduct->get('HasSubOwnPrices')) {
                #Insert ScalePrice for Supproduct into prepshop
                my $SubProductPrepShop = LoadObjectByPath($PrepShopPath.'/'.$SubProduct->get('RelativePath'));
                LoadClassByAlias('QuantityDiscount')->insertObject({
                    'PriceList' => $PriceList,
                    'Product'   => $SubProductPrepShop,
                    'Quantity'  => $hPriceListQuantityDiscount->{'Quantity'},
                    'Discount'  => $hPriceListQuantityDiscount->{'Discount'},
                });
            }
        }
    }

    LoadClassByAlias('QuantityDiscount')->insertObject({
        'PriceList' => $PriceList,
        'Product'   => $Product,
        'Quantity'  => $hPriceListQuantityDiscount->{'Quantity'},
        'Discount'  => $hPriceListQuantityDiscount->{'Discount'},
    });

    return;
}

#========================================================================================
# function     RemoveQuantityDiscountsForUnityBulkPrices
# state        public
#----------------------------------------------------------------------------------------
# syntax       RemoveQuantityDiscountsForUnityBulkPrices($Shop);
#----------------------------------------------------------------------------------------
# description  Deletes all QuantityDiscounts for the simplified bulk prices pricelist
#----------------------------------------------------------------------------------------
# input        $Shop | prep shop - shop object | object
#========================================================================================
sub RemoveQuantityDiscountsForUnityBulkPrices {
    my ($Shop) = @_;
    my $ShopPriceLists = $Shop->get('PriceLists');

    foreach my $PriceList (@$ShopPriceLists) {
        if ($PriceList->alias eq UNITY_PRICELIST_PREFIX.$Shop->get('CurrencyID')) {
            LoadClassByAlias('QuantityDiscount')->deleteObjects('ByPriceListID', [$PriceList->id]);
        }
    }
    return;
}

1;
