#==============================================================================
# package      DE_EPAGES::ShopTransfer::API::DBImySQL
# state        public
#----------------------------------------------------------------------------------------
# description  MySQL specific database access functions.
#==============================================================================
package DE_EPAGES::ShopTransfer::API::DBImySQL;
use base DE_EPAGES::ShopTransfer::API::DBI;

use strict;
use DE_EPAGES::Database::API::Connection qw( GetCurrentDBHandle);

# Blacklist of all tables, which have an auto increment column in mysql but
# a SP in Sybase instead of a _nextid table. These tables must be threated differentry
# in the folling processes (they are included in the so-called 'dynamic tables')
my $AUTO_INCREMENT_BLACKLIST = {
    #cartridge => 1,
    shorturl => 1,
    styletemplatevariation => 1,
    stylegroupmap => 1,
    ebayauctionshippinglocation => 1,
    ebaycall => 1,
    ebaycategory => 1,
    ebaytemplate => 1,
    ebaycalltype => 1,
    multiselectattribute => 1,
    referenceloop => 1,
    language => 1,
    traygroup => 1,
    sessionattribute => 1,
    formloop => 1,
    eventfavorite => 1,
    formfield => 1,
    selectedvariation => 1,
    searchfrom => 1,
    searchsort => 1,
    searchterm => 1,
    searchparameter => 1,
    shoplanguage => 1,
    cartridgeref => 1,
    hookfunction => 1,
    hook => 1,
};

# lock this table to serialize ALTER TABLE and INFORMATION_SCHEMA queries
my $LOCK_TABLE = 'language';

#========================================================================================
# function     getAllReferences
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hTableReferences = $dbi->getAllReferences;
#----------------------------------------------------------------------------------------
# description  Returns all foreign keys of all tables in the current database.
#----------------------------------------------------------------------------------------
# return       $hTableReferences |
#                   {
#                       $TableName => {
#                           $ColumnName => {
#                               ConstraintName => $ConstraintName,
#                               MasterTableName => $MasterTableName,
#                               MasterColumnName => $MasterColumnName,
#                           },...
#                       }, ...
#                   }
#               | ref.hash.hash.hash.string
#========================================================================================
sub getAllReferences {
    my $self = shift;

    unless( defined $self->{'AllReferences'} ) {
        my $aaRows = GetCurrentDBHandle()->execute(<<END_SQL);
SELECT CONSTRAINT_NAME, TABLE_NAME, COLUMN_NAME, REFERENCED_TABLE_NAME, REFERENCED_COLUMN_NAME
FROM INFORMATION_SCHEMA.KEY_COLUMN_USAGE
WHERE REFERENCED_COLUMN_NAME IS NOT NULL
  AND CONSTRAINT_SCHEMA = DATABASE()
END_SQL

        my %TableReferences;
        foreach my $aRow (@$aaRows) {
            my ($ConstraintName, $TableName, $ColumnName, $MasterTableName, $MasterColumnName) = @$aRow;
            $TableReferences{$TableName}{$ColumnName} = {
                ConstraintName => $ConstraintName,
                MasterTableName => $MasterTableName,
                MasterColumnName => $MasterColumnName,
            };
        }

        $self->{'AllReferences'} = \%TableReferences;
    }
    return $self->{'AllReferences'};
}

#========================================================================================
# function     getAllColumns
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hTableColumns = $dbi->getAllColumns;
#----------------------------------------------------------------------------------------
# description  Returns all columns of all tables in the current database.
#----------------------------------------------------------------------------------------
# return       $hTableReferences |
#                   {
#                       $TableName => {
#                           $ColumnName => {
#                               Position => $Position,
#                           },...
#                       }, ...
#                   }
#               | ref.hash.hash.hash.string
#========================================================================================
sub getAllColumns {
    my $self = shift;

    unless( defined $self->{'AllColumns'} ) {
        my $dbi = GetCurrentDBHandle();
        $dbi->lockTables( [ { 'table' => $LOCK_TABLE, 'mode' => 'WRITE' } ], sub {
            my $aaRows = GetCurrentDBHandle()->execute(<<END_SQL);
SELECT TABLE_NAME, COLUMN_NAME, ORDINAL_POSITION - 1
FROM INFORMATION_SCHEMA.COLUMNS
WHERE TABLE_SCHEMA = DATABASE()
END_SQL

            my %TableColumns;
            foreach my $aRow (@$aaRows) {
                my ($TableName, $ColumnName, $Position) = @$aRow;
                $TableColumns{$TableName}{$ColumnName} = {
                    Position => $Position,
                };
            }

            $self->{'AllColumns'} = \%TableColumns;
        });
    }
    return $self->{'AllColumns'};
}

#========================================================================================
# function     getAllPrimaryKeys
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hPrimaryKeys = $dbi->getAllPrimaryKeys;
#----------------------------------------------------------------------------------------
# description  Returns all primary keys of all tables in the current database.
#----------------------------------------------------------------------------------------
# return       $hPrimaryKeys |
#                   {
#                       $TableName => [ $ColumnName, ... ],
#                       ...
#                   }
#               | ref.hash.hash.string/array
#========================================================================================
sub getAllPrimaryKeys {
    my $self = shift;

    unless( defined $self->{'AllPrimaryKeys'} ) {
        my $dbi = GetCurrentDBHandle();
        $dbi->lockTables( [ { 'table' => $LOCK_TABLE, 'mode' => 'WRITE' } ], sub {

            my $aaRows = GetCurrentDBHandle()->execute(<<'END_SQL');
SELECT TABLE_NAME, COLUMN_NAME, ORDINAL_POSITION
FROM INFORMATION_SCHEMA.COLUMNS
WHERE TABLE_SCHEMA = DATABASE()
AND COLUMN_KEY='PRI'
ORDER BY TABLE_NAME, ORDINAL_POSITION
END_SQL

            my %TablePKeys;
            foreach my $aRow (@$aaRows) {
                my ($TableName, $ColumnName) = @$aRow;
                if( exists $TablePKeys{$TableName} ) {
                    push @{$TablePKeys{$TableName}}, $ColumnName;
                }
                else {
                    $TablePKeys{$TableName} = [$ColumnName];
                }
            }
            $self->{'AllPrimaryKeys'} = \%TablePKeys;
        });
    }
    return $self->{'AllPrimaryKeys'};
}

#========================================================================================
# function     getUniqueIndexes
# state        public
#----------------------------------------------------------------------------------------
# syntax       @Columns = $dbi->getUniqueIndexes($Table);
#----------------------------------------------------------------------------------------
# description  Returns first columns of all unique indexes.
#----------------------------------------------------------------------------------------
# input        $Table | table name | string
# return       @Columns | column names | array.string
#========================================================================================
sub getUniqueIndexes ($) {
    my $self = shift;
    my ($Table) = @_;

    my $dbi = GetCurrentDBHandle();
    # 'TABLE_NAME = ?' is required; you can't use '?' or `?`
    my $sql = <<END_SQL;
SELECT COLUMN_NAME
FROM INFORMATION_SCHEMA.COLUMNS
WHERE TABLE_SCHEMA = DATABASE()
  AND COLUMN_KEY = 'UNI'
  AND TABLE_NAME = ?
ORDER BY ORDINAL_POSITION
END_SQL
    my $aaResult = $dbi->execute( $sql, [$Table] );
    return map { $_->[0] } @$aaResult;
}
#========================================================================================
# function     getTypesOfColumns
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hTypes= $dbi->getTypesOfColumns($Table);
#----------------------------------------------------------------------------------------
# description  Returns types of columns. Currently used types: bit,char,datetime,float,
#               int,money,nchar,numeric,nvarchar,text,tinyint,varchar
#----------------------------------------------------------------------------------------
# input        $Table | table name | string
# return       $hTypes | column types key: columnname value: type | ref.hash.string
#========================================================================================
sub getTypesOfColumns ($) {
    my $self = shift;
    my ($Table) = @_;

    my $dbi = GetCurrentDBHandle();
    # 'TABLE_NAME = ?' is required; you can't use '?' or `?`
    my $aaResult = $dbi->execute(<<END_SQL, [$Table]);
SELECT COLUMN_NAME, DATA_TYPE
FROM INFORMATION_SCHEMA.COLUMNS
WHERE TABLE_SCHEMA = DATABASE()
  AND TABLE_NAME = ?
END_SQL

    my %Columns = map { $_->[0] => $_->[1] } @$aaResult;
    return \%Columns;
}

#========================================================================================
# function     getGeneralizeDateTimeStatement
# state        public
#----------------------------------------------------------------------------------------
# syntax       $statement = $dbi->getGeneralizeDateTimeStatement($Column);
#----------------------------------------------------------------------------------------
# description  Returns a statement for querying datetime columns in a generalized
#               format. Instead of returning the plain column name, a convert statement
#               like 'convert(char(19), $TablePrefix$Column, 117)' is returned and
#               can be integrated into a 'SELECT $statement from table where bla' query.
#----------------------------------------------------------------------------------------
# input        $Column | column name | string
# input        $TablePrefix | prefix, which should be inserted before the column name | string
# return       $statement | the new column statement | string
#========================================================================================
sub getGeneralizeDateTimeStatement() {
    my $self = shift;
    my ($Column, $TablePrefix) = @_;

    return "DATE_FORMAT($TablePrefix$Column, '%Y/%m/%d %H:%i:%S')";
}

#========================================================================================
# function     isColumnAutoIncrement
# state        public
#----------------------------------------------------------------------------------------
# syntax       $isAutoInc = $dbi->isColumnAutoIncrement();
#----------------------------------------------------------------------------------------
# description  Returns 1 if the table has an auto increment column or an _nextid table
#----------------------------------------------------------------------------------------
# input        $Table | table name | string
# return       $Parameter | binary value | int
#========================================================================================
sub isColumnAutoIncrement () {
    my $self = shift;
    my ($Table) = @_;

    # return 0 if table name is contained in blacklist!
    return 0 if $AUTO_INCREMENT_BLACKLIST->{$Table};

    # 'TABLE_NAME = ?' is required; you can't use '?' or `?`
    my $aaResult = GetCurrentDBHandle()->execute(<<END_SQL, [$Table]);
SELECT auto_increment
FROM INFORMATION_SCHEMA.TABLES
WHERE TABLE_SCHEMA = DATABASE()
  AND TABLE_NAME = ?
END_SQL

    return 0 unless scalar @$aaResult;
    return $aaResult->[0][0] ? 1 : 0;
}

#========================================================================================
# function     isNullable
# state        public
#----------------------------------------------------------------------------------------
# syntax       $IsNullable = $dbi->isNullable($Table, $Column);
#----------------------------------------------------------------------------------------
# description  Returns true if the table column is nullable.
#               Returns false if the column is not nullable or if it does not exist.
#----------------------------------------------------------------------------------------
# input        $Table | table name | string
# input        $Column | column name | string
# return       $IsNullable | true if the column is nullable | boolean
#========================================================================================
sub isNullable () {
    my $self = shift;
    my ($Table, $Column) = @_;

    # 'TABLE_NAME = ?' is required; you can't use '?' or `?`
    my $aaResult = GetCurrentDBHandle()->execute(<<END_SQL, [$Table, $Column]);
SELECT  IS_NULLABLE
FROM INFORMATION_SCHEMA.COLUMNS
WHERE TABLE_SCHEMA = DATABASE()
  AND TABLE_NAME = ?
  AND COLUMN_NAME = ?
END_SQL

    return 0 unless scalar @$aaResult;
    return $aaResult->[0][0] eq 'YES' ? 1 : 0;
}

#========================================================================================
# function     disableForeignKeyChecks
# state        public
#----------------------------------------------------------------------------------------
# syntax       $dbi->disableForeignKeyChecks($Code);
#----------------------------------------------------------------------------------------
# description  Disables foreign key checks during execution of $Code.
#----------------------------------------------------------------------------------------
# input        $Code | code reference | code
#========================================================================================
sub disableForeignKeyChecks {
    my $self = shift;
    my ($Code) = @_;

    GetCurrentDBHandle()->disableForeignKeyChecks( $Code );
    return;
}

#========================================================================================
# function     reserveIdentifier
# state        public
#----------------------------------------------------------------------------------------
# syntax       $FirstNextID = $dbi->reserveIdentifier($Table, $CountOfRows);
#----------------------------------------------------------------------------------------
# description  Increments the AUTO_INCREMENT by $CountOfRows to reserve a block of ids.
#----------------------------------------------------------------------------------------
# input        $Table | table name | string
# input        $CountOfRows | requested row count | integer
# return       $FirstNextID | first free identifier | integer
#========================================================================================
sub reserveIdentifier {
    my $self = shift;
    my ($Table, $CountOfRows) = @_;

    my $Column = $self->getPKeyOfTable($Table);
    my $dbi = GetCurrentDBHandle();

    my $NextID;
    my $aTables = [ { 'table' => $Table, 'mode' => 'WRITE' } ];
    push @$aTables, { 'table' => $LOCK_TABLE, 'mode' => 'WRITE' } unless $Table eq $LOCK_TABLE;
    $dbi->lockTables( $aTables, sub {

        my $OldAutoIncrement = $self->autoIncrementNextID( $Table );
        $NextID = $OldAutoIncrement;

        # make sure there are no ids >= AUTO_INCREMENT
        my $MaxID = $dbi->execute("SELECT max($Column) FROM `$Table`")->[0]->[0] // 0;
        $NextID = $MaxID + 1 if $MaxID >= $NextID;

        my $NewAutoIncrement = $NextID + $CountOfRows;
        if( $NewAutoIncrement > $OldAutoIncrement ) {
            if( $Table eq 'object' ) { # EPG-23403
                $dbi->do('DELETE FROM object WHERE guid = ?', ['AUTO_INCREMENT']);
                $dbi->do(<<'END_SQL', [ $NewAutoIncrement - 1 ]);
INSERT INTO object (objectid, classid, alias, position, guid, inherit, creationdate, siteid)
VALUES (?, 0, 'AUTO_INCREMENT', 0, 'AUTO_INCREMENT', 0, NOW(), 1)
END_SQL
                $dbi->do('DELETE FROM object WHERE guid = ?', ['AUTO_INCREMENT']);
            }
            else {
                $dbi->do("ALTER TABLE `$Table` AUTO_INCREMENT = $NewAutoIncrement");
            }
        }
    });

    return $NextID
}

#========================================================================================
# function     autoIncrementNextID
# state        public
#----------------------------------------------------------------------------------------
# syntax       $AutoIncrement = $dbi->autoIncrementNextID($Table);
#----------------------------------------------------------------------------------------
# description  Returns the AUTO_INCREMENT value of the given table, i.e. the id of
#               the next row to be inserted.
#----------------------------------------------------------------------------------------
# input        $Table | table name | string
# return       $AutoIncrement | AUTO_INCREMENT value | integer
#========================================================================================
sub autoIncrementNextID {
    shift;
    my ($Table) = @_;

    my $dbi = GetCurrentDBHandle();
    # 'TABLE_NAME = ?' is required; you can't use '?' or `?`
    my $NextID = $dbi->execute(<<'END_SQL', [$Table])->[0]->[0];
SELECT AUTO_INCREMENT
FROM INFORMATION_SCHEMA.TABLES
WHERE TABLE_SCHEMA = DATABASE()
  AND TABLE_NAME = ?
END_SQL

    die "Can't get AUTO_INCREMENT value of table $Table" unless defined $NextID;
    return $NextID;
}

sub doLimit {
    my $self = shift;
    my ($RowCount, $SQL, $aParams) = @_;

    my $dbi  = GetCurrentDBHandle();
    $dbi->do( "$SQL LIMIT $RowCount", $aParams );
    return;
}

1;
