#==============================================================================
# package      DE_EPAGES::ShopTransfer::API::Delete
# state        public
#------------------------------------------------------------------------------
# description  Implementation of functions and methods related to deletion.
#==============================================================================
package DE_EPAGES::ShopTransfer::API::Delete;
use base Exporter;

use strict;

use Time::HiRes qw (time);
use Readonly;
use DE_EPAGES::Core::API::Error qw (VarsOnError);
use DE_EPAGES::Core::API::Log qw (GetLog);
use DE_EPAGES::Core::API::PerlTools qw (UsePackage);
use DE_EPAGES::Database::API::Connection qw( GetCurrentDBHandle );
use DE_EPAGES::WebInterfacePoolDB::API::ASPoolDB qw( RunOnASPoolDB );

Readonly::Scalar my $BATCHSIZE => 5000;

sub new {
    my $class = shift;

    my $DBIPackage = GetCurrentDBHandle()->driverName =~ /mysql/i
        ? 'DE_EPAGES::ShopTransfer::API::DBImySQL'
        : 'DE_EPAGES::ShopTransfer::API::DBISybase';
    UsePackage($DBIPackage);

    return bless {
        'dbh' => $DBIPackage->new
    }, $class;
}

sub db {
    return shift->{'dbh'};
}

#========================================================================================
# function     _deleteTableSequence
# state        private
#----------------------------------------------------------------------------------------
# syntax       $WithoutError = $del->_deleteTableSequence($Table, $SiteID, $hOptions, $statement);
#----------------------------------------------------------------------------------------
# description  Removes data from table of a site (shop) in blocks of given batchSize.
#               SQL uses temp tables #export_ids, #table_ids and #delete_ids.
#----------------------------------------------------------------------------------------
# input        $Table | table name | string
# input        $SiteID | site identifier | integer
# input        $hOptions | options used keys: batchSize (default 5000) | ref.hash
# input        $statement | delete statement | string
# return       $DataRemoved | true if the data could be removed from table | boolean
#========================================================================================
sub _deleteTableSequence {
    my $self = shift;
    my ($Table, $SiteID, $hOptions, $statement) = @_;

    my $BatchSize = $hOptions->{'batchSize'} || $BATCHSIZE;
    return 0 if $Table eq 'session';

    my $dbi = GetCurrentDBHandle();

    my $tableids = $dbi->tempTableName('table_ids');
    my $exportids = $dbi->tempTableName('export_ids');
    my $deleteids = $dbi->tempTableName('delete_ids');
    my $Log = GetLog();

    # if object then try to delete at all object tables
    my $Column = $self->db->getObjectColumn($Table);
    my @PrimaryKeys = $self->db->getPKeyOfTable($Table);
    $Column = $PrimaryKeys[0] if scalar @PrimaryKeys == 1 and $self->db->isColumnObjectID($Table, $PrimaryKeys[0]);

    ##no critic (ProhibitVariablesInSQLStatements)
    if ($Column ne 'shopid' and $Column ne 'siteid') {
        # clear table_ids and add identifier of table
        $dbi->do("DELETE FROM `$tableids`");
        $dbi->do("INSERT INTO `$tableids` SELECT objectid FROM `$exportids` t WHERE objectid IN( SELECT $Column FROM `$Table` )");

        my $Count = $dbi->execute("SELECT COUNT(1) FROM `$tableids`")->[0]->[0];
        $Log->debug("$Table: delete $Count objects");
        while( $Count ) {
            # clear delete_ids and move identifier from table_ids to delete_ids
            $dbi->do("DELETE FROM `$deleteids`");
            $self->db->doLimit($BatchSize, "INSERT INTO `$deleteids` SELECT objectid FROM `$tableids`");
            $dbi->do("DELETE FROM `$tableids` WHERE objectid IN (SELECT d.objectid FROM `$deleteids` d)");
            $dbi->do("DELETE FROM `$Table` WHERE $Column IN (SELECT objectid FROM `$deleteids`)");
            $Count = $dbi->execute("SELECT COUNT(1) FROM `$tableids`")->[0]->[0];
            $Log->debug("$Table: $Count rows left");
        }
    }
    else {
        my $Count = $dbi->execute("SELECT COUNT(1) FROM `$Table` WHERE $Column = ?",[$SiteID])->[0]->[0];
        $Log->debug("$Table: delete $Count rows");
        while( $Count ) {
            $self->db->doLimit($BatchSize, "DELETE FROM `$Table` WHERE $Column = ?", [$SiteID]);
            $Count = $dbi->execute("SELECT COUNT(1) FROM `$Table` WHERE $Column = ?",[$SiteID])->[0]->[0];
            $Log->debug("$Table: $Count rows left");
        }
    }
    ##use critic

    return 1;
}

#========================================================================================
# function     _buildDeleteStatement
# state        private
#----------------------------------------------------------------------------------------
# syntax       $SQL = $del->_buildDeleteStatement($Table, $hOptions);
#----------------------------------------------------------------------------------------
# description  Builds the delete statement for table. SQL includes temp table #export_ids.
#----------------------------------------------------------------------------------------
# input        $Table | table name | string
# input        $hOptions | options used keys: storePath to save table info at ini-file| ref.hash
# return       $SQL | delete statement | string
#========================================================================================
sub _buildDeleteStatement {
    my $self = shift;
    my ($Table, $hOptions) = @_;

    my $dbi = GetCurrentDBHandle();
    my $exportids = $dbi->tempTableName('export_ids');
    return "UPDATE session SET userid = NULL WHERE userid IN (SELECT objectid FROM `$exportids`)" if $Table eq 'session';

    my $Column = $self->db->getObjectColumn($Table);
    return undef if $Column eq '';
    return sprintf 'DELETE FROM `%s` WHERE %s = ?',
        $Table, $Column if $Column eq 'shopid' or $Column eq 'siteid';
    return sprintf 'DELETE FROM `%s` WHERE %s IN (SELECT objectid FROM `%s`)',
        $Table, $Column, $exportids;
}

#========================================================================================
# function     deleteTable
# state        public
#----------------------------------------------------------------------------------------
# syntax       $del->deleteTable($Table, $SiteID, $hOptions);
#----------------------------------------------------------------------------------------
# description  Removes data from table of a site (shop).
#----------------------------------------------------------------------------------------
# input        $Table | table name | string
# input        $SiteID | site identifier | integer
# input        $hOptions | options used keys: batchSize (default 5000), storePath | ref.hash
#========================================================================================
sub deleteTable {
    my $self = shift;
    my ($Table, $SiteID, $hOptions) = @_;

    my $statement = $self->_buildDeleteStatement($Table, $hOptions);
    return unless defined $statement;

    $self->_deleteTableSequence($Table, $SiteID, $hOptions, $statement);
    return;
}

#========================================================================================
# function     deleteShop
# state        public
#----------------------------------------------------------------------------------------
# syntax       $del->deleteShop($ShopAlias, $hOptions, $ShopID);
#----------------------------------------------------------------------------------------
# description  Removes all data from shop.
#----------------------------------------------------------------------------------------
# input        $ShopAlias | alias | string
# input        $hOptions | options used keys: batchSize (default 5000), storePath | ref.hash
# input        $ShopID | optional identifier | integer
#========================================================================================
sub deleteShop {
    my $self = shift;
    my ($ShopAlias, $hOptions, $ShopID) = @_;

    my $dbi = GetCurrentDBHandle();
    my $tableids = $dbi->tempTableName('table_ids');
    my $exportids = $dbi->tempTableName('export_ids');
    my $deleteids = $dbi->tempTableName('delete_ids');

    if(defined $ShopID) {
        $ShopAlias = $self->db->getAliasByID( $ShopID );
        die "Shop $ShopID does not exist" unless defined $ShopAlias;
    }
    else {
        $ShopID = $self->db->getIDByPath('Shops', $ShopAlias);
        die "Shop /Shops/$ShopAlias does not exist" unless defined $ShopID;
    }

    # delete the shop from ASPoolDB
    my $ShopGUID = $self->db->getObjectGUID($ShopID);
    RunOnASPoolDB( sub {
        my $ASPoolDB = shift;
        $ASPoolDB->deleteSite($ShopGUID);
    });

    {
        # delete temporary tables if they still exist
        local $SIG{__WARN__} = sub {};
        ## no critic qw(RequireExistsError)
        eval { $dbi->dropTable($exportids) };
        eval { $dbi->dropTable($deleteids) };
        eval { $dbi->dropTable($tableids) };
        ## use critic
    }

    eval {
        print "find object of shop $ShopAlias\n" if $hOptions->{'verbose'};

        $dbi->createTempTable('export_ids (objectid int)');
        $dbi->createTempTable('delete_ids (objectid int PRIMARY KEY)');
        $dbi->createTempTable('table_ids (objectid int PRIMARY KEY)');
        $dbi->atIsolationLevel('READ COMMITTED', sub {
            my $time0 = time;

            GetLog()->debug(sprintf 'insert objectids of shop (%d: %s) into temp table ',$ShopID, $ShopAlias);
            $dbi->do("INSERT INTO `$exportids` (objectid) SELECT objectid FROM object WHERE siteid = ?", [$ShopID]);

            foreach my $Table (@{$self->db->getTables}) {
                print "delete table $Table " if $hOptions->{'verbose'};
                my $time1 = time;
                GetLog()->debug("Starting delete at table $Table");
                $self->db->disableForeignKeyChecks( sub {
                    $self->deleteTable($Table, $ShopID, $hOptions);
                });
                printf "(%.1f)\n", time - $time1 if $hOptions->{'verbose'};
            }
            GetLog()->info(sprintf "shop %s removed (%.1f)\n", $ShopAlias, time - $time0);
        });
        $dbi->dropTable($exportids);
        $dbi->dropTable($deleteids);
        $dbi->dropTable($tableids);
    };
    VarsOnError({'ShopAlias' => $ShopAlias});
}

#========================================================================================
# function     deleteAllShops
# state        public
#----------------------------------------------------------------------------------------
# syntax       $del->deleteAllShops($hOptions);
#----------------------------------------------------------------------------------------
# description  Removes all shops.
#----------------------------------------------------------------------------------------
# input        $hOptions | options used keys: batchSize (default 5000), storePath | ref.hash
#========================================================================================
sub deleteAllShops {
    my $self = shift;
    my ($hOptions) = @_;

    my $dbi = GetCurrentDBHandle();
    my $aShopAliases = map { $_->[0] } @{$dbi->execute(<<END_SQL)};
SELECT s_o.alias
FROM shop s
INNER JOIN object s_o ON s.shopid = s_o.objectid
INNER JOIN object o ON s.shopid = o.siteid
GROUP BY s.shopid, s_o.alias
ORDER BY COUNT(o.siteid)
END_SQL

    $self->deleteShop($_, $hOptions) foreach reverse @$aShopAliases;
}

1;
