#==============================================================================
# package      DE_EPAGES::ShopTransfer::API::ImportmySQL
# state        public
#----------------------------------------------------------------------------------------
# description  MySQL specific import functions.
#==============================================================================
package DE_EPAGES::ShopTransfer::API::ImportmySQL;
use base DE_EPAGES::ShopTransfer::API::Import;

use strict;

use DE_EPAGES::Core::API::File qw (ExistsFile);
use DE_EPAGES::Database::API::Connection qw (GetCurrentDBHandle);
use DE_EPAGES::ShopTransfer::API::BCP qw (CreateBCPFileIterator);
use DE_EPAGES::ShopTransfer::API::Constants qw (BCP_ROW_DELIMITER);
use DE_EPAGES::ShopTransfer::API::DBImySQL;

sub new {
    my $class = shift;
    my %options;
    $options{'dbh'} = DE_EPAGES::ShopTransfer::API::DBImySQL->new();

    my $self = bless \%options, $class;
    return $self;
}

#========================================================================================
# function     importTable
# state        public
#----------------------------------------------------------------------------------------
# syntax       $BcpExitCode = $import->importTable($Table, $ShopAlias, $hOptions);
#----------------------------------------------------------------------------------------
# description  Imports table to database.
#----------------------------------------------------------------------------------------
# input        $Table | table name | string
# input        $ShopAlias | shop alias | string
# input        $hOptions | options, keys: versions, storePath, db_name, bcpParameter, encoding | ref.hash
# return       $BcpExitCode | exit code, 0 on success | integer
#========================================================================================
sub importTable {
    my $self = shift;
    my($Table, $ShopAlias, $hOptions) = @_;

    my $FileName = join '/', $hOptions->{'storePath'},'import',$ShopAlias, $Table.'.bcp';
    return 0 unless ExistsFile($FileName);

    # build sql to insert a row
    my $IsAutoIncrement = ($self->isDynamicTable($Table) && $ShopAlias ne 'SYSTEM') ? 1 : 0;
    my @Columns = $self->getDBIInstance()->getColumns($Table);
    shift @Columns if $IsAutoIncrement; # auto-increment column should be the first one
    my @NullableIndex = grep { $self->getDBIInstance()->isNullable($Table,$Columns[$_]) } (0..$#Columns);
    my $Columns = join ', ', @Columns;
    my $Wildcards = join ',', map { '?' } @Columns;
    my $sqlInsert = "INSERT INTO `$Table` ($Columns) VALUES "; ## no critic qw(ProhibitVariablesInSQLStatements)

    local $/ = BCP_ROW_DELIMITER;
    my $Iterator = CreateBCPFileIterator( $FileName, $hOptions->{'encoding'} );
    my $dbi = GetCurrentDBHandle();
    $self->getDBIInstance()->disableForeignKeyChecks( sub {
        $dbi->transaction( sub {

            my $MaxRows = 100; ## no critic qw(ProhibitMagicNumbers)
            my $aRows = _ReadBulk( $Iterator, $MaxRows );
            my $RowCount = 0;
            my $sql; # reuse the same SQL as long as we have the same number of rows

            while( scalar @$aRows > 0 ) {

                if( scalar @$aRows != $RowCount ) {
                    $RowCount = scalar @$aRows;
                    $sql = $sqlInsert . join(', ', map { "($Wildcards)" } (1..$RowCount));
                }

                my @Values;
                foreach my $aRow ( @$aRows ) {
                    shift @$aRow if $IsAutoIncrement;
                    do { $_ = undef if $_ eq '' } foreach @{$aRow}[@NullableIndex];
                    push @Values, @$aRow;
                }

                $dbi->do( $sql, \@Values );

                $aRows = _ReadBulk( $Iterator, $MaxRows );
            }
        });
    });

    return 0;
}

# read up to $MaxRows rows from the iterator
sub _ReadBulk {
    my ($Iterator, $MaxRows) = @_;

    my $aRow;
    my @Rows;
    while( scalar @Rows < $MaxRows && ($aRow = $Iterator->next) ) {
        push @Rows, $aRow;
    }

    return \@Rows;
}

1;
