#========================================================================================
# package      DE_EPAGES::ShopTransfer::API::WebService::ShopTransferService
# state        public
#----------------------------------------------------------------------------------------
# description  Provides web service access to the ShopTransfer cartridge.
#----------------------------------------------------------------------------------------
# example      use DE_EPAGES::WebService::API::Client;
#               my $ShopTransferService = DE_EPAGES::WebService::API::Client
#                   ->uri('urn://epages.de/WebService/ShopTransferService/2009/07')
#                   ->proxy("http://%2fUsers%2f$Login:$Password@localhost:8080/epages/Store.soap");
#               $ShopTransferService->delete( $ShopID );
#========================================================================================
package DE_EPAGES::ShopTransfer::API::WebService::ShopTransferService;
use base DE_EPAGES::WebService::API::WebService::BaseAsyncService;

use strict;
use Config;
use Encode qw ( encode );

use DE_EPAGES::Core::API::Log qw ( LogDebug );
use DE_EPAGES::Core::API::Error qw ( Error ExistsError VarsOnError );
use DE_EPAGES::Core::API::File qw ( ExistsDirectory CreateDirectory ConvertPathChars SetAccessRights
                                    ExistsFile MoveFile DeleteFile WriteFile GetFileContent );
use DE_EPAGES::Core::API::TestParameter qw ( TestInteger TestString );
use DE_EPAGES::Core::API::SystemCommand qw (PackTar UnpackTar);
use DE_EPAGES::Object::API::Factory qw ( LoadObject LoadRootObject LoadObjectByPath ExistsObjectByPath JoinObjectPath);
use DE_EPAGES::Database::API::Connection qw ( GetCurrentDBHandle );
use DE_EPAGES::ShopTransfer::API::BCP qw ( BCPOutMapper );
use DE_EPAGES::ShopTransfer::API::Export;

#ERRORDEFINITION
sub ErrorDefinition {
    return {
        'ShopWithSameAliasExist' =>  'shop with alias #Alias still exist',
        'DirNotFound'            =>  'directory #Dir not found',
    }
}
#ERRORDEFINITION END

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Service = DE_EPAGES::ShopTransfer::API::WebService::ShopTransferService->new( %Options );
#----------------------------------------------------------------------------------------
# description  Creates a new web service object as a blessed hash reference and adds
#               the values from the %Options hash.
#----------------------------------------------------------------------------------------
# input        %Options | user-defined values | hash
# return       $Service | web service object | object
#========================================================================================
sub new {
    my $self = shift;
    my @Options = @_;
    $self->SUPER::new((
        'ServiceUrn' => 'urn://epages.de/WebService/ShopTransferService/2009/07',
        @Options,
    ));
}

sub getStatus {
    my $self = shift;
    my ($SessionID) = @_;
    return $self->getSessionStatus($SessionID);
}

sub startExport {
    my $self = shift;
    my ( $ShopAlias, $ExportPath, $Options ) = @_;
    TestString( 'ShopAlias', $ShopAlias );
    TestString( 'ExportPath', $ExportPath );

    my $WSession = $self->newSession();
    $WSession->setSub( $self, \&exportShop, [$ShopAlias, $ExportPath, $Options] );

    return $WSession->alias;
}

sub exportShop {
    my $self = shift;
    my ( $ShopAlias, $ExportPath, $Options, $WSession ) = @_;

    TestString( 'ShopAlias', $ShopAlias );
    TestString( 'ExportPath', $ExportPath );
    $self->{'WSession'} = $WSession;

    my $StoreName = LoadRootObject()->get('StoreName');

    # create the target directory
    CreateDirectory("$ExportPath/out/$StoreName");
    $self->setStatusAndLog("directory $ExportPath/out/$StoreName created");

    # remove tag file if exists (used for flex)
    my $TagFileName = "$ExportPath/export_ok.tgt";

    # set additional shop info
    my $hInfo = {
        'storeName' => $StoreName,
        'storePath' => "$ExportPath/out/$StoreName",
    };

    # export Shop
    $self->ExportShopData( $ShopAlias, $hInfo );
    $self->setStatusAndLog("shop export $ShopAlias complete");

    # export System
    $self->ExportSystem( $hInfo, $WSession );
    $self->setStatusAndLog('system export complete');

    # packing files
    PackTar("$ExportPath/$StoreName.$ShopAlias.tar.gz", '.',
            ["$ExportPath/out/$StoreName/export/$ShopAlias", "$ExportPath/out/$StoreName/export/SYSTEM"]);

    # create a file as an indicator for a complete shop export (used for flex)
    my $TagContent = "$StoreName $ShopAlias";
    WriteFile( $TagFileName, \$TagContent, {'append'=>1} );

    my $user = $ShopAlias;
    my $passwd = join '',map{('a'..'z',0..9)[rand 36]} 1..8; ##no critic qw(ProhibitMagicNumbers)
    $passwd = $Options->{passwd} if $Options->{passwd};
    # create .htaccess & .htpasswd file
    my $htacessFile = "$ExportPath/\.htaccess";
    my $htpasswdFile = "$ExportPath/\.htpasswd";
    my $htacessContent = "AuthUserFile $htpasswdFile\n".
        'AuthType Basic'."\n".
        'AuthBasicProvider file'."\n".
        'Require valid-user'."\n";
    my $htpasswdContent = "$user:"._CryptHtPasswd($passwd)."\n";
    WriteFile( $htacessFile, \$htacessContent );
    WriteFile( $htpasswdFile, \$htpasswdContent );


    # return file path and options
    $self->setStatusAndLog("export complete: $ExportPath/$StoreName.$ShopAlias.tar.gz $user $passwd");

    return 1;
}

sub startImport {
    my $self = shift;
    my ( $ShopAlias, $ExportPath, $Options ) = @_;
    TestString( 'ShopAlias', $ShopAlias );
    TestString( 'ExportPath', $ExportPath );

    my $WSession = $self->newSession();
    $WSession->setSub( $self, \&importShop, [$ShopAlias, $ExportPath, $Options] );

    return $WSession->alias;
}

sub importShop {
    my $self = shift;
    my ( $ShopAlias, $ExportPath, $Options, $WSession ) = @_;
    TestString( 'ShopAlias', $ShopAlias );
    TestString( 'ExportPath', $ExportPath );
    $self->{'WSession'} = $WSession;

    my $StoreName = LoadRootObject()->get('StoreName');
    if ( ExistsObjectByPath(JoinObjectPath(['', 'Shops', $ShopAlias]))) {

        if ( $Options->{'deleteShopWithSameAlias'} ) {
            my $ShopRef = LoadObjectByPath('/Shops')->child($ShopAlias);
            my $ShopID = $ShopRef->get('StoreObjectID');
            $self->delete( $ShopID );
            $self->setStatusAndLog("old shop with alias $ShopAlias deleted");

        } elsif ( $Options->{'renameShopWithSameAlias'} ) {
            my $ShopReName = 'OLD_Shop';

            #remame shop at site a la renameShopAtSite.pl -storename Store -shop $Alias -newalias OLD_Shop

            #update shorturl of old shop a la renameShopAtStore.pl -storename Store -shop $Alias -newalias OLD_Shop

            $self->setStatusAndLog("shop with alias $ShopAlias renamed to $ShopReName");

        } else {
            Error( 'ShopWithSameAliasExist', {'Alias'=>$ShopAlias} );
        }
    }

    # set additional shop info
    my $hInfo = {
        'storeName' => $StoreName,
        'storePath' => "$ExportPath/out/$StoreName",
    };


    # export System
    $self->ExportSystem( $hInfo, $WSession );
    $self->setStatusAndLog('system export complete');

    # transform shop data
    $hInfo->{'newguid'} = 1;
    $hInfo->{'ShopAlias'} = $ShopAlias;
    $self->TransformShop( $hInfo, $WSession );
    $self->setStatusAndLog("shop data of $ShopAlias transformed at $hInfo->{'storePath'}");

    # import shop
    $self->ImportShopData( $hInfo, $WSession );
    $self->setStatusAndLog("shop $ShopAlias imported");

# this follow function need to remove to a site based WS
#
#    # create shop ref
#    $self->CreateShopRef( $hInfo, $WSession );
#    $self->setStatusAndLog("shop refrence for $ShopAlias created");
#
#    # register shop in pool db
#    $self->InsertPoolDB( $hInfo, $WSession );
#    $self->setStatusAndLog("shop $ShopAlias registered at PoolDB");
#
#    # set master product references
#    $self->SetVariationStrings( $hInfo, $WSession );
#    $self->setStatusAndLog("set master product reference of shop $ShopAlias complete");
#
#    # open Shop
#    my $ShopRef = LoadObjectByPath( "/Shops/$ShopAlias" );
#    $ShopRef->set({ClosedByMerchant => 0 });
#    $self->setStatusAndLog("shop /Shops/$ShopAlias opened");
#

}


sub ExportSystem {
    my $self = shift;
    my ($hOptions) = @_;

    my $Directory = join '/', $hOptions->{'storePath'},'export','SYSTEM';
    CreateDirectory($Directory);
    $hOptions->{'outPath'} = $Directory;
    my $dbi = GetCurrentDBHandle();
    my $exportids = $dbi->tempTableName('export_ids');

    $dbi->createTempTable('export_ids (objectid int)');
    $self->setStatusAndLog("insert objectids into temp table $exportids");
    $dbi->do("insert into `$exportids` (objectid) select objectid from object where siteid = ?", [1]);

    my $export = 1;
    my $expt = DE_EPAGES::ShopTransfer::API::Export->new();
    my $exportDbi = $expt->getDBIInstance();
    foreach my $Table (@{$exportDbi->systemTables}) {
        $self->setStatusAndLog("export table $Table");
        $export = 0 unless $expt->exportTable($Table, 1, $hOptions);
    }
    $dbi->dropTable($exportids);
    $self->setStatusAndLog('system has already been exported') unless $export;
    $self->setStatusAndLog('export system complete');
}

sub ExportShopData {
    my $self = shift;
    my ($ShopAlias, $hOptions) = @_;

    my $Directory = join '/', $hOptions->{'storePath'},'export',$ShopAlias;
    CreateDirectory($Directory);
    $hOptions->{'outPath'} = $Directory;

    my $export = 1;
    if (not $hOptions->{'onlyfiles'}) {
        my $dbi = GetCurrentDBHandle();
        my $expt = DE_EPAGES::ShopTransfer::API::Export->new();
        my $exportDbi = $expt->getDBIInstance();
        my $ShopID = $exportDbi->getIDByPath('Shops', $ShopAlias);
        my $exportids = $dbi->tempTableName('export_ids');
        $dbi->createTempTable('export_ids (objectid int)');

        $self->setStatusAndLog("insert objectids into temp table $exportids");
        $dbi->do("insert into `$exportids` (objectid) select objectid from object where siteid = ?", [$ShopID]);

        foreach my $Table (@{$exportDbi->shopTables()}) {
            $self->setStatusAndLog("export table $Table");
            $export = $expt->exportTable($Table, $ShopID, $hOptions);
        }
        $dbi->dropTable($exportids);
    }

    # collect public and private files
    if (not $hOptions->{'onlydata'}) {
        my %SourceDir = (private => $ENV{EPAGES_STORES}, public => $ENV{EPAGES_WEBROOT});
        foreach my $Source (keys %SourceDir) {
            my $ShopsDir = "$SourceDir{$Source}/$hOptions->{storeName}/Shops";
            next unless ExistsDirectory("$ShopsDir/$ShopAlias");
            PackTar("$Directory/$Source.tar", $ShopsDir, [$ShopAlias]);
        }
    }
    $self->setStatusAndLog("shop $ShopAlias has already been exported") unless $export;
    $self->setStatusAndLog("shop $ShopAlias export complete");
}

#transformation of shop a la transform.pl -storename Store -source Store -shop $Alias -newguid
sub TransformShop {
    my $self = shift;
    my ($hOptions) = @_;
    my $ShopAlias = $hOptions->{'ShopAlias'};
    my $NewShopAlias = $hOptions->{'NewAlias'};
    $NewShopAlias = $ShopAlias unless defined $NewShopAlias;

    # create target directory
    CreateDirectory(join '/', $hOptions->{'storePath' },'import',$ShopAlias);

    _TransferShopDB($ShopAlias, $NewShopAlias, $hOptions) unless $hOptions->{'onlyfiles'};
    _TransferShopFiles($ShopAlias, $hOptions) unless $hOptions->{'onlydata'};
    _TransferShopName($ShopAlias, $NewShopAlias, $hOptions) unless $hOptions->{'onlyfiles'};

    $self->setStatusAndLog("transfere shop $ShopAlias");
}

#========================================================================================
# function     _TransferShopDB
# state        private
#----------------------------------------------------------------------------------------
# syntax       _TransferShopDB($ShopAlias, $hOptions)
#----------------------------------------------------------------------------------------
# description  Replaces old identifier of shop bcp files.
#----------------------------------------------------------------------------------------
# input        $ShopAlias | shop alias | string
# input        $NewShopAlias | new hop alias | string
# input        $hOptions | options, keys: versions, sourcePath,storePath,encoding,verbose | ref.hash
#========================================================================================
sub _TransferShopDB($$) {
    my ($ShopAlias, $NewShopAlias, $hOptions) = @_;

    my $SourceDirectory = join '/', $hOptions->{'sourcePath'},'export',$ShopAlias;
    Error('DirNotFound', {'Dir'=>$SourceDirectory}) unless ExistsDirectory($SourceDirectory);

    my $trans = DE_EPAGES::ShopTransfer::API::Transform->new();
    my $transDbi = $trans->getDBIInstance();

    my $hNonExistentLocales = $trans->getVisibleLocales($hOptions);
    delete $hOptions->{'ShopMappings'};
    foreach my $Table (reverse @{$transDbi->shopTables}) {
        next unless ExistsFile($SourceDirectory.'/'.$Table.'.bcp');
        $trans->transferTable($Table, $ShopAlias, $NewShopAlias, $hOptions, $hNonExistentLocales);
    }
}

#========================================================================================
# function     _TransferShopFiles
# state        private
#----------------------------------------------------------------------------------------
# syntax       _TransferShopFiles($ShopAlias, $hOptions)
#----------------------------------------------------------------------------------------
# description  Moves private and public files (pack at tar) to import directory.
#----------------------------------------------------------------------------------------
# input        $ShopAlias | shop alias | string
# input        $hOptions | options, keys: sourcePath,storePath | ref.hash
#========================================================================================
sub _TransferShopFiles($$) {
    my ($ShopAlias, $hOptions) = @_;

    my $TargetDirectory = join '/', $hOptions->{'storePath' },'import',$ShopAlias;
    my $SourceDirectory = join '/', $hOptions->{'sourcePath'},'export',$ShopAlias;
    if (not ExistsFile($TargetDirectory.'/private.tar') and ExistsFile($SourceDirectory.'/private.tar')) {
        MoveFile($SourceDirectory.'/private.tar', $TargetDirectory.'/private.tar');
    }
    if (not ExistsFile($TargetDirectory.'/public.tar') and ExistsFile($SourceDirectory.'/public.tar')) {
        MoveFile($SourceDirectory.'/public.tar', $TargetDirectory.'/public.tar');
    }
}

#========================================================================================
# function     _TransferShopName
# state        private
#----------------------------------------------------------------------------------------
# syntax       _TransferShopName($OldShopAlias, $NewShopAlias, $hOptions)
#----------------------------------------------------------------------------------------
# description  Replaces shop alias and store name at localized/text/string attributes.
#----------------------------------------------------------------------------------------
# input        $OldShopAlias | old shop alias | string
# input        $NewShopAlias | new shop alias | string
# input        $hOptions | options, keys: sourcePath,storePath,sourceStoreName,storeName,encoding | ref.hash
# return       $return | returns 0 if $OldPath eq $NewPath | integer
#========================================================================================
sub _TransferShopName($$$) {
    my ($OldShopAlias, $NewShopAlias, $hOptions) = @_;

    my $OldPath = encode('utf-8', sprintf '/WebRoot/%s/Shops/%s/',$hOptions->{'sourceStoreName'}, ConvertPathChars($OldShopAlias));
    my $NewPath = encode('utf-8', sprintf '/WebRoot/%s/Shops/%s/',$hOptions->{'storeName'}, ConvertPathChars($NewShopAlias));
    return if $OldPath eq $NewPath;

    my $OldUriAlias = uri_escape_utf8( $OldShopAlias );
    my $NewUriAlias = uri_escape_utf8( $NewShopAlias );

    my $trans = DE_EPAGES::ShopTransfer::API::Transform->new();

    foreach my $Table (qw (stringattribute localizedstringattribute)) {
        my $SourceFile = $trans->getSourceFile($hOptions->{'sourcePath'}, $OldShopAlias, $Table, $hOptions);
        next unless ExistsFile($SourceFile);
        my $TargetFile = join ('/', $hOptions->{'storePath'},'import',$OldShopAlias,$Table.'.bcp');
        if (ExistsFile($TargetFile)) {
            $SourceFile = join ('/', $hOptions->{'storePath'},'import',$OldShopAlias,$Table.'.bcp.shopname');
            MoveFile($TargetFile, $SourceFile);
        }
        eval {
            BCPOutMapper($SourceFile, $TargetFile,
                sub {
                    return map {
                        s/\Q$OldPath\E/$NewPath/g;
                        s/https?:\/\/[\w\.]+\/WebRoot\//\/WebRoot\//gx;
                        s/((\/|%2F)Shops(\/|%2F))\Q$OldUriAlias\E/$1$NewUriAlias/gx; ## no critic qw(ProhibitUnusedCapture)
                        $_;
                    } @_;
                },
                $hOptions->{'encoding'}
            );
        };
        if (ExistsError()) {
            VarsOnError({'SourceFile'=>$SourceFile});
            MoveFile($TargetFile,$TargetFile.'.error');
            MoveFile($SourceFile,$TargetFile) if $SourceFile =~ /\.shopname$/;
        } else {
            DeleteFile($SourceFile) if $SourceFile =~ /\.shopname$/;
        }
    }
}



#import shop a la import.pl -storename Store -shop $Alias
sub ImportShopData {
    my $self = shift;
    my ($hOptions) = @_;
    my $ShopAlias = $hOptions->{'ShopAlias'};
    my $import = DE_EPAGES::ShopTransfer::API::Import->new();

    if (not $hOptions->{'onlyfiles'}) {
        my $importDbi = $import->getDBIInstance();

        foreach my $Table (reverse @{$importDbi->shopTables}) {
            if ($import->importTable($Table, $ShopAlias, $hOptions)) {
                my $rContent = GetFileContent("$hOptions->{'filePath'}/bcp_debug.log");
                Error('IMPORT_FAILED', { 'Message'=>$$rContent, 'ShopAlias'=>$ShopAlias, 'Table'=>$Table }) if $$rContent =~ /\S/;
            }
            if (ExistsFile("$hOptions->{'filePath'}/bcp_error.log")) {
                my $rContent = GetFileContent("$hOptions->{'filePath'}/bcp_error.log");
                Error('IMPORT_FAILED', { 'Message'=>$$rContent, 'ShopAlias'=>$ShopAlias, 'Table'=>$Table }) if $$rContent =~ /\S/;
            }
        }
    }

    # collect public and private files
    if (not $hOptions->{'onlydata'}) {
        my $Directory = "$hOptions->{storePath}/import/$ShopAlias";
        my %SourceDir = (private => $ENV{EPAGES_STORES}, public => $ENV{EPAGES_WEBROOT});
        foreach my $Source (keys %SourceDir) {
            next unless ExistsFile("$Directory/$Source.tar");
            my $ShopsDir = "$SourceDir{$Source}/$hOptions->{storeName}/Shops";
            my $aPackedFiles = UnpackTar("$Directory/$Source.tar", $ShopsDir);
            SetAccessRights("$ShopsDir/$_") foreach @$aPackedFiles;
        }
    }
}

#generate shop reference a la createShopRef.pl -storename Store -shop $Alias
sub CreateShopRef {
    my $self = shift;
    my ($hOptions) = @_;
#    my $StoreName = $hOptions->{'storeName'};
#    my $Alias = $hOptions->{'ShopAlias'};

}

# register shop in PoolDB a la insertPoolDB.pl -storename Store -shop $Alias
sub InsertPoolDB {
    my $self = shift;
    my ($hOptions) = @_;
#    my $StoreName = $hOptions->{'storeName'};
#    my $Alias = $hOptions->{'ShopAlias'};

}

# set master product references for variations a la setVariationStrings.pl -storename Store -shop $Alias
sub SetVariationStrings {
    my $self = shift;
    my ($hOptions) = @_;
#    my $StoreName = $hOptions->{'storeName'};
#    my $Alias = $hOptions->{'ShopAlias'};

}

sub deleteShop {
    my $self = shift;
    my ($ShopID) = @_;
    TestInteger( 'ShopID', $ShopID );

    # delete all files
    LoadObject( $ShopID )->deleteFiles;

    # delete all data from the database
    #my %Options;
    #DeleteShop( undef, \%Options, $ShopID );
}

sub setStatusAndLog {
    my $self = shift;
    my ( $StatusMessage ) = @_;

    if ( $self->{'WSession'} ) {
        my $WSession = $self->{'WSession'};
        my ($currentStatus) = $WSession->status();
        $WSession->status( ++$currentStatus, $StatusMessage );
    }

    LogDebug( 'shopTransferService', $StatusMessage );

    return 1;
}

#crypt password for .htpasswd file
sub _CryptHtPasswd {
    my ( $passwd, $salt ) = @_;

    my @chars  = ( '.', '/', 0 .. 9, 'A' .. 'Z', 'a' .. 'z' );
    my $Magic = '$apr1$';    ##no critic qw(RequireInterpolationOfMetachars) # Apache specific Magic chars
    my $cryptType = (  $^O =~ /^MSWin/i ) ? 'MD5' : 'crypt';

    if ( $salt && $cryptType =~ /MD5/i && $salt =~ /^\Q$Magic/ ) {

        # Borrowed from Crypt::PasswdMD5
        $salt =~ s/^\Q$Magic//;       # Take care of the magic string if present
        $salt =~ s/^(.*)\$.*$/$1/;    # Salt can have up to 8 chars...
        $salt = substr( $salt, 0, 8 );    # That means no more than 8 chars too.
                                          # For old crypt only
    }
    elsif ( $salt && $cryptType =~ /crypt/i ) {
        if ($salt =~ /\$2a\$\d+\$(.{23})/x) {
            $salt = $1;
        } else {
            # Make sure only use 2 chars
            $salt = substr( $salt, 0, 2 );
        }
    }
    else {

# If we use MD5, create apache MD5 with 8 char salt: 3 randoms, 5 dots
        if ( $cryptType =~ /MD5/i ) {
            $salt =
              join ( '', map { $chars[ int rand @chars ] } ( 0 .. 2 ) )
              . '.' x 5;

            # Otherwise fallback to standard archaic crypt
        }
        else {
            $salt = join ( '', map { $chars[ int rand @chars ] } ( 0 .. 1 ) );
        }
    }

    if ( $cryptType =~ /MD5/i ) {
                require Crypt::PasswdMD5;
        return Crypt::PasswdMD5::apache_md5_crypt( $passwd, $salt );
    }
    else {
        return crypt( $passwd, $salt );
    }
}

1;
