use strict;
use Config;
use Getopt::Long;
use Time::HiRes qw (time);
use DE_EPAGES::Core::API::File qw (CreateDirectory ExistsDirectory);
use DE_EPAGES::Core::API::Script  qw( RunScript );
use DE_EPAGES::Core::API::SystemCommand qw (PackTar);
use DE_EPAGES::Database::API::Connection qw( RunOnStore GetCurrentDBHandle);
use DE_EPAGES::ShopTransfer::API::Common qw( GetShopList );
use DE_EPAGES::ShopTransfer::API::Export;
use DE_EPAGES::ShopTransfer::API::DataEncryption qw (DecryptShop);

sub ExportShop($$) {
    my ($ShopAlias, $hOptions) = @_;

    my $Directory = join '/', $hOptions->{'storePath'},'export',$ShopAlias;
    CreateDirectory($Directory);
    $hOptions->{'outPath'} = $Directory;

    my $export = 1;
    my $time0 = time;
    my $dbi = GetCurrentDBHandle();
    if (not $hOptions->{'onlyfiles'}) {

        #workaround empty passwords of anonymous user
        $dbi->do('UPDATE users SET password = \' \' WHERE password = \'\'');

        my $expt = DE_EPAGES::ShopTransfer::API::Export->new();
        my $exportDbi = $expt->getDBIInstance();
        my $ShopID = $exportDbi->getIDByPath('Shops', $ShopAlias);
        die "Shop $ShopAlias doesn't exist" unless defined $ShopID;
        my $exportids = $dbi->tempTableName('export_ids');

        $dbi->createTempTable('export_ids (objectid int)');
        print 'insert objectids into temp table ' if $hOptions->{'verbose'};
        $dbi->do("insert into `$exportids` (objectid) select objectid from object where siteid = ?", [$ShopID]);
        printf "(%.1f)\n", time - $time0 if $hOptions->{'verbose'};
        foreach my $Table (@{$exportDbi->shopTables}) {
            my $time1 = time;
            printf 'export table %s ', $Table if $hOptions->{'verbose'};
            $export = $expt->exportTable($Table, $ShopID, $hOptions);
            printf "(%.1f)\n", time - $time1 if $hOptions->{'verbose'};
        }
        $dbi->dropTable($exportids);
    }
    # collect public and private files
    if (not $hOptions->{'onlydata'}) {
        my %SourceDir = (private => $ENV{EPAGES_STORES}, public => $ENV{EPAGES_WEBROOT});
        foreach my $Source (keys %SourceDir) {
            my $ShopsDir = "$SourceDir{$Source}/$hOptions->{storeName}/Shops";
            next unless ExistsDirectory("$ShopsDir/$ShopAlias");
            PackTar("$Directory/$Source.tar", $ShopsDir, [$ShopAlias],
                    { 'Print' => $hOptions->{'verbose'} });
        }
    }
    # decrypt shop data
    DecryptShop( $ShopAlias, $hOptions) if defined $dbi->{'key'};

    printf "shop %s has already been exported\n", $ShopAlias unless $export;
    printf "export shop %s (%.1f)\n", $ShopAlias, time - $time0;
    return;
}

sub ExportSystem($) {
    my ($hOptions) = @_;

    my $Directory = join '/', $hOptions->{'storePath'},'export','SYSTEM';
    CreateDirectory($Directory);
    $hOptions->{'outPath'} = $Directory;
    my $dbi = GetCurrentDBHandle();
    my $exportids = $dbi->tempTableName('export_ids');

    $dbi->createTempTable('export_ids (objectid int)');
    print "insert objectids into temp table $exportids" if $hOptions->{'verbose'};
    my $export = 1;
    my $time0 = time;
    $dbi->do("insert into `$exportids` (objectid) select objectid from object where siteid = ?", [1]);
    printf "(%.1f)\n", time - $time0 if $hOptions->{'verbose'};
    my $expt = DE_EPAGES::ShopTransfer::API::Export->new();
    my $exportDbi = $expt->getDBIInstance();

    foreach my $Table (@{$exportDbi->systemTables}) {
        my $time1 = time;
        printf 'export table %s ', $Table if $hOptions->{'verbose'};
        $export = 0 unless $expt->exportTable($Table, 1, $hOptions);
        printf "(%.1f)\n", time - $time1 if $hOptions->{'verbose'};
    }
    $dbi->dropTable($exportids);
    printf "system has already been exported\n" unless $export;
    printf "export system (%.1f)\n", time - $time0;
    return;
}

sub ExportAllShops ($) {
    my ($hOptions) = @_;
    my $dbi = GetCurrentDBHandle();
    my $aaResult = $dbi->execute(<<'END_SQL');
SELECT s_o.alias
FROM shop s
INNER JOIN object s_o ON s.shopid = s_o.objectid
INNER JOIN object o ON s.shopid = o.siteid
GROUP BY s.shopid, s_o.alias
ORDER BY COUNT(o.siteid)
END_SQL
    ExportShop($_->[0], $hOptions) foreach @$aaResult;
    return;
}

sub Main {

    local $| = 1;

    my $Encoding  = 'utf8';
    my $EncodingList  = 'utf8';
    my $FilePath  = '.';
    my $List;
    my $Password;
    my $ShopAlias;
    my $StoreName;

    my $All;
    my $Help;
    my $OnlyData;
    my $OnlyFiles;
    my $System;
    my $Verbose;
    my $Replace;

    GetOptions(
        'encoding=s'    => \$Encoding,
        'encodinglist=s'=> \$EncodingList,
        'filepath=s'    => \$FilePath,
        'list=s'        => \$List,
        'passwd=s'      => \$Password,
        'shop=s'        => \$ShopAlias,
        'storename=s'   => \$StoreName,

        'all'           => \$All,
        'help'          => \$Help,
        'onlydata'      => \$OnlyData,
        'onlyfiles'     => \$OnlyFiles,
        'system'        => \$System,
        'verbose'       => \$Verbose,
        'replace'       => \$Replace,

    );
    usage() if $Help;
    unless( $ShopAlias or $System or $All or $List) {
        print STDERR "Missing parameter -shop, -system, -all or -list\n\n";
        usage();
    }
    unless($StoreName) {
        print STDERR "Missing parameter -storename\n\n";
        usage();
    }

    my $time1 = time;
    $FilePath .= '/out';

    RunOnStore(
        Store       => $StoreName,
        DBPassword  => $Password,
        Sub         => sub  {
            CreateDirectory("$FilePath/$StoreName");
            my $hInfo = {
                'storeName' => $StoreName,
                'encoding'  => $Encoding,
                'filePath'  => $FilePath,
                'storePath' => "$FilePath/$StoreName",
                'onlyfiles' => $OnlyFiles,
                'onlydata'  => $OnlyData,
                'verbose'   => $Verbose,
                'replace'   => $Replace,
            };
            if($System) {
                ExportSystem($hInfo);
            }
            if ($All) {
                ExportSystem($hInfo);
                ExportAllShops($hInfo);
            } elsif ($List) {
                ExportShop($_, $hInfo) foreach @{GetShopList($List, undef, $EncodingList)};
            } elsif ($ShopAlias) {
                ExportShop($ShopAlias,$hInfo);
            }
        },
    );
    my $time2 = time;
    printf "need %.1f seconds\n", $time2-$time1;
    return;
}

sub usage {
    print <<END_USAGE;
Usage:
    perl $0 [options] [flags]

options:
    -encoding   encode character                              (utf8)
    -encodinglist  encode character for list                  (utf8)
    -filepath   output directory name                         (.)
    -list       list of shop aliases                          required
    -passwd     database user password                        ()
    -shop       shop alias                                    required
    -storename  name of store                                 required

flags:
    -all        export all shops
    -help       show the command line options
    -system     exports system data, no shop export
    -verbose    print debug infos
    -onlydata   export only database content
    -onlyfiles  export only public and private files
    -replace    replace exported shop folder

Example:
    perl $0 -storename Store -shop DemoShop -encoding iso-8859-1
    perl $0 -storename Store -system
END_USAGE
    exit 2;
}

RunScript(
    'Sub' => \&Main,
);

1;
