#========================================================================================
# package      DE_EPAGES::Zettle::API::Logger
# state        public
#----------------------------------------------------------------------------------------
# description  Wrapper of LogContext DE_EPAGES::Zettle::API::Logger
#========================================================================================
package DE_EPAGES::Zettle::API::Logger;

use strict;

use Data::Dumper qw ( Dumper );
use Scalar::Util qw (blessed);

use DE_EPAGES::Core::API::Log qw ( GetCommunicationLog );
use DE_EPAGES::Object::API::Factory qw (LoadRootObject);
use DE_EPAGES::Zettle::API::Constants qw( INTERNAL_CARTRIDGE_NAME );

# Method used to filter sensitive information out of the logs
my $Filter = sub {
    my $LogString = shift;
    # filter client_secret
    $$LogString =~s/(?<='client_secret' => ')[^']*?(?=')/---replaced---/g;
    $$LogString =~s/(?<=client_secret=)[-a-zA-Z0-9]*?(?=(&|$|'|[[:space:]]))/---replaced---/g;
    # filter refresh_token
    $$LogString =~s/(?<='refresh_token' => ')[^']*?(?=')/---replaced---/g;
    $$LogString =~s/(?<=refresh_token=)[-a-zA-Z0-9]*?(?=(&|$|[[:space:]]))/---replaced---/g;
    $$LogString =~s/(?<="refresh_token":")[^"]*?(?=")/---replaced---/g;
};

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       DE_EPAGES::Zettle::API::Logger->new();
#----------------------------------------------------------------------------------------
# description  constructor
#----------------------------------------------------------------------------------------
# return       $self | self | DE_EPAGES::Zettle::API::Logger
#========================================================================================
sub new {
    my $class = shift;
    my $self  = bless {}, $class;
    return $self;
}

#========================================================================================
# function     logCommunication
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->logCommunication( $Name, $Structure);
#----------------------------------------------------------------------------------------
# description  Dumps the data structure $Structure to the log file if logging is activated.
#               See <function DE_EPAGES::Core::API::Log::LogCommunication>.
#----------------------------------------------------------------------------------------
# input        $Name | name of data structure for identification in the log file | string
# input        $Structure | the object | scalar (usually hash or array ref)
#========================================================================================
sub logCommunication {
    my $self = shift;
    my ($Name, $Structure) = @_;

    my $Category = INTERNAL_CARTRIDGE_NAME;
    my $CallerDepth = 1;
    my $DoLogging = 0;
    my $MaxDepth;
    
    if($self->_isInitialExportCall()){
        if($self->_isInitialExportLoggingActivated()){
            $DoLogging = 1;
            # Do not limit dumper depth for initial export logging (otherwise outbound export request is not entirely logged)
            $MaxDepth = 0;
        }
    }
    elsif($self->_isLoggingActivated()){
        $DoLogging = 1;
    }

    $self->_LogCommunication($Name, $Structure, $Category, $Filter, $CallerDepth, $MaxDepth);

    return;
}


#========================================================================================


#========================================================================================
# function     _LogCommunication
# state        public
#----------------------------------------------------------------------------------------
# syntax       _LogCommunication( $Name, $rStructure, $Category );
# syntax       _LogCommunication( $Name, $rStructure, $Category, $aFilter, $CallerDepth, $MaxDepth);
# example      _LogCommunication( 'TransReturn', $TransReturn, 'WorldPay::PaymentCallback', ['callbackPW']);
#----------------------------------------------------------------------------------------
# description  Dumps the data structure $rStructure to the log file if the log level for
#               category $Category (or current package) is 'DEBUG'.
#               See <function DE_EPAGES::Core::API::Log::LogCommunication>.
#----------------------------------------------------------------------------------------
# input        $Name | name of data structure for identification in the log file | string
# input        $rStructure | the object | scalar (usually hash or array ref)
# input        $Category | debug category | string
# input        $aFilter | (optional) filter (perl regular expression for
#                       s/$Filter[^\n]+\n/\n/gis)
#                       OR
#                       (optional) a function pointer like
#                       sub {my $LogString = shift; $$LogString =~s/(?<=<xmlTag>)[^<]*?(?=<\/xmlTag>)/---replaced---/xgis;}
#                        | ref.array.string OR ref.code
# input        $CallerDepth | number of caller functions to remove from the stack trace | int
# input        $MaxDepth | (optional) maximum depth to use when dumping structure (default: 5) | int
#----------------------------------------------------------------------------------------
# return       nothing | nothing | nothing
#========================================================================================
sub _LogCommunication ($$$;$$) {
    my $self = shift;

    my ($Name, $Structure, $Category, $aFilter, $CallerDepth, $MaxDepth) = @_;
    $CallerDepth = defined $CallerDepth ? $CallerDepth + 1 : 1;
    local $Log::Log4perl::caller_depth = $Log::Log4perl::caller_depth + $CallerDepth;
    GetCommunicationLog($Category)->info(sub {
        local $Data::Dumper::Pad        = '';
        local $Data::Dumper::Varname    = $Name;
        local $Data::Dumper::Indent     = 1;
        local $Data::Dumper::Deepcopy   = 1;
        local $Data::Dumper::Sortkeys   = 1;
        local $Data::Dumper::Purity     = 0;
        local $Data::Dumper::Maxdepth   = $MaxDepth // 5;

        my $PrettyStructure = $self->_prettify($Structure);
        my $LogString = Dumper($PrettyStructure);
        if (defined $aFilter) {
            if( ref( $aFilter ) eq 'CODE' ){
                $aFilter->(\$LogString);
            }else{
                foreach my $Filter (@$aFilter) {
                    $LogString =~ s/$Filter.*?\n/\n/gis;
                }
            }
        }
        return $LogString;
    });
    return;
}

#========================================================================================
sub _isLoggingActivated {
    my $self = shift;
    return LoadRootObject()->get('Zettle_Logging');
}

#========================================================================================
sub _isInitialExportLoggingActivated {
    my $self = shift;
    return LoadRootObject()->get('Zettle_LoggingInitialExport');
}

#========================================================================================
sub _prettify {
    my $self = shift;
    my ($Structure) = @_;
    my $NameOfClass = blessed($Structure);
    return $Structure unless defined $NameOfClass;

    if($NameOfClass eq 'HTTP::Response') {
        return $self->_prettyResponse($Structure);
    }
    return $Structure;
}

#========================================================================================
sub _prettyResponse {
    my $self = shift;
    my ($Response) = @_;
    return sprintf ("\n Request: %s \n Response: %s \n", $Response->request()->as_string(), $Response->as_string());
}

#========================================================================================
# function     _isInitialExportCall
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->_isInitialExportCall()
#----------------------------------------------------------------------------------------
# description  Search a call to ZettleService::initialExport in stacktrace.
#----------------------------------------------------------------------------------------
# return        TRUE if initialExport() call is found in stacktrace, false otherwise
#========================================================================================
sub _isInitialExportCall {
    my $self = shift;

    my $Depth = 0;
    my ($package, $filename, $line, $subroutine) = caller($Depth);
    while(defined $subroutine){
        # Check for: subroutine=DE_EPAGES::Zettle::API::ZettleService::initialExport
        return 1 if $subroutine =~ /ZettleService::initialExport/;

        # Stop if we get to a stack entry outside of Zettle package
        return 0 if $package !~ /DE_EPAGES::Zettle/;

        $Depth += 1;
        ($package, $filename, $line, $subroutine) = caller($Depth);
    }
    return 0;
}

1;
