#========================================================================================
# package      DE_EPAGES::Zettle::API::WebService::AuthService
# base         DE_EPAGES::Zettle::API::WebService::RestService
# state        private
#----------------------------------------------------------------------------------------
# description  Service wrapper around Zettle OAuth API
#========================================================================================
package DE_EPAGES::Zettle::API::WebService::AuthService;
use base DE_EPAGES::Zettle::API::WebService::RestService;

use strict;

use DE_EPAGES::Core::API::TestParameter  qw( TestString );
use DE_EPAGES::Object::API::Factory qw( LoadRootObject );
use DE_EPAGES::Zettle::API::WebService::EndpointHandler qw ( GetDisconnectApiUrl GetTokenApiUrl );
use DE_EPAGES::Zettle::API::ZettleUrlBuilder;

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       DE_EPAGES::Zettle::API::WebService::AuthService->new($Environment);
#----------------------------------------------------------------------------------------
# description  creates a new AuthService object
#----------------------------------------------------------------------------------------
# input        $Environment | a Zettle environment object | DE_EPAGES::Zettle::API::Object:ZettleEnvironment
#----------------------------------------------------------------------------------------
# return       $self | self | DE_EPAGES::Zettle::API::WebService::AuthService
#========================================================================================
sub new {
    my $class = shift;
    my ($Environment) = @_;

    my $self = $class->SUPER::new($Environment);
    return $self;
}

#========================================================================================
# function     getTokensOnAuthCode
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->getTokensOnAuthCode($AuthorizationCode);
#----------------------------------------------------------------------------------------
# description  Exchange auth code for Access and Refresh tokens using Zettle OAuth flow.
#               See: https://github.com/iZettle/api-documentation/blob/master/authorization.md#request-an-access-token
#----------------------------------------------------------------------------------------
# input        $AuthorizationCode | Authorization code retrieved from Zettle OAuth endpoint | String
#----------------------------------------------------------------------------------------
# return       $hTokenInfo | with following keys:
#               <ul>
#                   <li>access_token - access token for API calls - string
#                   <li>refresh_token - refresh token for updating access token when expired - string
#                   <li>expires_in - number of seconds till the expiration of access token - string
#               </ul> | ref.hash
#========================================================================================
sub getTokensOnAuthCode {
    my $self = shift;
    my ($AuthorizationCode) = @_;
    
    TestString("AuthorizationCode", $AuthorizationCode);

    my $TokenRequestUrl = GetTokenApiUrl();
    my $hHeadersParams = {
        'Content-type' => 'application/x-www-form-urlencoded',
        'Accept' => 'application/json'
    };
    my $hContent = $self->_buildTokenRequestContent($AuthorizationCode);

    my $Authenticated = 0;
    my $HttpResponse = $self->sendPostRequest($TokenRequestUrl, $hHeadersParams, $hContent, $Authenticated);
    my $hTokenInfo = $self->decodeResponseContent($HttpResponse);

    return $hTokenInfo;
}

#========================================================================================
# function     refreshTokenData
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->refreshTokenData($RefreshToken);
#----------------------------------------------------------------------------------------
# description  Use given $RefreshToken to retrieve updated Token data from Zettle OAuth API.
#               See: https://github.com/iZettle/api-documentation/blob/master/authorization.md#refreshing-an-access-token
#----------------------------------------------------------------------------------------
# input        $RefreshToken | Refresh token previously retrieved from Zettle OAuth endpoint | String
#----------------------------------------------------------------------------------------
# return       $hTokenInfo | with following keys:
#               <ul>
#                   <li>access_token - access token for API calls - string
#                   <li>refresh_token - refresh token for updating access token when expired - string
#                   <li>expires_in - number of seconds till the expiration of access token - string
#               </ul> | ref.hash
#========================================================================================
sub refreshTokenData {
    my $self = shift;
    my ($RefreshToken) = @_;

    my $TokenRequestUrl = GetTokenApiUrl();
    my $hHeadersParams = {
        'Content-type' => 'application/x-www-form-urlencoded',
    };
    my $hContent = $self->_buildRefreshRequestContent($RefreshToken);

    my $Authenticated = 0;
    my $HttpResponse = $self->sendPostRequest($TokenRequestUrl, $hHeadersParams, $hContent, $Authenticated);
    my $hTokenInfo = $self->decodeResponseContent($HttpResponse);

    return $hTokenInfo;
}

#========================================================================================
# function     disconnect
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ResponseCode = $self->disconnect();
#----------------------------------------------------------------------------------------
# description  disconnect Zettle integration.
#               See: https://github.com/iZettle/api-documentation/blob/master/authorization.md#disconnect-application-from-organization
#----------------------------------------------------------------------------------------
# return       $ResponseCode | Http response code to DELETE request
#                               <ul>
#                               <li>204 - disconnection complete
#                               <li>401 - token expired or not valid
#                               <li>404 - token not found (already deleted)
#                               </ul> | integer
#========================================================================================
sub disconnect {
    my $self = shift;
        
    my $DisconnectUrl = GetDisconnectApiUrl();

    my $HttpResponse = $self->sendDeleteRequest($DisconnectUrl);
    
    return $HttpResponse->code();
}


#========================================================================================

sub _buildTokenRequestContent {
    my $self = shift;
    my ($AuthorizationCode) = @_;

    # Reference: https://github.com/iZettle/api-documentation/blob/master/authorization.md#retrieve-access-and-refresh-tokens
    my $ZettleUrlBuilder = DE_EPAGES::Zettle::API::ZettleUrlBuilder->new();
    my $RedirectUri = $ZettleUrlBuilder->buildBacklinkRedirectUrl();
    my $hCredentials = _getClientCredentials();
    
    my $hContent = {
        grant_type => 'authorization_code',
        redirect_uri => $RedirectUri,
        code => $AuthorizationCode,
        client_secret => $hCredentials->{ClientSecret},
        client_id => $hCredentials->{ClientId},
    };

    return $hContent;
}

sub _buildRefreshRequestContent {
    my $self = shift;
    my ($RefreshToken) = @_;

    # Reference: https://github.com/iZettle/api-documentation/blob/master/authorization.md#refreshing-an-access-token
    my $hCredentials = _getClientCredentials();
    
    my $hContent = {
        grant_type => 'refresh_token',
        client_secret => $hCredentials->{ClientSecret},
        client_id => $hCredentials->{ClientId},
        refresh_token => $RefreshToken,
    };

    return $hContent;
}

#========================================================================================
# function     _getClientCredentials
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Service->_getClientCredentials();
#----------------------------------------------------------------------------------------
# description  retrieve Zettle App client credentials from store config
#----------------------------------------------------------------------------------------
# return       $hCredentials | with following keys:
#               <ul>
#                   <li>ClientId - string
#                   <li>ClientSecret - string
#               </ul> | ref.hash
#========================================================================================
sub _getClientCredentials {  
    my $self = shift;  
    my $System = LoadRootObject();

    my $hCredentials = {
        ClientId => $System->get('Zettle_AppClientID'),
        ClientSecret => $System->get('Zettle_AppClientSecret'),
    };

    return $hCredentials;
}

1;
