#========================================================================================
# package      DE_EPAGES::Zettle::API::WebService::ProductService
# base         DE_EPAGES::Zettle::API::WebService::RestService
# state        private
#----------------------------------------------------------------------------------------
# description  Service wrapper around Zettle Product and Image API
#               This service is based on the methods provided by Product API: https://products.izettle.com/swagger#/
#========================================================================================
package DE_EPAGES::Zettle::API::WebService::ProductService;
use base DE_EPAGES::Zettle::API::WebService::RestService;

use strict;

use HTTP::Status qw( HTTP_NO_CONTENT HTTP_NOT_FOUND );
use JSON;

use DE_EPAGES::Core::API::Error qw ( Error ExistsError GetError );
use DE_EPAGES::Core::API::Log qw ( LogError );
use DE_EPAGES::Core::API::TestParameter qw( TestObject );
use DE_EPAGES::Zettle::API::WebService::DTO::DTOFactory;
use DE_EPAGES::Zettle::API::WebService::DTO::ProductResponseDTO;
use DE_EPAGES::Zettle::API::WebService::EndpointHandler qw (
    GetAllProductsApiUrl
    GetBulkExportApiUrl
    GetBulkExportStatusApiUrl
    GetDeleteProductApiUrl
    GetDeleteProductListApiUrl
    GetProductApiUrl
    GetProductCountApiUrl
    GetProductUpdateApiUrl
);

# Maximum number of items to delete in a single bulk request
# See DE_EPAGES/Zettle/Documents/API/ZettleAPI-Extracts.md, Section #6.2.7.5. Delete a list of products: limitations
use constant BATCH_SIZE_DELETE => 45;

#ERRORDEFINITION
sub ErrorDefinition { return {
        'InitialExportError' => 'Initial export failed: #errorMessage',
        'ExportPricesError' => 'An error occured during prices export: #errorMessage'
    }
}
#ERRORDEFINITION END

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       DE_EPAGES::Zettle::API::WebService::ProductService->new($Environment);
#----------------------------------------------------------------------------------------
# description  creates a new ProductService object
#----------------------------------------------------------------------------------------
# input        $Environment | any Zettle environment object | DE_EPAGES::Object::API::Object:Object
#----------------------------------------------------------------------------------------
# return       $self | self | DE_EPAGES::Zettle::API::WebService::ProductService
#========================================================================================
sub new {
    my $class = shift;
    my ($Environment) = @_;

    my $self = $class->SUPER::new($Environment);
    return $self;
}

#========================================================================================
# function     exportProducts
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hResponse = $self->exportProducts($aProducts, $IncludePrices, $hImageUploadResults);
#----------------------------------------------------------------------------------------
# description  Export all product information from the shop to Zettle.
#               This is an asynchronous process so export state shall be requested afterwards
#               See <function DE_EPAGES::Zettle::API::WebService::ProductService::getInitialExportState>.
#               API reference: https://products.izettle.com/swagger#/import/importLibraryV2
#----------------------------------------------------------------------------------------
# input        $aProducts | Products to be exported | ref.array.object
# input        $IncludePrices | Determines whether price information will be exported | boolean
#----------------------------------------------------------------------------------------
# return       $hResponse | with following keys:
#               <ul>
#                   <li>uuid - export identifier (UUIDv1) - string
#                   <li>state - current export state, any of "IMPORTING", "FINISHED_SUCCESS", "FINISHED_FAILED" - string
#                   <li>items - number of items to be processed - integer
#                   <li>created - datetime when export was received by Zettle (ISO8601, i.e: "2021-07-17T18:52:15.885+0000") - string
#                   <li>finished - datetime when export was completed in Zettle (ISO8601, i.e: "2021-07-17T18:52:15.927+0000") - string or null
#                   <li>errorMessage - additional message when errors occur - string
#               </ul> | ref.hash
#----------------------------------------------------------------------------------------
# throws WebserviceError
#========================================================================================
sub exportProducts {
    my $self = shift;
    my ($aProducts, $IncludePrices, $hImageUploadResults) = @_;

    # Compile shop product data and send it to Zettle
    my $BulkExportUrl = GetBulkExportApiUrl();
    my $hHeadersParams = {
        'Content-type' => 'application/json',
    };
    my $hContent = $self->_buildBulkExportContent($aProducts, $IncludePrices, $hImageUploadResults);

    my $HttpResponse = $self->sendPostRequest($BulkExportUrl, $hHeadersParams, $hContent);
    
    my $hResponse = $self->decodeResponseContent($HttpResponse);

    return $hResponse;
}

#========================================================================================
# function     updateProduct
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hResponse = $self->updateProduct($Product, $ETag, $IncludePrices, $hImageUploadResults);
#----------------------------------------------------------------------------------------
# description  Updates a product entity using JSON merge patch. This means that only included fields will be changed.
#               API reference: https://products.izettle.com/swagger#/products/updateProduct
#----------------------------------------------------------------------------------------
# input        $Product | Product to be updated | DE_EPAGES::Product::API::Object::Product
# input        $ETag | ETag value as retrieved from GetProduct HttpResponse headers | string
# input        $IncludePrices | Determines whether price information will be exported | boolean
#----------------------------------------------------------------------------------------
# return       $Updated | TRUE if product was updated, false if product was not found | boolean
#----------------------------------------------------------------------------------------
# throws WebserviceError
#========================================================================================
sub updateProduct {
    my $self = shift;
    my ($Product, $ETag, $IncludePrices, $hImageUploadResults) = @_;

    # Compile shop product data and send it to Zettle
    my $ProductUUID = $Product->get('ZettleUUID');
    my $UpdateUrl = GetProductUpdateApiUrl($ProductUUID);
    my $hHeadersParams = {
        'Content-type' => 'application/json',
        'If-Match' => $ETag,
    };

    eval {
        my $hContent = $self->_buildUpdateContent($Product, $IncludePrices, $hImageUploadResults);
        $self->sendPutRequest($UpdateUrl, $hHeadersParams, $hContent);
    };
    if(ExistsError()) {
        my $Error = GetError();
        if($Error->vars()->{'Status'} == HTTP_NOT_FOUND){
            return 0;
        }
        Error($Error); #forward all other Errors
    }

    return 1;
}

#========================================================================================
# function     exportPrices
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hResponse = $self->exportPrices($aProducts);
#----------------------------------------------------------------------------------------
# description  Export prices for the given products from the shop to Zettle.
#               This is an asynchronous process so export state shall be requested afterwards
#               See <function DE_EPAGES::Zettle::API::WebService::ProductService::getInitialExportState>.
#               API reference: https://products.izettle.com/swagger#/import/importLibraryV2
#----------------------------------------------------------------------------------------
# input        $aProducts | List of products | ref.array.object
#----------------------------------------------------------------------------------------
# return       $hResponse | with following keys:
#               <ul>
#                   <li>uuid - export identifier (UUIDv1) - string
#                   <li>state - current export state, any of "IMPORTING", "FINISHED_SUCCESS", "FINISHED_FAILED" - string
#                   <li>items - number of items to be processed - integer
#                   <li>created - datetime when export was received by Zettle (ISO8601, i.e: "2021-07-17T18:52:15.885+0000") - string
#                   <li>finished - datetime when export was completed in Zettle (ISO8601, i.e: "2021-07-17T18:52:15.927+0000") - string or null
#                   <li>errorMessage - additional message when errors occur - string
#               </ul> | ref.hash
#----------------------------------------------------------------------------------------
# throws WebserviceError
#========================================================================================
sub exportPrices {
    my $self = shift;
    my ($aProducts) = @_;

    # Retrieve product prices and send them to Zettle
    my $BulkExportUrl = GetBulkExportApiUrl();
    my $hHeadersParams = {
        'Content-type' => 'application/json',
    };
    my $hContent = $self->_buildBulkExportPricesContent($aProducts);

    my $HttpResponse = $self->sendPostRequest($BulkExportUrl, $hHeadersParams, $hContent);
    
    my $hResponse = $self->decodeResponseContent($HttpResponse);
    
    Error('ExportPricesError', $hResponse) if $hResponse->{state} eq 'FINISHED_FAILED';

    return $hResponse;
}

#========================================================================================
# function     getInitialExportState
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->getInitialExportState();
#----------------------------------------------------------------------------------------
# description  Retrieve updated status from Zettle for bulk product export.
#----------------------------------------------------------------------------------------
# input        $ExportUUID | optional. Export UUID returned by Zettle | string
#----------------------------------------------------------------------------------------
# return       $State | state field from Zettle response: any of "IMPORTING", "FINISHED_SUCCESS", "FINISHED_FAILED" | string
#----------------------------------------------------------------------------------------
# throws       InitialExportError
#========================================================================================
sub getInitialExportState {
    my $self = shift;
    my ($ExportUUID) = @_;

    # If $State is not present, query Zettle for an updated state
    my $ExportStatusURL = GetBulkExportStatusApiUrl($ExportUUID);
    
    my $hHeadersParams = {
        'Content-type' => 'application/json',
    };

    my $HttpResponse = $self->sendGetRequest($ExportStatusURL, $hHeadersParams);

    # {"uuid":"1aca6d65-e730-11eb-9cc8-b68971ccb386", "state":"FINISHED_SUCCESS", "items":10, "created":"2021-07-17T18:52:15.885+0000", "finished":"2021-07-17T18:52:15.927+0000", "errorMessage": null}
    my $hResponse = $self->decodeResponseContent($HttpResponse);

    my $State = $hResponse->{state};
    Error('InitialExportError', $hResponse) if $State eq 'FINISHED_FAILED';

    return $State;
}

#========================================================================================
# function     getProductCount
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->getProductCount();
#----------------------------------------------------------------------------------------
# description  Retrieve all of the products from Zettle.
#               See: https://products.izettle.com/swagger#/products/countAllProducts
#----------------------------------------------------------------------------------------
# return       $hResponse | Product count in a hash with the key 'productCount' | ref.hash
#========================================================================================
sub getProductCount {
    my $self = shift;

    my $Url = GetProductCountApiUrl();

    my $HttpResponse = $self->sendGetRequest($Url);
    
    my $hResponse = $self->decodeResponseContent($HttpResponse);

    return $hResponse;
}

#========================================================================================
# function     getProduct
# state        public
#----------------------------------------------------------------------------------------
# syntax       $HttpResponse = $self->getProduct($ProductUUID);
#----------------------------------------------------------------------------------------
# description  Retrieve the specifed product from Zettle.
#               Raw HttpResponse is returned since 'ETag' header is required.
#               See: https://products.izettle.com/swagger#/products/getProduct
#----------------------------------------------------------------------------------------
# return       $HttpResponse | An HttpResponse object | HttpResponse
#========================================================================================
sub getProduct {
    my $self = shift;
    my ($ProductUUID) = @_;

    my $GetProductUrl = GetProductApiUrl($ProductUUID);

    my $HttpResponse = $self->sendGetRequest($GetProductUrl);
    
    # my $hResponse = $self->decodeResponseContent($HttpResponse);

    return $HttpResponse;
}

#========================================================================================
# function     getAllProducts
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aProductResponseDTO = $self->getAllProducts();
#----------------------------------------------------------------------------------------
# description  Retrieve all of the products from Zettle.
#               See: https://products.izettle.com/swagger#/products/getAllProductsInPos
#               See: https://products.izettle.com/swagger#/products/getAllProductsV2
#----------------------------------------------------------------------------------------
# return       $aProductResponseDTO | Array of ProductResponseDTO (see DE_EPAGES::Zettle::API::WebService::DTO::ProductResponseDTO) | ref.array.object
#========================================================================================
sub getAllProducts {
    my $self = shift;

    my $Url = GetAllProductsApiUrl();

    #TODO: include 'sort' parameter in request if passed as an argument in method call
    my $HttpResponse = $self->sendGetRequest($Url);
    # print "HttpResponse=".Data::Dumper::Dumper($HttpResponse)."\n";
    
    #TODO: check why decodeResponseContent() returns undef when processing a considerable amount of products (more than 50 products)
    # my $hResponse = $self->decodeResponseContent($HttpResponse);    
    # print "hResponse=".Data::Dumper::Dumper($hResponse)."\n";
    
    my $DecodedContent = $HttpResponse->decoded_content();
    # print "DecodedResponse=".Data::Dumper::Dumper($DecodedResponse)."\n";
    my $hResponse = decode_json($DecodedContent);

    my $aProductResponseDTO;
    foreach my $ProductResponse (@{$hResponse}){
        my $ProductResponseDTO = DE_EPAGES::Zettle::API::WebService::DTO::ProductResponseDTO->parseResponse($ProductResponse);
        push(@{$aProductResponseDTO}, $ProductResponseDTO);
    }

    return $aProductResponseDTO;
}

#========================================================================================
# function     deleteProduct
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Deleted = $self->deleteProduct($ProductUUID);
#----------------------------------------------------------------------------------------
# description  Delete a single product from Zettle.
#               See: https://products.izettle.com/swagger#/products/deleteProduct
#----------------------------------------------------------------------------------------
# input        $ProductUUID | Product UUID | string
#----------------------------------------------------------------------------------------
# return       $Deleted | TRUE if product was deleted, false if product was not found | boolean
#========================================================================================
sub deleteProduct {
    my $self = shift;
    my ($ProductUUID) = @_;

    my $Url = GetDeleteProductApiUrl($ProductUUID);

    eval {
        $self->sendDeleteRequest($Url);
    };
    if(ExistsError()) {
        my $Error = GetError();
        if($Error->vars()->{'Status'} == HTTP_NOT_FOUND){
            return 0;
        }
        Error($Error); #forward all other Errors
    }

    return 1;
}

#========================================================================================
# function     deleteProducts
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->deleteProducts($aProductUUIDs);
#----------------------------------------------------------------------------------------
# description  Delete a list of products from Zettle.
#               See: https://products.izettle.com/swagger#/products/deleteProducts
#----------------------------------------------------------------------------------------
# input        $aProductUUIDs | List of product UUIDs to delete | ref.array.string
#----------------------------------------------------------------------------------------
# return       $Deleted | TRUE if products were deleted | boolean
#========================================================================================
sub deleteProducts {
    my $self = shift;
    my ($aProductUUIDs) = @_;

    my $Url = GetDeleteProductListApiUrl();

    my $Deleted = 1;
    my $ProductCount = scalar @{$aProductUUIDs};
    # Only 45 products may be safely deleted at once (in one request). Group products in batches and send as many requests as necessary.
    for (my $StartIndex=0; $StartIndex<$ProductCount; $StartIndex+=BATCH_SIZE_DELETE){
        my $EndIndex = ($StartIndex+BATCH_SIZE_DELETE >= $ProductCount) ? $ProductCount-1 : $StartIndex+BATCH_SIZE_DELETE-1;
        my @aBatchProductUuid = @{$aProductUUIDs}[$StartIndex..$EndIndex];

        my $hContent = {
            'uuid' => \@aBatchProductUuid
        };
        # Zettle wrongfully defined delete products API request with HTTP Parameter Pollution (HPP): multiple parameters with the same key (uuid) and different values.
        # In order to construct query parameters we use https://metacpan.org/pod/URI#$uri-%3Equery_form(-\%hash-) passing a reference to an array of strings.
        my $hOptions = {
            'content' => $hContent,
        };

        my $HttpResponse = $self->sendDeleteRequest($Url, $hOptions);
        $Deleted = 0 unless ($HttpResponse->code == HTTP_NO_CONTENT);
    }

    return $Deleted;
}


#========================================================================================


#========================================================================================
# function     _buildBulkExportContent
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hContent = $self->_buildBulkExportContent($aProducts, $ExportPrices, $hImageUploadResults);
#----------------------------------------------------------------------------------------
# description  Processes given products and generates the contents expected by Zettle Product API.
#               It will include any previously exported image available in $hImageUploadResults.
#----------------------------------------------------------------------------------------
# input        $aProducts | a list of products | ref.array.object
# input        $ExportPrices | TRUE if prices shall be included, FALSE otherwise | boolean
# input        $hImageUploadResults | a map of ImageUploadResult objects indexed by product GUID
#                                      Image lookupKeys for products to be exported will be searched here.
#                                      See <DE_EPAGES::Zettle::API::Object::ImageUploadResult> | ref.hash.array.object
#----------------------------------------------------------------------------------------
# return       $hContent | export content as expected by Product API | ref.hash
#========================================================================================
sub _buildBulkExportContent {
    my $self = shift;
    my ($aProducts, $ExportPrices, $hImageUploadResults) = @_;

    # Copy original array to avoid side effects on caller method
    my $aProductsCopy = [ @{$aProducts} ];

    my $Environment = $self->getEnvironment();
    my $Shop = $Environment->getSite();
    my $Factory = DE_EPAGES::Zettle::API::WebService::DTO::DTOFactory->new($Shop);
    
    # Map of ProductDTOs indexed by product UUID used to merge VariationDTO data for subproducts with the same parent product
    my $hProducts = {};
    my @ZettleProducts = ();
    my $ProductDTO;
    foreach my $Product ( @{$aProductsCopy} ) {
        eval{
            $ProductDTO = $Factory->buildProductDTO($Product, $ExportPrices, $hImageUploadResults);
            my $Uuid = $ProductDTO->uuid();
            if(exists $hProducts->{$Uuid}){
                my $ExistingProductDTO = $hProducts->{$Uuid};
                push(@{$ExistingProductDTO->variations}, @{$ProductDTO->variations});
            }else{
                $hProducts->{$Uuid} = $ProductDTO;
            }
        };
        if(ExistsError()){
            my $Error = GetError();
            LogError($Error);
        }
    }

    #PATCH: bulk export would remove variations not present in update hence we include all variations
    foreach my $Product ( @{$aProductsCopy} ) {
        $Product = $Product->superProduct if defined $Product->superProduct;
        my $ProductUuid = $Product->get('ZettleUUID');
        next unless defined $ProductUuid;
        my $aSubProducts = $Product->get('SubProducts');
        if(scalar @{$aSubProducts} > 0){
            foreach my $SubProduct (@$aSubProducts) {
                eval{
                    $ProductDTO = $hProducts->{$ProductUuid};
                    next unless defined $ProductDTO;
                    TestObject('ProductDTO', $ProductDTO, 'DE_EPAGES::Zettle::API::WebService::DTO::ProductDTO');
                    my $aProductVariationDTOs = $ProductDTO->variations();
                    unless(grep { $_->uuid eq $SubProduct->get('ZettleUUID')} @{$aProductVariationDTOs}){
                        push(@{$aProductVariationDTOs}, $Factory->buildPatchVariationDTO($SubProduct));
                    }
                };
                if(ExistsError()){
                    my $Error = GetError();
                    LogError($Error);
                }
            }
        }
    }

    foreach my $UuidKey (keys %{$hProducts}){
        $ProductDTO = $hProducts->{$UuidKey};
        push(@ZettleProducts, $ProductDTO->toHash());
    }

    return { 'products' => \@ZettleProducts };
}
#========================================================================================
# function     _buildUpdateContent
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hContent = $self->_buildUpdateContent($Product, $ExportPrices, $hImageUploadResults);
#----------------------------------------------------------------------------------------
# description  Process given product and generate the contents expected by Zettle Product API.
#               It will include any previously exported image available in $hImageUploadResults.
#----------------------------------------------------------------------------------------
# input        $Product | a product object | DE_EPAGES::Product::API::Object::Product
# input        $ExportPrices | TRUE if prices shall be included, FALSE otherwise | boolean
# input        $hImageUploadResults | a map of ImageUploadResult objects indexed by product GUID
#                                      Image lookupKeys for products to be exported will be searched here.
#                                      See <DE_EPAGES::Zettle::API::Object::ImageUploadResult> | ref.hash.array.object
#----------------------------------------------------------------------------------------
# return       $hContent | export content as expected by Product API | ref.hash
#========================================================================================
sub _buildUpdateContent {
    my $self = shift;
    my ($Product, $ExportPrices, $hImageUploadResults) = @_;
    
    my $ProductDTO;
    eval{
        my $Environment = $self->getEnvironment();
        my $Shop = $Environment->getSite();
        my $Factory = DE_EPAGES::Zettle::API::WebService::DTO::DTOFactory->new($Shop);
        $ProductDTO = $Factory->buildProductDTO($Product, $ExportPrices, $hImageUploadResults);
    };
    if(ExistsError()){
        my $Error = GetError();
        LogError($Error);
    }

    return $ProductDTO->toHash();
}

#========================================================================================
# function     _buildBulkExportPricesContent
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hContent = $self->_buildBulkExportPricesContent($aProducts);
#----------------------------------------------------------------------------------------
# description  Process given products/subproducts and generate the contents expected by
#               Zettle Product API in order to only update prices.
#----------------------------------------------------------------------------------------
# input        $aProducts | a list of products/subproducts | ref.array.object
#----------------------------------------------------------------------------------------
# return       $hContent | export content as expected by Product API | ref.hash
#========================================================================================
sub _buildBulkExportPricesContent {
    my $self = shift;
    my ($aProducts) = @_;

    my $Environment = $self->getEnvironment();
    my $Shop = $Environment->getSite();
    my $Factory = DE_EPAGES::Zettle::API::WebService::DTO::DTOFactory->new($Shop);
    
    # Map of ProductDTOs indexed by product UUID used to merge VariationDTO data for subproducts with the same parent product
    my $hProducts = {};
    my @ZettleProducts = ();
    my $ProductPriceDTO;
    foreach my $Product ( @{$aProducts} ) {
        eval{
            $ProductPriceDTO = $Factory->buildProductPriceDTO($Product);
            my $Uuid = $ProductPriceDTO->uuid();
            if(exists $hProducts->{$Uuid}){
                my $ExistingProductPriceDTO = $hProducts->{$Uuid};
                push(@{$ExistingProductPriceDTO->variations}, @{$ProductPriceDTO->variations});
            }else{
                $hProducts->{$Uuid} = $ProductPriceDTO;
            }
        };
        if(ExistsError()){
            my $Error = GetError();
            LogError($Error);
        }
    }

    #PATCH: bulk export will remove variations not present in update hence we include all variations
    foreach my $Product ( @{$aProducts} ) {
        $Product = $Product->superProduct if defined $Product->superProduct;
        my $ProductUuid = $Product->get('ZettleUUID');
        next unless defined $ProductUuid;
        my $aSubProducts = $Product->get('SubProducts');
        if(scalar @{$aSubProducts} > 0){
            foreach my $SubProduct (@$aSubProducts) {
                eval{
                    $ProductPriceDTO = $hProducts->{$ProductUuid};
                    next unless defined $ProductPriceDTO;
                    TestObject('ProductDTO', $ProductPriceDTO, 'DE_EPAGES::Zettle::API::WebService::DTO::ProductDTO');
                    my $aProductVariations = $ProductPriceDTO->variations();
                    unless(grep { $_->uuid eq $SubProduct->get('ZettleUUID')} @{$aProductVariations}){
                        push(@{$aProductVariations}, $Factory->buildPatchVariationDTO($SubProduct));
                    }
                };
                if(ExistsError()){
                    my $Error = GetError();
                    LogError($Error);
                }

            }
        }
    }

    foreach my $Key (keys %{$hProducts}){
        $ProductPriceDTO = $hProducts->{$Key};
        push(@ZettleProducts, $ProductPriceDTO->toHash());
    }

    return { 'products' => \@ZettleProducts };
}

1;
